/******************************************************************************
 *   sissi.c                                                                  *
 *                                                                            *
 *   SISSI0.93 (Prerelease Version)                                           *
 *   SImulating Site-Specific Interactions along phylogenetic trees           *
 *                                                                            *
 *   Copyright (C) 2004-2005 by Tanja Gesell, Arndt von Haeseler              *
 *   sissi@cs.uni-duesseldorf.de                                              *
 *                                                                            *
 *   All parts of the source code except where indicated are distributed      *
 *	 under the GNU public licence.                                            *
 *	 See http://www.opensource.org for details.                               *
 *                                                                            *
 *   This program is distributed in the hope that it will be useful,          *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 *   GNU General Public License for more details.                             *
 *                                                                            *
 *   Any feedback is very welcome!              sissi@cs.uni-duesseldorf.de   *
 *****************************************************************************/
 /*****************************************************************************
 *  The files tree.h, treefile.h and treefile.c are completely from           *                                                              *
 *  Sequence Generator - seq-gen, version 1.3:                                *
 *  Corresponding some indicated functions in this file are modified          *
 *  from seq-gen:                                                             *
 *	Note the Copyright of seq-gen in tree.h, treefile.h and treefile.c .      *                                                                  *
 *****************************************************************************/

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "dkiss.h"
#include "global.h"
#include "treefile.h"
#include "models.h"
#include "neighfile.h"
#include "evolve.h"
#include "mutate.h"
#if 0
#include <sys/stat.h>
#endif

#if 0
  #define SPRNG 0
  #if SPRNG == 1
#  include "sprng/sprng.h"
#endif  /* SPRNG */
#endif

#define NEARZERO 1.0e-10

#define MIX 0 
#define DISTANCE 0         
#define VERSION "0.99"
/*Distance one sequence into another */
/* if MIX 1, change main if(pvalue==1) in OpenNeighFile*/

  /***************************************/
  /*      GLOBALE VARIABLEN              */
  /***************************************/

  /**************************************/
   int    nDatasets;              /* number of datsets*/
   int    treeFile, ctFile, sequenceFile, triFile;
   int    numTrees;            /*  number of trees, compute in the function OpenTreeFile()*/
   int    sitefactorFile;
  /**************************************/																																																														 /***************************************/
   char   treeFileName[256];
   char   neighborFileName[256];
   char   triFileName[256];
   char   sequenceFileName[256];
   char   ctFileName[256];
   char   sitefactorFileName[256];
   char   sissi_sitefactorFileName[256];
   char   sissi_sitefactorFileName_norm[256];
  /***************************************/
   int    *partitionLengths;   /*for partitionfunction*/
   double *partitionRates;
   /***************************************/
   FILE  *neighbor_fv;       
   FILE  *ct_fv;
   FILE  *sequence_fv;
   FILE  *tri_fv;
   FILE  *tree_fv;
   FILE  *sitefactor_fv;
   FILE  *sissi_sitefactor_fv;
   FILE  *sissi_sitefactor_norm_fv;
   /***************************************/
   int    scaleTrees, scaleBranches, ancestorSeq, writeAncestors, writeRates;
   
   double treeScale, branchScale;
   int    hasAlignment;
   char   **names;
   char   **sequences;
   int    maxPartitions; /*option von seq-gen: later useful for possible increments of sissi*/
   int    numDatasets, numTrees;  /*numTrees:  later useful for possible increments of sissi*/
   /***************************************/
   #if MIX     
      int    pvalue;  
   #endif    
   #if DISTANCE 
       double distance;           /*  time , distance d from the tree*/	
   #endif   
  /***************************************/
  /*            PROTOTYPEN               */
  /***************************************/
   static void PrintTitel();
   static void PrintUsage();
   void        PrintVerbose(FILE *fv);
   void        ReadParams(int argc, char **argv); 
  /***************************************/
   static int  OpenTreeFile();
   static int  OpenNeighFile();   
   double      *OpenTriFile();
   double      *OpenSiteFactorFile();
  /***************************************/
   /***************************************/
   double      timeoverall(double rsum);
   /***************************************/
      void     AllocateMemory();
  /***************************************/

     
  /***************************************/
  /*             FUNCTIONS                */
  /***************************************/
   static void PrintTitel(){
     fprintf(stderr, "\nPRE-RELEASE SISSI-%s:\n", VERSION);
	 fprintf(stderr, "Simulating Sequence Evolution with Site Specifc Interactions\n");
	 fprintf(stderr, "(c) Copyright, 2005 Tanja Gesell and Arndt von Haeseler\n");
	 fprintf(stderr, "Heinrich-Heine Universitaet Duesseldorf, Germany\n");
	 fprintf(stderr, "sissi@cs.uni-duesseldorf.de\n");
	 fprintf(stderr, "\nSome parts of source code are from ");
	 fprintf(stderr, "seq-gen (Rambaut and Grassly, 1997)\nand SPRNG (Mascagni and Srinivasan, 2000):\n");
	 fprintf(stderr, "For further information see README and COPYING files.\n");
   }	 
   
   static void PrintUsage()
   {                  /*|<-                         80 characters                                     ->| */
	fprintf(stderr, "\n");
	fprintf(stderr, "sissi%s [OPTIONS][-nFORMAT NEIGHBOURHOODSYSTEMFILE][-fSTATE FREQUENCIES][treefile]\n", VERSION);
	fprintf(stderr, "POSSIBLE OPTIONS:\n");
	/*               |<-                                                        ->| */
	fprintf(stderr, " Miscellaneous options:\n");
	fprintf(stderr, "   -l # = Sequence length              [default = 100].\n");
	fprintf(stderr, "   -a # = Simulated datasets per tree  [default = 1].\n");
	fprintf(stderr, "   -d   = Generate DNA sequences       [default = RNA].\n");
	fprintf(stderr, "   -k   = Use an ancestor-sequence-file[default = random].\n");
	fprintf(stderr, "   -h   = Print this help message\n");
	fprintf(stderr, "   -z # = Seed for random number generator [default = system generated].\n");
	fprintf(stderr, "   -y   = SPRNG random generator [default = dkiss random generator].\n"); /*SPRNG will be the final random generator in the public version*/
	fprintf(stderr, "   -i   = Optimise running time for q(x) (see info at the end)\n");
	fprintf(stderr, "   -t # = Transition-Transversion, TS/TV equal 0.5 is JC/Fel [default=0.5]\n");
	fprintf(stderr, "   -q   = Quiet\n");
	/*               |<-                                                        ->| */
	fprintf(stderr, " Neighbourhood System:\n");
	fprintf(stderr, "  nFORMAT neighbourhoodsystem file [default = no correlations]: \n");
	/*                      |<-                                    ->| */
    /*fprintf(stderr, "     -np  = neighbourhoodsystem file \n");*/
	fprintf(stderr, "       -nn  = sissi0.1 format (see bsubtilis401.nei)\n");
    fprintf(stderr, "       -nr  = ctFile as neighbourhoodsystem (see /B.subtilis.ct2) \n");
	fprintf(stderr, "       -nc  = sissi0.2 format (see codon.cnei)\n");
	fprintf(stderr, "       You must generate a neighbourhoodsystem for your sepecial case.\n");
	fprintf(stderr, "       Please, do not hesitate to ask.\n");
	                 /*|<-                         80 characters                                     ->| */
	fprintf(stderr, " Nucleotid Model: \n");
	fprintf(stderr, "  F81 model -f option\n");
	fprintf(stderr, "  JC 69: Note, neighbourhood system has no effect [default]\n");
	fprintf(stderr, "  K80 -t option\n");
	fprintf(stderr, "  HKY use -t option and -f option\n");
	fprintf(stderr, "  Tamura use -rr option and -f option\n");
    fprintf(stderr, "  GTR use -rr and -f option\n");
	fprintf(stderr, "  More complex combinations are possible.\n");
	/*                |<-                                                        ->| */
	fprintf(stderr, " eSTATE Rateheterogeneity [default = all equal]: \n");
	fprintf(stderr, "	-ee   = file site-specific scaling factor (ssf) (#1...#l = seqlen)\n");	
    fprintf(stderr, "	-en   = file normalise ssf (#1...#l = seqlen)\n");	
	fprintf(stderr, "	-ed   = file ssf in the format (#pos #ssf)\n");	
    fprintf(stderr, "	-em   = file normalise ssf (#pos #ssf)\n");	
	/*                |<-                                                        ->| */
    fprintf(stderr, " rGTR [default = all equal]: \n");
	fprintf(stderr, "	-rr # 5  = GTR abcdef: Not possible with t option!\n");	
    fprintf(stderr, "	-rd # 24 = 24 rates extended GTR nk=1\n") ;	
	fprintf(stderr, "	-rt # 95 = 95 rates extended GTR nk=2 \n");
    /*fprintf(stderr, "-ri # 11  = GTR abcdefABCEDF: Not possible with t option!\n");	
    fprintf(stderr, "  -r #     = 24 rates GTR nk=1\n") ;	
	fprintf(stderr, "  -rt #     = 95 rates for GTR with overlappings\n");*/
	fprintf(stderr, "	-ro  ovelapping with neighbourhood system\n");
	fprintf(stderr, "	-rc # 2  = CpG: #1 CpG->TpG; #2 CpG -> CpA\n");
	fprintf(stderr, "	-ra      = Take mononucleotide frequencies for nk=1\n") ;	
	fprintf(stderr, "	-rb      = Take mononucleotide frequencies for nk=2\n");		
  
    fprintf(stderr, " fSTATE frequencies [default = all equal]: \n");
	/*                      |<-                                    ->| */
	fprintf(stderr, "	-fs #A #C #G #U =  4 'single' nucleotide frequencies  (nk=0)\n");
	fprintf(stderr, "	-fd #AA ... #UU = 16 'doublet' nucleotide frequencies (nk=1)\n");
	/*fprintf(stderr, "        -fi ='doublet' frequencies dependent of the marginal distribution\n");*/
	fprintf(stderr, "	-ft file: (#AAA...#UUU = 64 'triplet' frequencies) (nk=2)\n");
	
	
	/*fprintf(stderr, " Miscellaneous options:\n");*/
	fprintf(stderr, "  wOPTION Write additional information [default=none]: \n");
	fprintf(stderr, "     -wa = Write ancestral sequences for each node\n");
	fprintf(stderr, "     -wpfilename = Print in FILES\n");
#if HAMMINGDISTANCE
     fprintf(stderr, "    -wh = Write Hamming distance after each node\n");
	 fprintf(stderr, "    -wo = Print note for simulating branch length with no subst.\n");
	 fprintf(stderr, "    -wt = Print special treefiles with additional information\n");
	 fprintf(stderr, "        <intern.htree>          with internal nodes\n");
     fprintf(stderr, "        <internlength.htree>    with internal nodes and branch length\n");
	 fprintf(stderr, "        <hammingdistance.htree> with internal nodes and Hamming distance\n");
	 fprintf(stderr, "    -wb = Binary Alphabet in FILES (with wp option): \n");
     fprintf(stderr, "        0: Purine     R : A || G \n");
	 fprintf(stderr, "        1: Pyrimidine Y : C || U || T \n");
#endif
	 fprintf(stderr, " sOPTION scaling branch length or tree [default=none]: \n");
     fprintf(stderr, "    -sb = Branch length scaling factor [default = 1.0].\n");
	 fprintf(stderr, "    -st = Total tree scale [default = use branch lengths].\n");		
	 fprintf(stderr, "          !don't use it with zero branch length in the tree\n");	
	 
	 fprintf(stderr, " oOPTION Output File format [default=Phylip]: \n");
	 fprintf(stderr, "    -of = Fasta output\n");
	 fprintf(stderr, "    -or = relaxedPhylip output\n");
	 fprintf(stderr, "    -oc = Clustal output \n");

	/* fprintf(stderr, " Verbose:\n");*/
	 fprintf(stderr, "  vOPTION Verbose [default=none]: \n");
	 fprintf(stderr, "    -vf = Print frequencies\n");
	 fprintf(stderr, "    -vq = Print rate matrices\n");
     fprintf(stderr, "    -vn = Print neighborhood in sissi0.1 format\n");
	/* fprintf(stderr, "        -vw = Print q(x), total instantaneous substitution rate\n");*/
     fprintf(stderr, "    -vt = Print number of substitutions and Hamming distance\n");
	 fprintf(stderr, "    -va = Print note for no substitutions\n");
	 fprintf(stderr, "    -vr = Print expected number of substitutions per site\n");
	 fprintf(stderr, "    -vz = Print random generator seed\n");

	 fprintf(stderr, "TREEFILE: Name of the treefile [default=generates no sequences!]\n\n");	
	 fprintf(stderr, "Sequence length (-l) must agree with the length of the neighbourhood system!\n");
	 fprintf(stderr, "Some faster versions are available on request for sepecial Ns.\n");
			         /*|<-                         80 characters                                     ->| */
   }

 /*********************************************************/
 /*             READ PARAMETERS/OPTIONS                   */
 /*********************************************************/

   void ReadParams(int argc, char **argv)
   {
     int i, j;
     char ch,*P, dummy;
	 double CpGarray[2];
	 
	 printfile= 0;

     mpi[0]=0.25;
     mpi[1]=0.25;
     mpi[2]=0.25;
     mpi[3]=0.25;


     dipi[0] =0.0625;   /*aa*/
     dipi[1] =0.0625;   /*ac*/
     dipi[2] =0.0625;   /*ag*/
     dipi[3] =0.0625;   /*at*/
     dipi[4] =0.0625;   /*ca*/
     dipi[5] =0.0625;   /*cc*/
     dipi[6] =0.0625;   /*cg*/
     dipi[7] =0.0625;   /*ct*/
     dipi[8] =0.0625;   /*ga*/
     dipi[9] =0.0625;   /*gc*/
     dipi[10]=0.0625;   /*gg*/
     dipi[11]=0.0625;   /*gt*/
     dipi[12]=0.0625;   /*ta,ua*/
     dipi[13]=0.0625;   /*tc,uc*/
     dipi[14]=0.0625;   /*tg,ug*/
     dipi[15]=0.0625;   /*tt,uu*/

     for(i=0;i<64;i++) tripi[i]=0.015625;
     triFile=0;

     numseq=1;
     nDatasets=1;
     Rmat[0]=Rmat[1]=Rmat[2]=Rmat[3]=Rmat[4]=Rmat[5]=1.0 ;
	 CpGTpG=CpGCpA=1.0;
     nucleotide[0]='A',nucleotide[1]='C',nucleotide[2]='G',nucleotide[3]='U';
     overlappings=0;
	 CpG=0;
	 CpGarray[0]=1.00;
	 CpGarray[1]=1.00;


     seqlen=100 ;
     treeFile=0;
#if distance	 
     distance=0.0;
#endif	 
     /*correlationvalue=1;*/
     /*Cdipi=0;*/
     /*nonorm=0;*/
     /*nodiv=0;*/
      matrixSH=0;
     /*gvariable=0;*/
	  sitefactor=0;
	  sitefactornorm=0;
	  sitefactor2=0;
	  sitefactornorm2=0;
	  	
	  
	  minimumvalue=0.5;
	  ctcountfreq=0;
	  scaleTrees = 0;
	  treeScale = 0;
	  scaleBranches = 0;
	  branchScale = 0;
	  hasAlignment = 0;
	  ancestorSeq = 0;
	  writeAncestors = 0;
	  writeRates = 0;
	  maxPartitions=1;    
#if MIX	  
	  pvalue=0;
#endif	  
	  catnei=0;
	  neighborFile=0;
		
	  verbose=1;
	  verboseQMatrix=0;
	  verboseNeighborNei=0;
	  verboseNeighborMatrix=0;
	  verboseNeighborCt=0;
	  verboseEvolutiontime=0;
	  verboseRate=0;
	  verboseFrequencies=0;
	  verboserandom=0;
	  verbosewaiting=0;
	  verboseWarning=0;
	  verboseStep=0;
	
	  relaxedPhylip=0;
	  FastaAln=0;
	  Clustal=0;
	  quiet=0;
	  mix=0;
	  fastrates=0;
	  
	  binalphabet=0;
	  verboseancestor=0;
	  
	  prmat=0;
	  prmat2t=0;
	  prmat16=0;
	  transitiontransverion=0;
	  trtv=0.5;
	  
	  numBases=-1;
	  numDatasets=1; 
	  
	  mononucneigh1=0;
	  mononucneigh2=0;
	  
	  #if HAMMINGDISTANCE
      writeHammingdistance=0;
	  writeHammingdistanceTree=0;
	  writenote=0;
      #endif	

	  userSeed=0; /*random seed*/
	  verboseRandomSeed=0;
	  randomgeneratordkiss=1;
	
     for(i=1; i<argc; i++){
         P=argv[i];
         if (*P!='-'){
            if (treeFile){
                 fprintf(stderr, "Illegal command parameter: %s\n\n" ,argv[i]);
				 PrintUsage();
                 exit(0);
            }
            treeFile=1;
            strcpy(treeFileName, argv[i]) ;
			
		 }else {
		    
			   P=argv[i];
               P++;
               ch=*P;
               P++;
               switch (ch) {
			         case 'H': case 'h':
					    PrintTitel();
						PrintUsage();
						exit(0);
					 break;	
					 case 'N': case 'n':
					 dummy=(P++)[0] ;
					 switch(dummy){ 
#if MIX
					         case 'P': case 'p':
							 pvalue=1;
                             neighborFile=1;
                             strcpy(neighborFileName, argv[i]+3);
					         if(neighborFileName[0]=='\0'){
					               i++;
						           if(argv[i][0]=='-'){
						                fprintf(stderr, "Illegal neighbour filename: %s\n\n", argv[i]);
						                PrintUsage();
						                exit(1);
							       }
							       strcpy(neighborFileName, argv[i]);
							}
							break;
#endif  
							case 'N': case 'n':
                             neighborFile=1;
                             strcpy(neighborFileName, argv[i]+3);
					         if(neighborFileName[0]=='\0'){
					               i++;
						           if(argv[i][0]=='-'){
						                fprintf(stderr, "Illegal neighbour filename: %s\n\n", argv[i]);
						                PrintUsage();
						                exit(1);
							       }
							       strcpy(neighborFileName, argv[i]);
							}
							break;
							case 'C': case 'c':
							 catnei=1;
                             neighborFile=1;
                             strcpy(neighborFileName, argv[i]+3);
					         if(neighborFileName[0]=='\0'){
					               i++;
						           if(argv[i][0]=='-'){
						                fprintf(stderr, "Illegal neighborfilename filename: %s\n\n", argv[i]);
						                PrintUsage();
						                exit(1);
							       }
							       strcpy(neighborFileName, argv[i]);
							}
							break;
							
							case 'R': case 'r':
                             ctFile=1;
							 neighborFile=1;
							 strcpy(ctFileName, argv[i]+3); 
							 strcpy(neighborFileName, argv[i]+3);
							 if(ctFileName[0]=='\0'){
					               i++;
						           if(argv[i][0]=='-'){
						                fprintf(stderr, "Illegal ct filename: %s\n\n", argv[i]);
						                PrintUsage();
						                exit(1);
							       }
							       strcpy(ctFileName, argv[i]);
							}
					       break;
					 }	  
                     break;
					 case 'K': case 'k':
                     sequenceFile=1;
                     strcpy(sequenceFileName, argv[i]+2);
					 if(sequenceFileName[0]=='\0'){
					    i++;
						if(argv[i][0]=='-'){
						  PrintUsage();
						  exit(1);
						}
						strcpy(sequenceFileName, argv[i]);
					 }	  
					 break;
                     case 'L': case 'l':
                     seqlen = getintNumber(P);
                     break;
					 case 'T': case 't':
					 transitiontransverion=1;
                     trtv = getdoubleNumber(P);
					 fprintf(stderr, "transition-transverion parameter %lf\n", trtv);
                     break;
                     case 'A': case 'a':
                     numDatasets = getintNumber(P);
                     break;
					 case 'R': case 'r':					
					 dummy=(P++)[0] ;
					 switch(dummy){ 
					       case 'O': case 'o':
						   overlappings=1;
						   break;
						   case 'C': case 'c':
						   CpG=1;
						     if (GetDoubleParams(argc, argv, &i, P, 2, CpGarray)) {
                                  fprintf(stderr, "Bad DoubletFrequencies: %s\n\n", argv[i]);
                                  exit(0);
				        	}
                           CpGTpG = CpGarray[0];
						   CpGCpA = CpGarray[1];
						   fprintf(stderr, " CpG %lf %lf\n ", CpGarray[0],CpGarray[1]);
						   break;
						   case 'A': case 'a':
						   mononucneigh1=1;
						   break;
						   case 'B': case 'b':
						   mononucneigh2=1;
						   break;
						   case 'R': case 'r':   /*for later extension to all other models*/
						   prmat=1;
                           if (GetDoubleParams(argc, argv, &i, P, 6, Rmat)/*==0*/) {
                              fprintf(stderr, "Bad General Rate Matrix: %s\n\n", argv[i]);
                              exit(0);
                           }
						   if (Rmat[5]!=1.0) {
                              for (j=0; j<5; j++)
                                Rmat[j]/=Rmat[5];
                                Rmat[5]=1.0;

                          }
						  break;
						  case 'D': case 'd':   /*for later extension to all other models*/
						   prmat16=1;
                           if (GetDoubleParams(argc, argv, &i, P, 24, Rmat16)/*==0*/) {
                              fprintf(stderr, "Bad General DI-Rate Matrix: %s\n\n", argv[i]);
                              exit(0);
                           }
						   if (Rmat16[23]!=1.0) {
                              for (j=0; j<23; j++)
                                Rmat16[j]/=Rmat16[23];
                                Rmat16[23]=1.0;

                          }
						  break;
						  case 'T': case 't':   /*for later extension to all other models*/
						   prmat2t=1;
                           if (GetDoubleParams(argc, argv, &i, P, 96, Rmat2T)/*==0*/) {
                              fprintf(stderr, "Bad General Rate Matrix: %s\n\n", argv[i]);
                              exit(0);
                           }
						   if (Rmat2T[95]!=1.0) {
                              for (j=0; j<95; j++)
                                Rmat2T[j]/=Rmat2T[95];
                                Rmat2T[95]=1.0;

                          }
						  break;
						  
					 }
					 break;	  
                     case 'F': case 'f':					
					    dummy=(P++)[0] ;
					    switch(dummy){ 
					         case 'S': case 's':
							 if (GetDoubleParams(argc, argv, &i, P, 4,mpi)) {
                                 fprintf(stderr, "Bad Nucleotide Frequencies: %s\n\n", argv[i]);
                                 exit(0);
					         }
						     break;			
					         case 'D': case 'd':
					         if (GetDoubleParams(argc, argv, &i, P, 16, dipi)) {
                                  fprintf(stderr, "Bad DoubletFrequencies: %s\n\n", argv[i]);
                                  exit(0);
				        	}
					        break;
					        case 'I': case 'i': 
					              /*independent to the random frequencies*/
	                              dipi[0] =mpi[0]*mpi[0];   /*aa*/
								  dipi[1] =mpi[0]*mpi[1];   /*ac*/
                                  dipi[2] =mpi[0]*mpi[2];   /*ag*/
								  dipi[3] =mpi[0]*mpi[3];   /*at*/
                                  dipi[4] =mpi[1]*mpi[0];   /*ca*/
								  dipi[5] =mpi[1]*mpi[1];   /*cc*/
                                  dipi[6] =mpi[1]*mpi[2];   /*cg*/
                                  dipi[7] =mpi[1]*mpi[3];   /*ct*/
                                  dipi[8] =mpi[2]*mpi[0];   /*ga*/
                                  dipi[9] =mpi[2]*mpi[1];   /*gc*/
                                  dipi[10]=mpi[2]*mpi[2];   /*gg*/
                                  dipi[11]=mpi[2]*mpi[3];   /*gt*/
                                  dipi[12]=mpi[3]*mpi[0];   /*ta,ua*/
                                  dipi[13]=mpi[3]*mpi[1];   /*tc,uc*/
                                  dipi[14]=mpi[3]*mpi[2];   /*tg,ug*/
                                  dipi[15]=mpi[3]*mpi[3];   /*tt,uu*/
					         break;
							 case 'U': case 'u':
							       triFile=1;
							       for(i=0;i<64;i++) tripi[i]=dipi[i%16]/4 ;
							 break;	
							 case 'V': case 'v':
							       triFile=1;
								   tripi[0]=dipi[0%16]/4;
								   tripi[1]=tripi[4]=tripi[16]=dipi[1%16]/4;
								   tripi[2]=tripi[8]=tripi[32]=dipi[2%16]/4;
								   tripi[3]=tripi[12]=tripi[48]=dipi[3%16]/4;
								   tripi[5]=tripi[17]=tripi[20]=dipi[5%16]/4;
								   tripi[6]=tripi[9]=tripi[18]=tripi[24]=tripi[33]=tripi[36]=dipi[6%16]/4;
								   tripi[7]=tripi[13]=tripi[19]=tripi[28]=tripi[49]=tripi[52]=dipi[7%16]/4;
                                   tripi[10]=tripi[34]=tripi[40]=dipi[10%16]/4;
								   tripi[11]=tripi[14]=tripi[35]=tripi[44]=tripi[56]=tripi[50]=dipi[11%16]/4;
								   tripi[15]=tripi[51]=tripi[60]=dipi[15%16]/4;
								   tripi[21]=dipi[21%16]/4;
								   tripi[22]=tripi[25]=tripi[37]=dipi[22%16]/4;
								   tripi[23]=tripi[29]=tripi[53]=dipi[23%16]/4;
								   tripi[26]=tripi[38]=tripi[41]=dipi[26%16]/4;
                                   tripi[27]=tripi[30]=tripi[39]=tripi[45]=tripi[54]=tripi[57]=dipi[27%16]/4;
								   tripi[31]=tripi[55]=tripi[66]=dipi[15%16]/4;
								   tripi[42]=dipi[42%16]/4;
                                   tripi[43]=tripi[46]=tripi[58]=dipi[43%16]/4;
								   tripi[47]=tripi[59]=tripi[62]=dipi[47%16]/4;
                                   tripi[63]=dipi[63%16]/4;
								   
								  fprintf(stderr,"%s\n",triplet[0]);
								  fprintf(stderr,"%s %s %s \n",triplet[1], triplet[4], triplet[16]);
                                  fprintf(stderr,"%s %s %s \n",triplet[2], triplet[8], triplet[32]);
								  fprintf(stderr,"%s %s %s \n",triplet[3], triplet[12], triplet[48]);
								  fprintf(stderr,"%s %s %s \n",triplet[5], triplet[17], triplet[20]);
								  fprintf(stderr,"%s %s %s %s %s %s\n",triplet[6], triplet[9], triplet[18],triplet[24], triplet[33], triplet[36]);
								  fprintf(stderr,"%s %s %s %s %s %s\n",triplet[7], triplet[13], triplet[19],triplet[28], triplet[49], triplet[52]);
								  fprintf(stderr,"%s %s %s \n",triplet[10], triplet[34], triplet[40]);
								  fprintf(stderr,"%s %s %s %s %s %s\n",triplet[11], triplet[14], triplet[35],triplet[44], triplet[56], triplet[59]);
								  fprintf(stderr,"%s %s %s \n",triplet[15], triplet[51], triplet[60]);
								  fprintf(stderr,"%s\n",triplet[21]);
								  fprintf(stderr,"%s %s %s \n",triplet[22], triplet[25], triplet[37]);
								  fprintf(stderr,"%s %s %s \n",triplet[23], triplet[29], triplet[53]);
								  fprintf(stderr,"%s %s %s \n",triplet[26], triplet[38], triplet[41]);
								  fprintf(stderr,"%s %s %s %s %s %s\n",triplet[27], triplet[30], triplet[39],triplet[45], triplet[54], triplet[57]);
								  fprintf(stderr,"%s %s %s \n",triplet[31], triplet[55], triplet[61]);
								  fprintf(stderr,"%s\n",triplet[42]);
								  fprintf(stderr,"%s %s %s \n",triplet[43], triplet[46], triplet[58]);
								  fprintf(stderr,"%s %s %s \n",triplet[47], triplet[59], triplet[62]);
								  fprintf(stderr,"%s\n",triplet[63]);
                             break;	   
                             case 'Y': case 'y':
								   tripi[0]=0.25;
								   tripi[1]=tripi[4]=tripi[16]=0.01;
								   tripi[2]=tripi[8]=tripi[32]=0.01;
								   tripi[3]=tripi[12]=tripi[48]=0.01;
								   tripi[5]=tripi[17]=tripi[20]=0.01;
								   tripi[6]=tripi[9]=tripi[18]=tripi[24]=tripi[33]=tripi[36]=0.01;
								   tripi[7]=tripi[13]=tripi[19]=tripi[28]=tripi[49]=tripi[52]=0.01;
                                   tripi[10]=tripi[34]=tripi[40]=0.01;
								   tripi[11]=tripi[14]=tripi[35]=tripi[44]=tripi[56]=tripi[50]=0.01;
								   tripi[15]=tripi[51]=tripi[60]=0.01;
								   tripi[21]=0.01;
								   tripi[22]=tripi[25]=tripi[37]=0.01;
								   tripi[23]=tripi[29]=tripi[53]=0.01;
								   tripi[26]=tripi[38]=tripi[41]=0.01;
                                   tripi[27]=tripi[30]=tripi[39]=tripi[45]=tripi[54]=tripi[57]=0.01;
								   tripi[31]=tripi[55]=tripi[66]=0.01;
								   tripi[42]=0.01;
                                   tripi[43]=tripi[46]=tripi[58]=0.01;
								   tripi[47]=tripi[59]=tripi[62]=0.01;
                                   tripi[63]=0.01;
                             break;
							 case 'T': case 't':
                             triFile=1;
                             strcpy(triFileName, argv[i]+3);
							 if(triFileName[0]=='\0'){
								  i++;
								  if(argv[i][0]=='-'){
									fprintf(stderr, "Illegal neighbour filename: %s\n\n", argv[i]);
						            PrintUsage();
						            exit(1);
								 }
						         strcpy(triFileName, argv[i]);
					         }						 	  
					        break;
							case 'M': case 'm':
                            minimumvalue = getdoubleNumber(P); /* only with ctfile - option nr*/
							ctcountfreq=1;
							break;
							
				    }
					break;				
					case 'D' : case 'd':
					nucleotide[0]='A',nucleotide[1]='C',nucleotide[2]='G',nucleotide[3]='T';
					doublet[3]="AT",doublet[7]="CT",doublet[11]="GT",doublet[12]="TA",doublet[13]="TC",doublet[14]="TG",doublet[15]="TT";
					 /*do triplets!*/
	                 triplet[0]="AAA",triplet[1]="AAC",triplet[2]= "AAG",triplet[3] ="AAT";
 	                 triplet[4]="ACA",triplet[5]="ACC",triplet[6]= "ACG",triplet[7] ="ACT";
	                 triplet[8]="AGA",triplet[9]="AGC",triplet[10]="AGG",triplet[11]="AGT";
					 triplet[12]="ATA",triplet[13]="ATC",triplet[14]="ATG",triplet[15]="ATT";
					 triplet[16]="CAA",triplet[17]="CAC",triplet[18]="CAG",triplet[19]="CAT";
					 triplet[20]="CCA",triplet[21]="CCC",triplet[22]="CCG",triplet[23]="CCT";
					 triplet[24]="CGA",triplet[25]="CGC",triplet[26]="CGG",triplet[27]="CGT";
					 triplet[28]="CTA",triplet[29]="CTC",triplet[30]="CTG",triplet[31]="CTT";
	                 triplet[32]="GAA",triplet[33]="GAC",triplet[34]="GAG",triplet[35]="GAT";
	                 triplet[36]="GCA",triplet[37]="GCC",triplet[38]="GCG",triplet[39]="GCT";
	                 triplet[40]="GGA",triplet[41]="GGC",triplet[42]="GGG",triplet[43]="GGT";
	                 triplet[44]="GTA",triplet[45]="GTC",triplet[46]="GTG",triplet[47]="GTT";
	                 triplet[48]="TAA",triplet[49]="TAC",triplet[50]="TAG",triplet[51]="TAT";
	                 triplet[52]="TCA",triplet[53]="TCC",triplet[54]="TCG",triplet[55]="TCT";
					 triplet[56]="TGA",triplet[57]="TGC",triplet[58]="TGG",triplet[59]="TGT";
	                 triplet[60]="TTA",triplet[61]="TTC",triplet[62]="TTG",triplet[63]="TTT";	
					break;
#if distance					
					case 'T':  case 't':
					distance=getdoubleNumber(P);
                    break;
#endif					
					case 'M': case 'm':
					matrixSH=1;
					break;
                
					case 'O': case 'o':
					     dummy=(P++)[0] ;
					     switch(dummy){ 
					         case 'R': case 'r':
							 relaxedPhylip=1;
						     break;
							 case 'F': case 'f':
							 FastaAln=1;
						     break;
							 case 'C': case 'c':
							 Clustal=1;
						     break;
						 }	
					 break;
					 case 'W': case 'w':
					     dummy=(P++)[0] ;
					     switch(dummy){ 
					         case 'A': case 'a':
							 writeAncestors=1;
						     break;	
#if HAMMINGDISTANCE
						     case 'H': case 'h':
							 writeHammingdistance=1; 
							 break;
						     case 'T': case 't':
							 writeHammingdistanceTree=1;
							 break;
							 case 'O': case 'o':
							 writenote=1; 
							 break;
							  case 'P': case 'p':
                             printfile=1;
						     strcpy(filename, argv[i]+3);
							 if(filename[0]=='\0'){
								  i++;
								  if(argv[i][0]=='-'){
									fprintf(stderr, "Illegal filename in option -wp: %s\n\n", argv[i]);
						            PrintUsage();
						            exit(1);
							 }else fprintf(stderr, "Illegal filename in option -wp: %s\n\n", argv[i]);

						         strcpy(filename, argv[i]);
					         }				
#endif				         
                            break;
							case 'B': case 'b':
							binalphabet=1;
							break;
							 
					 }
					 break;
					 case 'V': case 'v':
					     dummy=(P++)[0] ;
					     switch(dummy){ 
					         case 'Q': case 'q':
							 verboseQMatrix=1;
						     break;	
							 /*neighborFiles*/
							 case 'N': case 'n':
							 verboseNeighborNei=1;
						     break;	
							 case 'M': case 'm':
							 verboseNeighborMatrix=1;
						     break;		
							 case 'C': case 'c':
							 verboseNeighborCt=1;
						     break;
							 case 'T': case 't':
							 verboseEvolutiontime=1;
						     break;
							 case 'R': case 'r':
							 verboseRate=1;
						     break;
							 case 'U': case 'u':
							 verboserandom=1;
						     break;
							 case 'W': case 'w':
							 verbosewaiting=1;
						     break;
							 case 'F': case 'f':
							 verboseFrequencies=1;
						     break;
							 case 'Z': case 'z':
							 verboseRandomSeed=1;
						     break;
							 case 'A': case 'a':
							 verboseWarning=1;
						     break;
							 case 'S': case 's':
							 verboseStep=1;
						     break;
					 }
					 break;
					 case 'S': case 's':
					     dummy=(P++)[0] ;
					     switch(dummy){ 
					         case 'B': case 'b':
							 scaleBranches=1;
							 if (GetDoubleParams(argc, argv, &i, P, 1, &branchScale) || branchScale<=0.0) {
						         fprintf(stderr, "Bad Branch Length Scale: %s\n\n", argv[i]);
						         exit(0);
					         }
					         if (scaleTrees) {
					          	 fprintf(stderr, "You can't specify both the -sb and -st options\n\n");
						         exit(0);
					         }
							 break;	
							 case 'T': case 't':
							 scaleTrees=1;
							 scaleTrees=1;
					         if (GetDoubleParams(argc, argv, &i, P, 1, &treeScale) || treeScale<=0.0) {
					          	fprintf(stderr, "Bad Total Tree Scale: %s\n\n", argv[i]);
					        	exit(0);
					         }
					         if (scaleBranches) {
								fprintf(stderr, "You can't specify both the -sb and -st options\n\n");
								exit(0);
					         }
							 break;	
                         }
					 break;	 
					  case 'E': case 'e':
					     dummy=(P++)[0] ;
					     switch(dummy){ 
					         case 'E': case 'e':
							 sitefactor=1;
					         sitefactorFile=1;
                             strcpy(sitefactorFileName, argv[i]+3);
							 if(sitefactorFileName[0]=='\0'){
								  i++;
								  if(argv[i][0]=='-'){
									fprintf(stderr, "Illegal site-factor filename: %s\n\n", argv[i]);
						            PrintUsage();
						            exit(1);
								 }
						         strcpy(sitefactorFileName, argv[i]);
					         }		
						     break;	
							 case 'N': case 'n':
							 sitefactornorm=1;
					         sitefactorFile=1;
                             strcpy(sitefactorFileName, argv[i]+3);
							 if(sitefactorFileName[0]=='\0'){
								  i++;
								  if(argv[i][0]=='-'){
									fprintf(stderr, "Illegal site-factor filename: %s\n\n", argv[i]);
						            PrintUsage();
						            exit(1);
								 }
						         strcpy(sitefactorFileName, argv[i]);
					         }		
							break;	
							case 'D': case 'd':
							 sitefactor2=1;
					         sitefactorFile=1;
                             strcpy(sitefactorFileName, argv[i]+3);
							 if(sitefactorFileName[0]=='\0'){
								  i++;
								  if(argv[i][0]=='-'){
									fprintf(stderr, "Illegal site-factor filename: %s\n\n", argv[i]);
						            PrintUsage();
						            exit(1);
								 }
						         strcpy(sitefactorFileName, argv[i]);
					         }		
						     break;	
							 case 'M': case 'm':
							 sitefactornorm2=1;
					         sitefactorFile=1;
                             strcpy(sitefactorFileName, argv[i]+3);
							 if(sitefactorFileName[0]=='\0'){
								  i++;
								  if(argv[i][0]=='-'){
									fprintf(stderr, "Illegal site-factor filename: %s\n\n", argv[i]);
						            PrintUsage();
						            exit(1);
								 }
						         strcpy(sitefactorFileName, argv[i]);
					         }		
							break;	
                         }
					 break;	 

					 case 'Q': case 'q':
                     quiet=1;
					 break;
					 case 'I': case 'i':
                     fastrates=1;
					 break;
					 case 'X': case 'x':
                     mix=1;
					 break;
					 case 'Y': case 'y':
                     randomgeneratordkiss=0;
					 break;
					 case 'Z': case 'z':
					 userSeed = 1;
					 if (GetUnsignedLongParams(argc, argv, &i, P, 1, &randomSeed)) {  /*random seed*/
						fprintf(stderr, "Bad random number generator seed: %s\n\n", argv[i]);
						exit(0);
					}
				    break;


               }
           }
        }
    }

 /****************************************************************************************************************/
 /*             Verbose                                                                                          */
 /****************************************************************************************************************/
 void PrintVerbose(FILE *fv){
   fprintf(fv, "\nSimulations of %d taxa, %d nucleotides\n",   numTaxa, seqlen);   /*seqlen=numSites*/
   fprintf(fv, "for %d tree(s) with %d dataset(s) per tree\n", numTrees, numDatasets);
 
   if(neighborFile==1) fprintf(fv, "name of the use neighborhoodfile: %s\n", neighborFileName);
   if(sequenceFile==1) fprintf(fv, "name of the root sequence: %s\n", sequenceFileName);
   if(triFile==1)      fprintf(fv,"tri file name: %s\n", triFileName);
   if(sitefactorFile==1)fprintf(fv,"site-specific scaling factor %s\n", sitefactorFileName);
   if(seqlen > MAX_POS){ fprintf(fv, "Maximal number of positions is %d!\n", MAX_POS); 
                         fprintf(fv, "If you need more sites, change MAX_POS in neigh.h\n"); 
                         fprintf(fv, "or write an email to sissi@cs.uni-duesseldorf.de.\n");  
						 fprintf(fv, "\n");
                         exit(0);
					   }                  
   if(treeFile==1)     fprintf(fv,"tree file name: %s\n", treeFileName);
       else            fprintf(fv,"MISSING TREE!");
   fprintf(fv, "\n");	   
  }
   /****************************************************************************************************************/
 /*             READ site-specific scaling factor                                                                     */
 /****************************************************************************************************************/

  double  *OpenSiteFactorFile()
  {
    int i;
	double sum;
	double *a;
	
     if(sitefactorFile==1){
        if((sitefactor_fv=fopen(sitefactorFileName,"r"))==NULL){
           fprintf(stderr,"Error opening sitefactorFile: '%s'\n", sitefactorFileName);
           exit (0);
         }
      }
	  
	 scalingfactor=(double *)malloc(seqlen*sizeof(double));
	 
	 for(i=0; i<seqlen; i++) scalingfactor[i]=0;
	 if(verboseScalingFactor==1)fprintf(stderr, "\n site-specific scalingfactor:\n ");
	 
	 if(sitefactor ==1 || sitefactornorm ==1){
	     for(i=0; i<seqlen; i++) { fscanf(sitefactor_fv,"%lf", &(scalingfactor[i])) ;}
		 for(i=0; i<seqlen; i++) fprintf (stderr,"%lf\n", scalingfactor[i]) ;
		  fscanf(sitefactor_fv,"%lf", &a);
		  if(((int) a > 0 ) && scalingfactor[seqlen-1] > 0.0000) fprintf(stderr, "Warning: Check number of scaling-factor with site-specific rate, sequence length too short!\n");
		  if(((int) a < 0 ) && scalingfactor[seqlen-1] == 0.000) fprintf(stderr, "Warning: Check number of scaling-factor with site-specific rate, Last entry 0 or sequence lenght too long!\n");
	 }
	
	  if(sitefactor2 == 1 || sitefactornorm2 == 1){
	   for(i=0; i<seqlen; i++) fscanf(sitefactor_fv,"%d %lf", &a, &(scalingfactor[i])) ;
	    fscanf(sitefactor_fv,"%d ",&a);
		if( (int) a != seqlen ){
		     fprintf(stderr, "Check the number of site-specific rates with the sequencelength!\n");
			 exit (1);
		}	 
	   }
	   
	   
	if(printfile==1){
		sprintf(sissi_sitefactorFileName, "%s_sitefacotor1.sissi", filename);
		if(( sissi_sitefactor_fv=fopen(sissi_sitefactorFileName, "w"))==NULL)
				fprintf(stderr, "sissi_sitefactorFileName could not open\n");
		for(i=0;i<seqlen;i++) fprintf(sissi_sitefactor_fv, "%d  %.16lf\n", (i+1), scalingfactor[i]);
	}	
   
	sum=0.00;
	for(i=0; i<seqlen; i++) sum= sum + scalingfactor[i];
	if(!quiet) fprintf(stderr,"\n sequence length %d and sum of ssf =%lf\n", seqlen, sum);
	/**Normierung**/
	if(((sum < seqlen - NEARZERO) || (sum > seqlen + NEARZERO)) && (sitefactornorm==1 || sitefactornorm2==1) ){
	       for(i=0; i<seqlen; i++) scalingfactor[i]=(scalingfactor[i]/sum) * seqlen ;
	       sum=0.0;
	       for(i=0; i<seqlen; i++) sum= sum + scalingfactor[i];
		   if(verboseScalingFactor==1){
		       fprintf(stderr,"\n sequence length %d and new sum of ssf =%lf\n",seqlen, sum);
		       for(i=0; i<seqlen; i++) fprintf(stderr, "pos:%d fk:%f ", i, scalingfactor[i]);
	}
	       
		   
	if(!quiet) fprintf(stderr,"\n SISSI has normalized the scalingfactor with sum  %lf (%d sequence length)!", sum, seqlen);
	if(!quiet) fprintf(stderr,"\n More details with option: -ve!\n");
	if(!quiet) printf("\n");
	if(verboseScalingFactor==1) for(i=0; i<seqlen; i++) fprintf(stderr, "pos: %d fk:%lf\n", i, scalingfactor[i]);
		   
	if(printfile==1 && (sitefactor2 == 1 || sitefactornorm2 == 1)){
		   sprintf(sissi_sitefactorFileName_norm, "%s_sitefacotor2norm.sissi", filename);
            if(( sissi_sitefactor_norm_fv=fopen(sissi_sitefactorFileName_norm, "a"))==NULL)
				fprintf(stderr, "sissi_sitefactorFileName_norm could not open\n");
			   for(i=0;i<seqlen;i++) fprintf(sissi_sitefactor_norm_fv, "%d %lf\n", (i+1), scalingfactor[i]);
			   fclose(sissi_sitefactor_norm_fv);
		    }	
	}
	
	
	if(printfile==1) {
	  fclose(sissi_sitefactor_fv);
	}   
    if(!quiet) fprintf(stderr, " \n");
 	fclose(sitefactor_fv);
    return(tripi) ;
}

  
 
 /****************************************************************************************************************/
 /*             READ frequence triplet file                                                                      */
 /****************************************************************************************************************/
  double  *OpenTriFile()
  {
  
     int i;
     if(triFile==1){
        if((tri_fv=fopen(triFileName,"r"))==NULL){
           fprintf(stderr,"Error opening tripleFile: '%s'\n", triFileName);
           exit (0);
         }
         for(i=0; i<64; i++)
               fscanf(tri_fv,"%lf", &(tripi[i])) ;
	}
if(	verboseQMatrix==1){
    if(nucleotide[3]=='t' || nucleotide[3]=='T'){
    fprintf(stderr, "frequencies:aaa:%f | aac:%f | aag:%f | aat:%f\t\n",  tripi[0],  tripi[1],  tripi[2],  tripi[3]);
    fprintf(stderr, "           :aca:%f | acc:%f | acg:%f | act:%f\t\n",  tripi[4],  tripi[5],  tripi[6],  tripi[7]);
    fprintf(stderr, "           :aga:%f | agc:%f | agg:%f | agt:%f\t\n",  tripi[8],  tripi[9], tripi[10], tripi[11]);
    fprintf(stderr, "           :ata:%f | atc:%f | atg:%f | att:%f\t\n",  tripi[12], tripi[13], tripi[14], tripi[15]);
    fprintf(stderr, "           :caa:%f | cac:%f | cag:%f | cat:%f\t\n",  tripi[16], tripi[17], tripi[18], tripi[19]);
    fprintf(stderr, "           :cca:%f | ccc:%f | ccg:%f | cct:%f\t\n",  tripi[20], tripi[21], tripi[22], tripi[23]);
    fprintf(stderr, "           :cga:%f | cgc:%f | cgg:%f | cgt:%f\t\n",  tripi[24], tripi[25], tripi[26], tripi[27]);
    fprintf(stderr, "           :cta:%f | ctc:%f | ctg:%f | ctt:%f\t\n",  tripi[28], tripi[29], tripi[30], tripi[31]);
    fprintf(stderr, "           :gaa:%f | gac:%f | gag:%f | gat:%f\t\n",  tripi[32], tripi[33], tripi[34], tripi[35]);
    fprintf(stderr, "           :gca:%f | gcc:%f | gcg:%f | gct:%f\t\n",  tripi[36], tripi[37], tripi[38], tripi[39]);
    fprintf(stderr, "           :gga:%f | ggc:%f | ggg:%f | ggt:%f\t\n",  tripi[40], tripi[41], tripi[42], tripi[43]);
    fprintf(stderr, "           :gta:%f | gtc:%f | gtg:%f | gtt:%f\t\n",  tripi[44], tripi[45], tripi[46], tripi[47]);
    fprintf(stderr, "           :taa:%f | tac:%f | tag:%f | tat:%f\t\n",  tripi[48], tripi[49], tripi[50], tripi[51]);
    fprintf(stderr, "           :tca:%f | tcc:%f | tcg:%f | tct:%f\t\n",  tripi[52], tripi[53], tripi[54], tripi[55]);
    fprintf(stderr, "           :tga:%f | tgc:%f | tgg:%f | tgt:%f\t\n",  tripi[56], tripi[57], tripi[58], tripi[59]);
    fprintf(stderr, "           :tta:%f | ttc:%f | ttg:%f | ttt:%f\t\n",  tripi[60], tripi[61], tripi[62], tripi[63]);
    }else {
    fprintf(stderr, "           :aaa:%f aac:%f aag:%f aau:%f\t\n",   tripi[0],  tripi[1],  tripi[2],  tripi[3]);
    fprintf(stderr, "           :aca:%f acc:%f acg:%f acu:%f\t\n",   tripi[4],  tripi[5],  tripi[6],  tripi[7]);
    fprintf(stderr, "           :aga:%f agc:%f agg:%f agu:%f\t\n",   tripi[8],  tripi[9], tripi[10], tripi[11]);
    fprintf(stderr, "           :aua:%f auc:%f aug:%f auu:%f\t\n",  tripi[12], tripi[13], tripi[14], tripi[15]);
    fprintf(stderr, "           :caa:%f cac:%f cag:%f cau:%f\t\n",  tripi[16], tripi[17], tripi[18], tripi[19]);
    fprintf(stderr, "           :cca:%f ccc:%f ccg:%f ccu:%f\t\n",  tripi[20], tripi[21], tripi[22], tripi[23]);
    fprintf(stderr, "           :cga:%f cgc:%f cgg:%f cgu:%f\t\n",  tripi[24], tripi[25], tripi[26], tripi[27]);
    fprintf(stderr, "           :cua:%f cuc:%f cug:%f cuu:%f\t\n",  tripi[28], tripi[29], tripi[30], tripi[31]);
    fprintf(stderr, "           :gaa:%f gac:%f gag:%f gau:%f\t\n",  tripi[32], tripi[33], tripi[34], tripi[35]);
    fprintf(stderr, "           :gca:%f gcc:%f gcg:%f gcu:%f\t\n",  tripi[36], tripi[37], tripi[38], tripi[39]);
    fprintf(stderr, "           :gga:%f ggc:%f ggg:%f ggu:%f\t\n",  tripi[40], tripi[41], tripi[42], tripi[43]);
    fprintf(stderr, "           :gua:%f guc:%f gug:%f guu:%f\t\n",  tripi[44], tripi[45], tripi[46], tripi[47]);
    fprintf(stderr, "           :uaa:%f uac:%f uag:%f uau:%f\t\n",  tripi[48], tripi[49], tripi[50], tripi[51]);
    fprintf(stderr, "           :uca:%f ucc:%f ucg:%f ucu:%f\t\n",  tripi[52], tripi[53], tripi[54], tripi[55]);
    fprintf(stderr, "           :uga:%f ugc:%f ugg:%f ugu:%f\t\n",  tripi[56], tripi[57], tripi[58], tripi[59]);
    fprintf(stderr, "           :uua:%f uuc:%f uug:%f uuu:%f\t\n",  tripi[60], tripi[61], tripi[62], tripi[63]);
    }
 }	
	fclose(tri_fv);
    return(tripi) ;
}
   
  /*******************************************************/
  /*      READ THE TREEFILE AND COUNT THE TREES          */
  /*******************************************************/
/*this is a function of seqgen: see copyright seq-gen */
int OpenTreeFile(){   
 	char st[256];
	int n;
		
	if (treeFile) {
		if ( (tree_fv=fopen(treeFileName, "rt"))==NULL ) {
			fprintf(stderr, "Error opening tree file: '%s'\n", treeFileName);
			exit(0);
		}
		n=CountTrees(tree_fv);
	} else {
		tree_fv=stdin;
		if (hasAlignment) {
			fgets(st, 255, stdin);
			if ( sscanf(st, " %d ", &n)!=1 ) {
				fprintf(stderr, "Tree is missing from end of sequence file\n");
				exit(0);
			}
		} else
			n=CountTrees(stdin);
	}
	
	return n;
}
 /************************************************************************************************/
 /*               NEIGHBORHOODFILE                                                               */
 /************************************************************************************************/
  int OpenNeighFile(){   
	int n;
	
	if ( (neighbor_fv=fopen(neighborFileName, "rt"))==NULL ) {
			fprintf(stderr, "Error opening neighbor file: '%s'\n", neighborFileName);
			exit(0);
		}
		n=1;     /*Increments later: different neighbourhood systems simulatanously*/
#if MIX
   if(pvalue==1){
	  	    readneighPvalues(neighbor_fv, neighborFileName);
   }else if(catnei==1){
	        readneighcat(neighbor_fv,neighborFileName) ;
   }else if(ctFile==1){
			readctfile(neighbor_fv, neighborFileName); 
		
   }else 
	        readneighnorm(neighbor_fv, neighborFileName);	 /*nn option */
#endif	
if(catnei==1){
	        readneighcat(neighbor_fv,neighborFileName) ;
   }else if(ctFile==1){
			readctfile(neighbor_fv, neighborFileName); 
   }else 
	        readneighnorm(neighbor_fv, neighborFileName);	 /*nn option */
		
		
   return n;
}

/****************************************************************************************************************/
/*this is a function of seq-gen*/
void AllocateMemory()
{
	int i;
	
	names=(char **)AllocMem(sizeof(char *)*numseq, "names", "AllocateMemory", 0);
	sequences=(char **)AllocMem(sizeof(char *)*numseq, "sequences", "AllocateMemory", 0);
	for (i=0; i<numseq; i++) {
	names[i]=(char *)AllocMem(sizeof(char)*(MAX_NAME_LEN+1), "names[]", "AllocateMemory", 0);
	sequences[i]=(char *)AllocMem(sizeof(char)*seqlen, "sequences[]", "AllocateMemory", 0);
	}
}
/****************************************************************************************************************/
/****************************************************************************************************************/
/*                                      MAIN FUNKTION                                                           */
/****************************************************************************************************************/

int main(int argc, char **argv){
	int j,i;
	int s0 = -1;
	int treeNo ;
	double *Qij=NULL;
	double **QSHij=NULL;
	double **QTij=NULL;
	double **QT2ij=NULL;
	double **QT3ij=NULL;

	/*double **p_value_table;*/   /* INCREMENTS : correlation values, P-values */
	double totalSecs;
	double scale;
	
	TTree **treeSet;		
	char *ancestor;
	char *rseq;
	
	clock_t totalStart;	
	/*totalStart=clock();*/
	/***************************************/
	ancestor=NULL;
    numTaxa=-1 ; 
    scale=1.0;
	/***************************************/
	ReadParams(argc,argv);
	if(triFile==1) OpenTriFile();
	ancestor=NULL;
	/***************************************/
    /*** should be moved to mutate.c into a function initrandom(randomSeed) ***/
	/*random gener.*/
	srand((unsigned)time(NULL));  /*initation by clock*/
	if(verboseRandomSeed==1) fprintf(stderr, "\nSPRNG=%d\n", SPRNG);
    if (randomgeneratordkiss == 1){
	     if(verboseRandomSeed==1)
		    fprintf(stderr, "Random generator dkiss\n");
	     if (randomSeed == -1) {
	        s0=rand();
	        start_kiss((long)s0);      /*random with dkiss() */
			if(verboseRandomSeed==1) 
			    fprintf(stderr, "System generated random number generator seed: %ld\n\n", ((long)s0) );
		  }else{
		    start_kiss(randomSeed);	   /*random with dkiss() and user seed*/
		    if(verboseRandomSeed==1)
				fprintf(stderr, "User defined random number generator seed: %ld\n\n", randomSeed);
		 }
	 } else { 
	    if(SPRNG==0) {
		   fprintf(stderr, "SPRNG library not included! Don't use -y option\n"); 
		   exit(0);
		}	 
#if SPRNG	 
	    else
	    {
		   if(verboseRandomSeed==1) fprintf(stderr, "Random generator SPRNG\n");
           if (randomSeed == -1) {
		       s0 = make_sprng_seed(); 
		       if(verboseRandomSeed==1) fprintf(stderr, "System generated random number generator seed: %ld\n\n", ((long)s0) );
		   }	
		   if(userSeed==1) s0=randomSeed;
		   randstream = init_sprng(1,10,s0,SPRNG_DEFAULT); /*initialize stream*/ 
		   /* init SPRNG random number streams (here, stream no. 1 out of 10 different streams (9 unused here), using seed s0 */
		   if(verboseRandomSeed == 1 && userSeed ==1) fprintf(stderr, "User defined random number generator seed: %ld\n\n", ((long)s0));
	     }	
#endif	 
    } 	 		
	/***************************************/

	if(sequenceFile==1) 
	 {
	   if((sequence_fv=fopen(sequenceFileName,"r"))==NULL){
	     fprintf(stderr, "Error opening seqneceFile: '%s'\n", sequenceFileName);
         exit (0);
       }
	   rseq=(char*)malloc(seqlen*sizeof(char));
	   fread(rseq,sizeof(char),seqlen,sequence_fv);
	   ancestor=rseq;
	   fclose(sequence_fv);
	}
	/********read neighborhoodsystem***************************/
	if(neighborFile==1 || ctFile==1)
	     OpenNeighFile();
	numBases=seqlen;	 /*IMPROVEMENTS*/
	/**************site-specific scalingfactor*****************/
	if(sitefactor==1 || sitefactornorm==1 || sitefactor2==1 || sitefactornorm2==1) OpenSiteFactorFile();
	/*****************matrix***********************************/
	/*****************matrix*****************************************/
    Qij=rateRV();           /*independent |Nk|=o */
	QSHij=rateSH();         /*doublet      |Nk|=1 */
    /*for(i=0;i<4;i++)	     rateD(QSHij,i);*/
    if(overlappings==1 && catnei==0) {
	       QT2ij=rateT2(); 
		   fprintf(stderr, "include mutatez\n");
		   exit (1);
	}	   
    else if(triFile==1  || catnei==1 ){                       /*Improvement:count max neighbours*/
		  QTij=rateT();          /*triplet  1  |Nk|=2 */
	      QT2ij=rateT2();        /*triplet  2  |Nk|=2 */
	      QT3ij=rateT3();        /*triplet  3  |Nk|=2 */
	}
	/**********************************************************/
	if(treeFile==0) {    
       fprintf(stderr,"MISSING TREE!\n\n");
       PrintUsage();
       exit(0);
	 }  
	 
    treeSet= (TTree **)malloc(sizeof(TTree *) * maxPartitions);   /*seq-gen: treesets, partionslengths ... , here only for later increments of sissi */
	 if (treeSet==NULL) {            
		fprintf(stderr, "Out of memory\n");
		exit(0);
	}
	
	partitionLengths = (int *)malloc(sizeof(int) * maxPartitions);    
	if (partitionLengths==NULL) {            
		fprintf(stderr, "Out of memory\n");
		exit(0);
	}
	
	
	partitionRates = (double *)malloc(sizeof(double) * maxPartitions);  
	if (partitionRates==NULL) {            
		fprintf(stderr, "Out of memory\n");
		exit(0);
	}

	for(i=0; i< maxPartitions; i++){                                  
    	if ((treeSet[i]=NewTree())==NULL){
	        fprintf(stderr, "Out of memory\n");
	        exit(0);
		}	
	}  				
									
																    
    numTrees = OpenTreeFile();  
	
	totalStart=clock();
		
    treeNo=0;
	do {
	    partitionLengths[0] = -1;
		ReadTree(tree_fv, treeSet[0], treeNo+1,0, NULL, &partitionLengths[0], &partitionRates[0]);
		
		if (treeNo==0) {
			numTaxa=treeSet[0]->numTips;
			CreateSequences(treeSet[0]);
			
			if (scaleTrees) { 
				if (!treeSet[0]->rooted) {
					fprintf(stderr, "To scale tree length, they must be rooted and ultrametric.\n");
					exit(0);
				}
				scale *= treeScale/treeSet[0]->totalLength;
			} else if (scaleBranches)
				scale *=branchScale;
				
			if(!quiet){
			   PrintTitel(stderr);
			   PrintVerbose(stderr);
			}   	
			if(verboseStep == 1 && !quiet)
			   fprintf(stderr, "Matrix and reading ready. Evolve sequences:\n");   

		} else if (treeSet[0]->numTips!=numTaxa) {
			fprintf(stderr, "All trees must have the same number of tips.\n");
			exit(0);
		}  
		
      for (j=0; j<numDatasets; j++) {
			EvolveSequences(treeSet[0], scale, ancestor,Qij,QSHij,QTij,QT2ij,QT3ij);
			if(writeAncestors){
			   WriteAncestralSequences(stdout, treeSet[0]);
			}else{
			  /* if(verboseStep == 1 && !quiet)   fprintf(stderr, ".");*/
			   WriteSequences(stdout, treeSet[0]);
			}   
#if HAMMINGDISTANCE
           if(writenote || writeHammingdistance || writeHammingdistanceTree)
			  WriteHammingdistanceSeq(stderr, treeSet[0]);
#endif	
			
}		
		
				
		DisposeTree(treeSet[0]);
		treeNo++;
	} while (IsTreeAvail(tree_fv));

	
	FreeTree(treeSet[0]);
	
	if (treeFile)
		fclose(tree_fv);
		
		
	/*
	 * Improvement later
	 */
	free(Qij);
	free(QSHij);	
	if(triFile==1  || catnei==1 ){                
	     free(QTij);          /*triplet  1  |Nk|=2 */
	     free(QT2ij);         /*triplet  2  |Nk|=2 */
	     free(QT3ij);         /*triplet  3  |Nk|=2 */
	 }
    /***/
	if(sitefactor==1 || sitefactornorm==1 || sitefactor2==1 || sitefactornorm2==1) free(scalingfactor);
	free(ancestor);
	

	totalSecs = (double)(clock() - totalStart) / CLOCKS_PER_SEC;
	
	if (!quiet) {
		fprintf(stderr, "\n Time taken: %G seconds\n", totalSecs);
		if (verboseMemory)
			fprintf(stderr, "Total memory used: %ld\n", totalMem);
	}
	
   return 0;
}
