/******************************************************************************
 *   models.c                                                                  *
 *                                                                            *
 *   SISSI0.93 (Prerelease Version)                                           *
 *   SImulating Site-Specific Interactions along phylogenetic trees           *
 *                                                                            *
 *   Copyright (C) 2004-2005 by Tanja Gesell, Arndt von Haeseler              *
 *   sissi@cs.uni-duesseldorf.de                                              *
 *                                                                            *
 *   All parts of the source code except where indicated are distributed      *
 *	 under the GNU public licence.                                            *
 *	 See http://www.opensource.org for details.                               *
 *                                                                            *
 *   This program is distributed in the hope that it will be useful,          *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 *   GNU General Public License for more details.                             *
 *                                                                            *
 *   Any feedback is very welcome!              sissi@cs.uni-duesseldorf.de   *
 *****************************************************************************/


 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include "models.h"
 #include "global.h"
 
 
 #define NEARZERO 1.0e-10

 double mpi[4];             /* stationary nucleotide distribution [agct]*/
 double dipi[16];           /* stationary dinucleotide distribution     */
 double tripi[64];          /*  stationary trinucleotide distribution   */
 int    matrixSH;
 double Rmat[6];            /*  6 free rate parameters*/
 double Rmat16[24];
 double Rmat2T[96];
 double ratio;              /*  apha/beta in the model of HKY*/
 double quo;                /*  ratio expected #transitions/#transversions*/
 int    mix;
 double trtv;
 int    transitiontransverion, prmat, prmat2t, prmat16, overlappings;
 double K;
 int    mononucneigh2, mononucneigh1;
 int    CpG;
 double CpGTpG, CpGCpA;
 
 char   nucleotide[4];       /*  nucleotide:m RNA:{A,C,G,U}, with optin -d: DNA:{A,C,G,T}*/
 char   *doublet[]={"AA","AC","AG","AU","CA","CC","CG","CU","GA","GC","GG","GU","UA","UC","UG","UU"};
 char   *triplet[]={"AAA","AAC","AAG","AAU","ACA","ACC","ACG","ACU","AGA","AGC","AGG","AGU","AUA","AUC","AUG","AUU",
					"CAA","CAC","CAG","CAU","CCA","CCC","CCG","CCU","CGA","CGC","CGG","CGU","CUA","CUC","CUG","CUU",
					"GAA","GAC","GAG","GAU","GCA","GCC","GCG","GCU","GGA","GGC","GGG","GGU","GUA","GUC","GUG","GUU",
					"UAA","UAC","UAG","UAU","UCA","UCC","UCG","UCU","UGA","UGC","UGG","UGU","UUA","UUC","UUG","UUU",};
  
/*functions*/
 /****************************************************************************************************************/
 /*                CALCULATE RATE MATRIX                                                                         */
 /****************************************************************************************************************/
 /***************************************/
 /*  RATE MATRIX ACCORDING TO REV       */
 /***************************************/

    double *rateRV(){

    int i, j, k;
    double mr;
    double *Qij;
	double sumpi;

    Qij=(double *)malloc(16*sizeof(double));
 
    if(verboseQMatrix==1 || verboseFrequencies==1){
      fprintf(stderr,"\n");
	  fprintf(stderr,"\t\t\t***MODEL-(nk=0)***\n");
      fprintf(stderr,"\n");
      fprintf(stderr,"pi(nk=0):%c:%f, %c:%f, %c:%f, %c:%f\n"
                           ,nucleotide[0],mpi[0],nucleotide[1],mpi[1],nucleotide[2],mpi[2],nucleotide[3],mpi[3]);
    }						   

    sumpi=0;
	for(i=0;i<4;i++)
	    sumpi += mpi[i];
		
	if(verboseQMatrix==1  || verboseFrequencies==1) fprintf(stderr,"sum of frequencies: %f\n\n", sumpi);	/* if not 1 ....*/
	
	/*if(sumpi!=1.00){ */
	 if((sumpi < 1.00000 - NEARZERO) || (sumpi > 1.00000 + NEARZERO)){
	    for(i=0;i<4;i++)
	      mpi[i] = mpi[i]/sumpi;
		sumpi = 0;
		 for(i=0;i<4;i++) sumpi += mpi[i];
		if(verboseQMatrix==1 || verboseFrequencies==1){  
		  fprintf(stderr,"New\n pi(nk=0):%c:%f, %c:%f, %c:%f, %c:%f\n",nucleotide[0],mpi[0],nucleotide[1],mpi[1],nucleotide[2],mpi[2],nucleotide[3],mpi[3]);
		  fprintf(stderr,"sum pi(n_k=0)%f\n",sumpi);
		}
     }
	 
    if(verboseQMatrix==1) fprintf(stderr,"rate matrix=gamma1:%f, alpha1:%f, beta1:%f,\n beta2:%f, alpha2:%f, gamma2:%f\n\n",Rmat[0],Rmat[1],Rmat[2],Rmat[3],Rmat[4],Rmat[5]);

    k=0;
	
    /*
	 *Initialize the rate parameter
	 */
	 
    for(i=0;i<3;i++)
        for(j=i+1;j<4;j++)
          if(i*4+j !=11)
             Qij[i*4+j]=Qij[j*4+i]=Rmat[k++];     /*reversible*/
    Qij[3*4+2]=Qij[2*4+3]=1.0;                    /*the last one with 1.0*/
	
	
	if(transitiontransverion==1 && prmat==0) {
	     K=2*trtv;
		 fprintf(stderr, "\n HKY Model or Kimura Model with k=%lf trtv=%lf\n", K, trtv);
	     Qij[0*4+2]=Qij[0*4+2]*K;
		 Qij[1*4+3]=Qij[1*4+3]*K;
		 Qij[2*4+0]=Qij[2*4+0]*K;
		 Qij[3*4+1]=Qij[3*4+1]*K;
	}
	if(transitiontransverion==1 && prmat==1){
	   fprintf(stderr, "You cannot use TS/TV Parameter and the GTR model!!\n");
	   exit (0);
	}   

	
	 /*
	 * print ratenmatrix(REV)
	 */
	if(verboseQMatrix==1){
      fprintf(stderr,"unnormalise Rk=0 (independent instantaneous substitution matrix)\n:\n");
     
      for(i=0;i<4;i++){
         fprintf(stderr,"%c ||", nucleotide[i]);
         for(j=0;j<4;j++)
            fprintf(stderr, " %8.5f |",Qij[i*4+j]);
            fprintf(stderr, "\n");
	  }
	
	}
	
	
	/*
	 * multiplied by stationary distribution
	 */
    for(i=0;i<4;i++)
       for(j=0;j<4;j++)
         Qij[i*4+j]*=mpi[j];

	/*
	 * the diagonal values
	 */
    mr=0;
    for (i=0; i<4; i++){
        Qij[i*4+i]=0;                                              /*diagonal entries zero*/
        Qij[i*4+i]=-(Qij[i*4]+Qij[i*4+1]+Qij[i*4+2]+Qij[i*4+3]);   /*mathematical requiremnet: sum of the row*/
        mr-=mpi[i]*Qij[i*4+i];                                     /*sum of the diagonal values*pi */
    }

    /*
	 * print ratenmatrix(REV)
	 */
	if(verboseQMatrix==1){
      fprintf(stderr,"unnormalise Qk=0 (independent instantaneous substitution matrix)\n:\n");
     
      for(i=0;i<4;i++){
         fprintf(stderr,"%c ||", nucleotide[i]);
         for(j=0;j<4;j++)
            fprintf(stderr, " %8.5f |",Qij[i*4+j]);
            fprintf(stderr, "\n");
	  }
	
         fprintf(stderr, "sum(diagonal.values*pi):expected number of substitutions = %f\n", mr);
         fprintf(stderr, "\n");
	  	 
    }
   
    /*
	 * normalize the rate matrix
	 */
      for(i=0;i<4;i++)
      for(j=0;j<4;j++)
        Qij[i*4+j] /= mr;        /*mr:sum of the diagonal values*pi*/


    if(verboseQMatrix==1){
      fprintf(stderr,"  normalise Qk=0 (independent instantaneous substitution matrix)\n");
      for(i=0;i<4;i++){
            fprintf(stderr,"%c ||", nucleotide[i]);
			for(j=0;j<4;j++)
               fprintf(stderr," %8.5f |",Qij[i*4+j]);
            fprintf(stderr,"\n");
	   }
     }			

    /*
	 * diagonal values
	 */
    mr=0;
    for (i=0; i<4; i++){
       mr-=mpi[i]*Qij[i*4+i];      /*sum of the diagonal values: must be one!*/

    }
	
	if(verboseRate==1 || verboseQMatrix==1)
     fprintf(stderr, " Q(nk=0): sum(diagonal.values*pi):expected number of substitutions = %f\n", mr);

    return(Qij);
}


  /*******************************************************/
  /*RATE MATRIX n(k)=2 (DOUBLETT)                        */
  /*******************************************************/

  double **rateSH(){
    int i,j, ii,jj;
    double dig, rowsum[16];
    double **QSHij;
	 double **RSHij;
	double sumdipi;

    QSHij=(double**)malloc(16*sizeof(double*));
    for(i=0;i<16;i++)
       QSHij[i]=(double *)malloc(16*sizeof(double));
	   
	RSHij=(double**)malloc(16*sizeof(double*));
    for(i=0;i<16;i++)
       RSHij[i]=(double *)malloc(16*sizeof(double));   

    if(verboseQMatrix==1  || verboseFrequencies==1){
      fprintf(stderr, "\n");
      fprintf(stderr, "\t\t\t***MODEL-(nk=1)***\n");
      fprintf(stderr, "\n");
      if(nucleotide[3]=='t' || nucleotide[3]=='T'){
        fprintf(stderr, "frequencies:aa:%f\t ac:%f\t ag:%f\t at:%f\t\n",  dipi[0], dipi[1], dipi[2], dipi[3]);
        fprintf(stderr, "           :ca:%f\t cc:%f\t cg:%f\t ct:%f\t\n",  dipi[4], dipi[5], dipi[6], dipi[7]);
        fprintf(stderr, "           :ga:%f\t gc:%f\t gg:%f\t gt:%f\t\n",  dipi[8], dipi[9],dipi[10],dipi[11]);
        fprintf(stderr,"           :ta:%f\t tc:%f\t tg:%f\t tt:%f\t\n", dipi[12],dipi[13],dipi[14],dipi[15]);
      }else {
        fprintf(stderr, "frequencies:aa:%f\t ac:%f\t ag:%f\t au:%f\t\n",  dipi[0], dipi[1], dipi[2], dipi[3]);
        fprintf(stderr, "           :ca:%f\t cc:%f\t cg:%f\t cu:%f\t\n",  dipi[4], dipi[5], dipi[6], dipi[7]);
        fprintf(stderr, "           :ga:%f\t gc:%f\t gg:%f\t gu:%f\t\n",  dipi[8], dipi[9],dipi[10],dipi[11]);
        fprintf(stderr, "           :ua:%f\t uc:%f\t ug:%f\t uu:%f\t\n", dipi[12],dipi[13],dipi[14],dipi[15]);
      }
	 } 
	
	 sumdipi=0;
	 for(i=0;i<16;i++)
	 sumdipi+= dipi[i];	
	 if(verboseQMatrix==1  || verboseFrequencies==1) fprintf(stderr, "sum pi(nk=1) %f\n", sumdipi);
	 
	/* if(sumdipi!= 1.00000){ */
	   if((sumdipi < 1.00000 - NEARZERO) || (sumdipi > 1.00000 + NEARZERO)){
	    if(verboseQMatrix==1  || verboseFrequencies==1) fprintf(stderr, "new:\n");
		for(i=0;i<16;i++)
	          dipi[i] = dipi[i]/sumdipi;
		sumdipi=0;	
		for(i=0;i<16;i++)  
		     sumdipi+= dipi[i];	
	 
	    if(verboseQMatrix==1 || verboseFrequencies==1){
		  if(nucleotide[3]=='t' || nucleotide[3]=='T'){
             fprintf(stderr, "frequencies:aa:%f\t ac:%f\t ag:%f\t at:%f\t\n",  dipi[0], dipi[1], dipi[2], dipi[3]);
             fprintf(stderr, "           :ca:%f\t cc:%f\t cg:%f\t ct:%f\t\n",  dipi[4], dipi[5], dipi[6], dipi[7]);
			 fprintf(stderr, "           :ga:%f\t gc:%f\t gg:%f\t gt:%f\t\n",  dipi[8], dipi[9],dipi[10],dipi[11]);
             fprintf(stderr, "           :ta:%f\t tc:%f\t tg:%f\t tt:%f\t\n", dipi[12],dipi[13],dipi[14],dipi[15]);
		   }else {
             fprintf(stderr, "frequencies:aa:%f\t ac:%f\t ag:%f\t au:%f\t\n",  dipi[0], dipi[1], dipi[2], dipi[3]);
		  	 fprintf(stderr, "           :ca:%f\t cc:%f\t cg:%f\t cu:%f\t\n",  dipi[4], dipi[5], dipi[6], dipi[7]);
             fprintf(stderr, "           :ga:%f\t gc:%f\t gg:%f\t gu:%f\t\n",  dipi[8], dipi[9],dipi[10],dipi[11]);
             fprintf(stderr, "           :ua:%f\t uc:%f\t ug:%f\t uu:%f\t\n", dipi[12],dipi[13],dipi[14],dipi[15]);
		   }
		   fprintf(stderr, "sum pi(nk=1) %f\n", sumdipi);
	     }
		 
	  }	
	  
		  
	/*for(i=0;i<3;i++)
        for(j=i+1;j<4;j++)
          if(i*4+j !=11)
             Qij[i*4+j]=Qij[j*4+i]=Rmat[k++];     
    Qij[3*4+2]=Qij[2*4+3]=1.0;                    
	
	
	if(transitiontransverion==1 && prmat==0) {
	     K=2*trtv;
		 fprintf(stderr, "\n HKY Model or Kimura Model with k=%lf trtv=%lf\n", K, trtv);
	     Qij[0*4+2]=Qij[0*4+2]*K;
		 Qij[1*4+3]=Qij[1*4+3]*K;
		 Qij[2*4+0]=Qij[2*4+0]*K;
		 Qij[3*4+1]=Qij[3*4+1]*K;
	}*/
		  
		  
	for(i=0;i<16;i++)	  
			for(j=0;j<16;j++)	  
		       RSHij[i][j]=1.00;
		
	if(prmat==1 && prmat16==0){			  
	  for(i=0;i<16;i++)
      {
        ii=(i+4)/4;
        for(j=0;j<16;j++)
        {
        jj=(j+4)/4;
		 if((ii-1)%4==0 && (jj-1)%4==1) RSHij[i][j]=Rmat[0];
		 if((ii-1)%4==0 && (jj-1)%4==2) RSHij[i][j]=Rmat[1];
		 if((ii-1)%4==0 && (jj-1)%4==3) RSHij[i][j]=Rmat[2];
		 if((ii-1)%4==1 && (jj-1)%4==2) RSHij[i][j]=Rmat[3];
		 if((ii-1)%4==1 && (jj-1)%4==3) RSHij[i][j]=Rmat[4];
		 if((ii-1)%4==2 && (jj-1)%4==3) RSHij[i][j]=Rmat[5];
		 
		 if((ii-1)%4==1 && (jj-1)%4==0) RSHij[i][j]=Rmat[0];
		 if((ii-1)%4==2 && (jj-1)%4==0) RSHij[i][j]=Rmat[1];
		 if((ii-1)%4==3 && (jj-1)%4==0) RSHij[i][j]=Rmat[2];
		 if((ii-1)%4==2 && (jj-1)%4==1) RSHij[i][j]=Rmat[3];
		 if((ii-1)%4==3 && (jj-1)%4==1) RSHij[i][j]=Rmat[4];
		 if((ii-1)%4==3 && (jj-1)%4==2) RSHij[i][j]=Rmat[5];
		 if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )  /* hammingdistance==2 */
           RSHij[i][j]=0;
		  if((ii == jj)) RSHij[i][j]=0;
	    }
	   } 
	 }   	
	 
	 
	 if(prmat16==1){			  
	  for(i=0;i<16;i++)
      {
        ii=(i+4)/4;
        for(j=0;j<16;j++)
        {
        jj=(j+4)/4;
		/* if((ii-1)%4==0 && j==4) RSHij[i][j]=Rmat[0];
		 if((ii-1)%4==0 && j==8 ) RSHij[i][j]=Rmat[1];
		 if((ii-1)%4==0 && j==12) RSHij[i][j]=Rmat[2];
		 if((ii-1)%4==1 && j==8 ) RSHij[i][j]=Rmat[3];
		 if((ii-1)%4==1 && j==12) RSHij[i][j]=Rmat[4];
		 if((ii-1)%4==2 && j==12) RSHij[i][j]=Rmat[5];
		 
		 if((ii-1)%4==1 && j==0) RSHij[i][j]=Rmat[0];
		 if((ii-1)%4==2 && j==0) RSHij[i][j]=Rmat[1];
		 if((ii-1)%4==3 && j==0 RSHij[i][j]=Rmat[2];
		 if((ii-1)%4==2 && j==4) RSHij[i][j]=Rmat[3];
		 if((ii-1)%4==3 && j==4) RSHij[i][j]=Rmat[4];
		 if((ii-1)%4==3 && j==8) RSHij[i][j]=Rmat16[5];*/
		 if((i==0 && j==4))  RSHij[i][j]=Rmat16[0];
		 if((i==0 && j==8 )) RSHij[i][j]=Rmat16[1];
		 if((i==0 && j==12)) RSHij[i][j]=Rmat16[2];
		 if((i==4 && j==8 )) RSHij[i][j]=Rmat16[3];
		 if((i==4 && j==12)) RSHij[i][j]=Rmat16[4];
		 if((i==8 && j==12)) RSHij[i][j]=Rmat16[5];
		  if(i==4 && j==0) RSHij[i][j]=Rmat16[0];
		 if(i==8 && j==0) RSHij[i][j]=Rmat16[1];
		 if(i==12 && j==0) RSHij[i][j]=Rmat16[2];
		 if(i==8 && j==4) RSHij[i][j]=Rmat16[3];
		 if(i==12 && j==4) RSHij[i][j]=Rmat16[4];
		 if(i==12 && j==8) RSHij[i][j]=Rmat16[5];
		 
		 if((i==1 && j==5))  RSHij[i][j]=Rmat16[6];
		 if((i==1 && j==9 )) RSHij[i][j]=Rmat16[7];
		 if((i==1 && j==13)) RSHij[i][j]=Rmat16[8];
		 if((i==5 && j==9 )) RSHij[i][j]=Rmat16[9];
		 if((i==5 && j==13)) RSHij[i][j]=Rmat16[10];
		  if((i==9 && j==13)) RSHij[i][j]=Rmat16[11];
		 if(i==5 && j==1) RSHij[i][j]=Rmat16[6];
		 if(i==9 && j==1) RSHij[i][j]=Rmat16[7];
		 if(i==13 && j==1) RSHij[i][j]=Rmat16[8];
		 if(i==9 && j==5) RSHij[i][j]=Rmat16[9];
		 if(i==13 && j==5) RSHij[i][j]=Rmat16[10];
		 if(i==13 && j==9) RSHij[i][j]=Rmat16[11];
		 
		 if((i==2 && j==6))  RSHij[i][j]=Rmat16[12];
		 if((i==2 && j==10) ) RSHij[i][j]=Rmat16[13];
		 if((i==2 && j==14)) RSHij[i][j]=Rmat16[14];
		 if((i==6 && j==10)) RSHij[i][j]=Rmat16[15];
		 if((i==6 && j==14)) RSHij[i][j]=Rmat16[16];
		  if((i==10 && j==14)) RSHij[i][j]=Rmat16[17];
		 if(i==6 && j==2) RSHij[i][j]=Rmat16[12];
		 if(i==10 && j==2) RSHij[i][j]=Rmat16[13];
		 if(i==14 && j==2) RSHij[i][j]=Rmat16[14];
		 if(i==10 && j==6) RSHij[i][j]=Rmat16[15];
		 if(i==14 && j==6) RSHij[i][j]=Rmat16[16];
		 if(i==14 && j==10) RSHij[i][j]=Rmat16[17];
		 
		  if((i==3 && j==7))  RSHij[i][j]=Rmat16[18];
		 if((i==3 && j==11 )) RSHij[i][j]=Rmat16[19];
		 if((i==3 && j==15)) RSHij[i][j]=Rmat16[20];
		 if((i==7 && j==11)) RSHij[i][j]=Rmat16[21];
		 if((i==7 && j==15)) RSHij[i][j]=Rmat16[22];
		  if((i==11 && j==15)) RSHij[i][j]=Rmat16[23];
		 if(i==7 && j==3) RSHij[i][j]=Rmat16[18];
		 if(i==11 && j==3) RSHij[i][j]=Rmat16[19];
		 if(i==15 && j==3) RSHij[i][j]=Rmat16[20];
		 if(i==11 && j==7) RSHij[i][j]=Rmat16[21];
		 if(i==15 && j==7) RSHij[i][j]=Rmat16[22];
		 if(i==15 && j==11) RSHij[i][j]=Rmat16[23];


		 
		 
		 if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )  /* hammingdistance==2 */
           RSHij[i][j]=0;
		  if((ii == jj)) RSHij[i][j]=0;
	    }
	   } 
	 }   
	 
		
	if(transitiontransverion==1) {
	     K=2*trtv;
		  fprintf(stderr, "\nnk=1  HKY Model or Kimura Model with k=%lf trtv=%lf\n", K, trtv);
		 for(i=0;i<16;i++)
		  for(j=0;j<16;j++){
		
		if((i==0 && j==8 )) RSHij[i][j]=K*RSHij[i][j];
		 if((i==4 && j==12)) RSHij[i][j]=K*RSHij[i][j];
		 if(i==8 && j==0) RSHij[i][j]=K*RSHij[i][j];
		 if(i==12 && j==4) RSHij[i][j]=K*RSHij[i][j];
         if((i==1 && j==9 )) RSHij[i][j]=K*RSHij[i][j];
		 if((i==5 && j==13)) RSHij[i][j]=K*RSHij[i][j];
		 if(i==9 && j==1) RSHij[i][j]=K*RSHij[i][j];
		 if(i==13 && j==5) RSHij[i][j]=K*RSHij[i][j];
		 if((i==2 && j==10) ) RSHij[i][j]=K*RSHij[i][j];
	     if((i==6 && j==14)) RSHij[i][j]=K*RSHij[i][j];
		 if(i==10 && j==2) RSHij[i][j]=K*RSHij[i][j];
		 if(i==14 && j==6) RSHij[i][j]=K*RSHij[i][j];
         if((i==3 && j==11 )) RSHij[i][j]=K*RSHij[i][j];
		 if((i==7 && j==15)) RSHij[i][j]=K*RSHij[i][j];
		 if(i==11 && j==3) RSHij[i][j]=K*RSHij[i][j];
		 if(i==15 && j==7) RSHij[i][j]=K*RSHij[i][j];
	}}
	if(transitiontransverion==1 && prmat==1){
	   fprintf(stderr, "You cannot use TS/TV Parameter and the GTR model!!\n");
	   exit (0);
	}   	

	if(verboseQMatrix==1){
      fprintf(stderr, "\n");
	  fprintf(stderr, "\n");
	  fprintf(stderr, "  ||  ||");
      for(j=0; j<16; j++) fprintf(stderr, "%5s||",doublet[j]);
	    fprintf(stderr, "\n");
		  fprintf(stderr, "  ||  ||");
      for(j=0; j<16; j++) fprintf(stderr, "%5c||",nucleotide[((j+4)/4)-1]);
	    fprintf(stderr, "\n");
	
      for(i=0; i<16; i++){
          fprintf(stderr, "%2s||%c||",doublet[i],nucleotide[((i+4)/4)-1]);
          for(j=0; j<16; j++)
             fprintf(stderr, "%6.3f|",RSHij[i][j]);
             fprintf(stderr, "\n");
       }
    }	  
							  			  
								  		  
	for(i=0;i<16;i++)
    {
      ii=(i+4)/4;
      for(j=0;j<16;j++)
      {
        jj=(j+4)/4;
		if(mononucneigh1==1)  QSHij[i][j]=RSHij[i][j]*mpi[j/4];
        else  QSHij[i][j]=RSHij[i][j]*dipi[j];
        if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )  /* hammingdistance==2 */
           QSHij[i][j]=0;
        if(matrixSH==0){if((ii == jj)) QSHij[i][j]=0;}   /* Qk: hier is the difference to SH model*/
        if(i==j) QSHij[i][j]=0;                          /* diagonale entries zero        */
      }
    }


    dig=0;
    for(i=0; i<16; i++){
      rowsum[i]=0;
      for(j=0; j<16; j++)
         rowsum[i]=rowsum[i]+ QSHij[i][j];
      dig=dig+dipi[i]*rowsum[i];                        /*sum of diagonalentries multipli. with pi*/
    }

    for(i=0;i<16;i++)
      QSHij[i][i]= -1*rowsum[i];                         /*Diagonalentries*/


   if(verboseQMatrix==1)
      fprintf(stderr,"\nunnormalise Qk=1 (instantaneous substitution matrix):");

   if(verboseQMatrix==1){
      fprintf(stderr, "\n");
      for(i=0; i<16; i++){
          fprintf(stderr, "%2s||",doublet[i]);
          for(j=0; j<16; j++)
             fprintf(stderr, "%6.3f|",QSHij[i][j]);
             fprintf(stderr, "\n");
       }
    }	  
	
	if(verboseQMatrix==1) fprintf(stderr,"sum(diagonal.values*pi):expected number of substitutions = %f\n", dig);
	/*if(verboseQMatrix==1) fprintf(stderr,"dig*4=%f\n",dig*4);*/



	/*
	 * normalize the rate matrix
	 */
     for(i=0;i<16;i++)
        for(j=0;j<16;j++)
                QSHij[i][j] /= dig ;                     /*AB HIER PI*RATE -> NORMIERUNG*/
				
				
	if(verboseQMatrix==1)
      fprintf(stderr,"\nunnormalise Qk=1 (instantaneous substitution matrix):");			


      if(verboseQMatrix==1){

         fprintf(stderr,"\n");
         for(i=0; i<16; i++){
               fprintf(stderr, "%2s||",doublet[i]);
               for(j=0; j<16; j++)
               fprintf(stderr, "%6.3f|",QSHij[i][j]);
               fprintf(stderr, "\n");
         }
	 }	 

   
    dig=0;
    for (i=0; i<16; i++)
       dig -= dipi[i]*QSHij[i][i];      /*sum of the diagonal values*/
                                                                                                     
    if(verboseRate==1 || verboseQMatrix==1)
    fprintf(stderr, " Q(nk=1): sum(diagonal.values*pi):expected number of substitutions = %f\n", dig);
    if(verboseQMatrix==1) fprintf(stderr, "\n");
	
	/* if(verboseQMatrix==1){
	   digsum=0;
	   for(m=0; m<4; m++){ 
          fprintf(stderr,"\n");
		  fprintf(stderr,"Q%c\n", nucleotide[m]);
		  for(i=0; i<4; i++){
			   fprintf(stderr, "%2s||", doublet[(i*4)+m]);
			   for(j=0; j<4; j++)
                   fprintf(stderr, "%6.3f|",QSHij[(i*4)+m][(j*4)+m]);
               fprintf(stderr, "\n");
		  }
		  dig=0;
          for (i=0; i<4; i++)
          dig -= dipi[(i*4)+m]*QSHij[(i*4)+m][(i*4)+m];    
		  fprintf(stderr, "Q%c:  Sum of the diagonal-values*pi= expected number of substitutions: %f\n", m,dig);
          digsum= digsum +dig;
	   }
	   fprintf(stderr, "\nQD:  sum ofexpected number of substitutions= %f\n" ,digsum);	
	 }	*/ 

      free(RSHij);
	  return(QSHij);
	

   }
   
   
  /******************************************************************/
  /*                TRIPLET RATE MATRIX                             */
  /******************************************************************/
  double **rateT(){
    int i,j, ii,jj ,iii,jjj, k;
    double dig, rowsum[64];
    double **QTij, **RTij;
	double sumtripi;

    QTij=(double**)malloc(64*sizeof(double*));
    for(i=0;i<64;i++)
       QTij[i]=(double *)malloc(64*sizeof(double));
	   
    RTij=(double**)malloc(64*sizeof(double*));
     for(i=0;i<64;i++)
       RTij[i]=(double *)malloc(64*sizeof(double));

    for(i=0;i<64;i++)
	   for(j=0;j<64;j++) RTij[i][j]=1.00;
	   
	   dig=0.00;
	   
	/*******************************************************/
	/**/
	/******************************************************/   
	   
	   
	if(prmat==1 && prmat2t==0){			  
	  for(i=0;i<64;i++)
      {
        ii=(i+4)/4;
		iii=(ii+3)/4;
        for(j=0;j<64;j++)
        {
        jj=(j+4)/4;
		jjj=(jj+3)/4;
		 if(((i)%16)/4==0 && ((j)%16)/4==1) RTij[i][j]=Rmat[0]; /*A-C*/
		 if(((i)%16)/4==0 && ((j)%16)/4==2) RTij[i][j]=Rmat[1];
	 	 if(((i)%16)/4==0 && ((j)%16)/4==3) RTij[i][j]=Rmat[2];
		 if(((i)%16)/4==1 && ((j)%16)/4==2) RTij[i][j]=Rmat[3];
		 if(((i)%16)/4==1 && ((j)%16)/4==3) RTij[i][j]=Rmat[4];
		 if(((i)%16)/4==2 && ((j)%16)/4==3) RTij[i][j]=Rmat[5];
		 if(((i)%16)/4==1 && ((j)%16)/4==0) RTij[i][j]=Rmat[0];
		 if(((i)%16)/4==2 && ((j)%16)/4==0) RTij[i][j]=Rmat[1];
	 	 if(((i)%16)/4==3 && ((j)%16)/4==0) RTij[i][j]=Rmat[2];
		 if(((i)%16)/4==2 && ((j)%16)/4==1) RTij[i][j]=Rmat[3];
		 if(((i)%16)/4==3 && ((j)%16)/4==1) RTij[i][j]=Rmat[4];
		 if(((i)%16)/4==3 && ((j)%16)/4==2) RTij[i][j]=Rmat[5];
		 
		  if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )               /*Hammingdistance==2*/
           RTij[i][j]=0;
          if( (iii != jjj) && ( (ii-4*(iii-1))!=(jj -4*(jjj-1)) ) )     /*Hammingdistance==2*/
           RTij[i][j]=0;
          if(matrixSH==0){
		   /*  if(matrixSH==0){if((iii == jjj)) QTij[i][j]=0;} */
		    /* if( ((ii-1)-4*(iii-1) == (jj-1)-4*(jjj-1))  ) RTij[i][j]=0;}  */             /*expand model, only substitution on one position*/
		    if((iii == jjj)) RTij[i][j]=0;
		  }
		  if(i==j) RTij[i][j]=0.00;                                        /*diagonale*/
		 }
	   }
	   
}   	
	 


if(prmat2t==1){			  
   for(j=0;j<96;j++) fprintf(stderr, " rmat%d Rmat2T %lf\n", j,Rmat2T[j]);	 
	for(k=0;k<16;k++){
	    for(i=0;i<64;i++)
        {
        ii=(i+4)/4;
		iii=(ii+3)/4;
	    for(j=0;j<64;j++)
        {
        jj=(j+4)/4;
		jjj=(jj+3)/4;
		/*if( (k==(j%4)+(jjj-1)*4) && (k== i%4 + (iii-1)*4)){ */    /*2Pos*/
		if( (k==(j%4)+(jjj-1)*4) && (k== i%4 + (iii-1)*4)){     /*2Pos*/
		 fprintf(stderr, "not implemented at them moment \n");
		 exit(0);
		 /* fprintf(stderr, "\n k=%d j=%d %d=%s\n", k,j, (i%4 + (iii-1)*4), triplet[j]);*/
		 if(((i)%16)/4==0 && ((j)%16)/4==1) RTij[i][j]=Rmat2T[0+(k*6)];
		 if(((i)%16)/4==0 && ((j)%16)/4==2) RTij[i][j]=Rmat2T[1+(k*6)];
	 	 if(((i)%16)/4==0 && ((j)%16)/4==3) RTij[i][j]=Rmat2T[2+(k*6)];
		 if(((i)%16)/4==1 && ((j)%16)/4==2) RTij[i][j]=Rmat2T[3+(k*6)];
		 if(((i)%16)/4==1 && ((j)%16)/4==3) RTij[i][j]=Rmat2T[4+(k*6)];
		 if(((i)%16)/4==2 && ((j)%16)/4==3) RTij[i][j]=Rmat2T[5+(k*6)];
		 if(((i)%16)/4==1 && ((j)%16)/4==0) RTij[i][j]=Rmat2T[0+(k*6)];
		 if(((i)%16)/4==2 && ((j)%16)/4==0) RTij[i][j]=Rmat2T[1+(k*6)];
	 	 if(((i)%16)/4==3 && ((j)%16)/4==0) RTij[i][j]=Rmat2T[2+(k*6)];
		 if(((i)%16)/4==2 && ((j)%16)/4==1) RTij[i][j]=Rmat2T[3+(k*6)];
		 if(((i)%16)/4==3 && ((j)%16)/4==1) RTij[i][j]=Rmat2T[4+(k*6)];
		 if(((i)%16)/4==3 && ((j)%16)/4==2) RTij[i][j]=Rmat2T[5+(k*6)];
         }
		  if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )               /*Hammingdistance==2*/
           RTij[i][j]=0;
          if( (iii != jjj) && ( (ii-4*(iii-1))!=(jj -4*(jjj-1)) ) )     /*Hammingdistance==2*/
           RTij[i][j]=0;
          if(matrixSH==0){
		     /* if( ((ii-1)-4*(iii-1) == (jj-1)-4*(jjj-1))  ) RTij[i][j]=0;}   */            /*expand model, only substitution on one position*/
			    if((iii == jjj)) RTij[i][j]=0;}
			  if(i==j) RTij[i][j]=0.00;                                        /*diagonale*/
			}	
		}
	   } 
	 }   	
	 

		if(transitiontransverion==1) {
	     K=2*trtv;
		 fprintf(stderr, "\nhallo  HKY Model or Kimura Model with k=%lf trtv=%lf\n", K, trtv);
		 for(i=0;i<64;i++){
		   ii=(i+4)/4;
		   iii=(ii+3)/4;
		       for(j=0;j<64;j++){
		       jj=(j+4)/4;
		       jjj=(jj+3)/4;
	   /*  if(((i)%16)/4==0 && ((j)%16)/4==2) RTij[i][j]=K*RTij[i][j];
		 if(((i)%16)/4==1 && ((j)%16)/4==3) RTij[i][j]=K*RTij[i][j];
		 if(((i)%16)/4==2 && ((j)%16)/4==0) RTij[i][j]=K*RTij[i][j];
		 if(((i)%16)/4==3 && ((j)%16)/4==1) RTij[i][j]=K*RTij[i][j];*/
		  if(i/16==0 &&j/16==2){ RTij[i][j]=K*RTij[i][j];
		 /* fprintf(stderr, "%c -> %c %lf\n", nucleotide[i/16], nucleotide[j/16], K*RTij[i][j] );*/}
		 if(i/16==1  && j/16==3) RTij[i][j]=K*RTij[i][j];
		 if(i/16==2  && j/16==0) RTij[i][j]=K*RTij[i][j];
		 if(i/16==3  && j/16==1) RTij[i][j]=K*RTij[i][j];
   
		 if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )               
           RTij[i][j]=0;
		 if( (iii != jjj) && ( (ii-4*(iii-1))!=(jj -4*(jjj-1)) ) )   
           RTij[i][j]=0;
          if(matrixSH==0){
			  if((iii == jjj)) RTij[i][j]=0;}
		                /*expand model, only substitution on one position*/
		  if(i==j) RTij[i][j]=0.00;                 
	}}}
	if(transitiontransverion==1 && prmat==1){
	   fprintf(stderr, "You cannot use TS/TV Parameter and the GTR model!!\n");
	   exit (0);
	}   	 
	 
   	   
  if(verboseQMatrix==1){
     fprintf(stderr,"\n RT1 \n");
     fprintf(stderr,"Triplet2:dig=%f\n",dig);
     fprintf(stderr,"\n");
     fprintf(stderr,"        ");
     for(i=0; i<64; i++) fprintf(stderr,"%6s|",triplet[i]);
     fprintf(stderr,"\n");
	  fprintf(stderr,"        ");
	 for(i=0; i<64; i++) fprintf(stderr,"%6c|",nucleotide[((i)%16)/4]);
     fprintf(stderr,"\n");

     for(i=0; i<64; i++){
       fprintf(stderr,"%s||",triplet[i]);
	   fprintf(stderr,"%c||",nucleotide[((i)%16)/4]);
       for(j=0; j<64; j++)
       fprintf(stderr,"% .3f|",RTij[i][j]);
       fprintf(stderr,"\n");
     }
	 fprintf(stderr,"Triplet2: dig=%f\n",dig);
  }
     
	   
	/******************************************************/
	/*                                                    */
	/******************************************************/
	if(verboseQMatrix==1 || verboseFrequencies==1){

    fprintf(stderr, "\n");
    fprintf(stderr, "\t\t\t***MODEL-nk=2***First Postition\n");
    fprintf(stderr, "\n");
    if(nucleotide[3]=='t' || nucleotide[3]=='T'){
    fprintf(stderr, "frequencies:aaa:%f | aac:%f | aag:%f | aat:%f\t\n", tripi[0],  tripi[1],  tripi[2],  tripi[3]);
    fprintf(stderr,"           :aca:%f | acc:%f | acg:%f | act:%f\t\n",   tripi[4],  tripi[5],  tripi[6],  tripi[7]);
    fprintf(stderr,"           :aga:%f | agc:%f | agg:%f | agt:%f\t\n",   tripi[8],  tripi[9], tripi[10], tripi[11]);
    fprintf(stderr,"           :ata:%f | atc:%f | atg:%f | att:%f\t\n",  tripi[12], tripi[13], tripi[14], tripi[15]);
    fprintf(stderr,"           :caa:%f | cac:%f | cag:%f | cat:%f\t\n",  tripi[16], tripi[17], tripi[18], tripi[19]);
    fprintf(stderr,"           :cca:%f | ccc:%f | ccg:%f | cct:%f\t\n",  tripi[20], tripi[21], tripi[22], tripi[23]);
    fprintf(stderr,"           :cga:%f | cgc:%f | cgg:%f | cgt:%f\t\n",  tripi[24], tripi[25], tripi[26], tripi[27]);
    fprintf(stderr,"           :cta:%f | ctc:%f | ctg:%f | ctt:%f\t\n",  tripi[28], tripi[29], tripi[30], tripi[31]);
    fprintf(stderr,"           :gaa:%f | gac:%f | gag:%f | gat:%f\t\n",  tripi[32], tripi[33], tripi[34], tripi[35]);
    fprintf(stderr,"           :gca:%f | gcc:%f | gcg:%f | gct:%f\t\n",  tripi[36], tripi[37], tripi[38], tripi[39]);
    fprintf(stderr,"           :gga:%f | ggc:%f | ggg:%f | ggt:%f\t\n",  tripi[40], tripi[41], tripi[42], tripi[43]);
    fprintf(stderr,"           :gta:%f | gtc:%f | gtg:%f | gtt:%f\t\n",  tripi[44], tripi[45], tripi[46], tripi[47]);
    fprintf(stderr,"           :taa:%f | tac:%f | tag:%f | tat:%f\t\n",  tripi[48], tripi[49], tripi[50], tripi[51]);
    fprintf(stderr,"           :tca:%f | tcc:%f | tcg:%f | tct:%f\t\n",  tripi[52], tripi[53], tripi[54], tripi[55]);
    fprintf(stderr,"           :tga:%f | tgc:%f | tgg:%f | tgt:%f\t\n",  tripi[56], tripi[57], tripi[58], tripi[59]);
    fprintf(stderr,"           :tta:%f | ttc:%f | ttg:%f | ttt:%f\t\n",  tripi[60], tripi[61], tripi[62], tripi[63]);
    }else {
    fprintf(stderr,"           :aaa:%f aac:%f aag:%f aau:%f\t\n",   tripi[0],  tripi[1],  tripi[2],  tripi[3]);
    fprintf(stderr,"           :aca:%f acc:%f acg:%f acu:%f\t\n",   tripi[4],  tripi[5],  tripi[6],  tripi[7]);
    fprintf(stderr,"           :aga:%f agc:%f agg:%f agu:%f\t\n",   tripi[8],  tripi[9], tripi[10], tripi[11]);
    fprintf(stderr,"           :aua:%f auc:%f aug:%f auu:%f\t\n",  tripi[12], tripi[13], tripi[14], tripi[15]);
    fprintf(stderr,"           :caa:%f cac:%f cag:%f cau:%f\t\n",  tripi[16], tripi[17], tripi[18], tripi[19]);
    fprintf(stderr,"           :cca:%f ccc:%f ccg:%f ccu:%f\t\n",  tripi[20], tripi[21], tripi[22], tripi[23]);
    fprintf(stderr,"           :cga:%f cgc:%f cgg:%f cgu:%f\t\n",  tripi[24], tripi[25], tripi[26], tripi[27]);
    fprintf(stderr,"           :cua:%f cuc:%f cug:%f cuu:%f\t\n",  tripi[28], tripi[29], tripi[30], tripi[31]);
    fprintf(stderr,"           :gaa:%f gac:%f gag:%f gau:%f\t\n",  tripi[32], tripi[33], tripi[34], tripi[35]);
    fprintf(stderr,"           :gca:%f gcc:%f gcg:%f gcu:%f\t\n",  tripi[36], tripi[37], tripi[38], tripi[39]);
    fprintf(stderr,"           :gga:%f ggc:%f ggg:%f ggu:%f\t\n",  tripi[40], tripi[41], tripi[42], tripi[43]);
    fprintf(stderr,"           :gua:%f guc:%f gug:%f guu:%f\t\n",  tripi[44], tripi[45], tripi[46], tripi[47]);
    fprintf(stderr,"           :uaa:%f uac:%f uag:%f uau:%f\t\n",  tripi[48], tripi[49], tripi[50], tripi[51]);
    fprintf(stderr,"           :uca:%f ucc:%f ucg:%f ucu:%f\t\n",  tripi[52], tripi[53], tripi[54], tripi[55]);
    fprintf(stderr,"           :uga:%f ugc:%f ugg:%f ugu:%f\t\n",  tripi[56], tripi[57], tripi[58], tripi[59]);
    fprintf(stderr,"           :uua:%f uuc:%f uug:%f uuu:%f\t\n",  tripi[60], tripi[61], tripi[62], tripi[63]); 
    }
   }	
	
	sumtripi=0.00;
	for(i=0;i<64;i++)
	    sumtripi += tripi[i];
	if(verboseQMatrix==1 || verboseFrequencies==1)	fprintf(stderr, "sum pi(nk=1) %f\n", sumtripi);
	if((sumtripi < 1.00000 - NEARZERO) || (sumtripi > 1.00000 + NEARZERO)){	
	/*if(sumtripi != (double) 1.00){*/
	    if(verboseQMatrix==1 || verboseFrequencies==1) fprintf(stderr, "pi(nk=1) unequal 1.00: diff=%e :\n", 1.0 - sumtripi);
	    for(i=0;i<64;i++)
	        tripi[i] = tripi[i]/sumtripi;
		sumtripi=0.00;
		for(i=0;i<64;i++)
	    sumtripi += tripi[i];	
			
	if(verboseQMatrix==1 || verboseFrequencies==1){

    fprintf(stderr, "\n");
   /* fprintf(stderr, "\t\t\tQ_k2 instantaneous rate matrix\n");*/
    fprintf(stderr, "\n");
    if(nucleotide[3]=='t' || nucleotide[3]=='T'){
    fprintf(stderr, "frequencies:aaa:%f | aac:%f | aag:%f | aat:%f\t\n",  tripi[0],  tripi[1],  tripi[2],  tripi[3]);
    fprintf(stderr,"           :aca:%f | acc:%f | acg:%f | act:%f\t\n",   tripi[4],  tripi[5],  tripi[6],  tripi[7]);
    fprintf(stderr,"           :aga:%f | agc:%f | agg:%f | agt:%f\t\n",   tripi[8],  tripi[9], tripi[10], tripi[11]);
    fprintf(stderr,"           :ata:%f | atc:%f | atg:%f | att:%f\t\n",  tripi[12], tripi[13], tripi[14], tripi[15]);
    fprintf(stderr,"           :caa:%f | cac:%f | cag:%f | cat:%f\t\n",  tripi[16], tripi[17], tripi[18], tripi[19]);
    fprintf(stderr,"           :cca:%f | ccc:%f | ccg:%f | cct:%f\t\n",  tripi[20], tripi[21], tripi[22], tripi[23]);
    fprintf(stderr,"           :cga:%f | cgc:%f | cgg:%f | cgt:%f\t\n",  tripi[24], tripi[25], tripi[26], tripi[27]);
    fprintf(stderr,"           :cta:%f | ctc:%f | ctg:%f | ctt:%f\t\n",  tripi[28], tripi[29], tripi[30], tripi[31]);
    fprintf(stderr,"           :gaa:%f | gac:%f | gag:%f | gat:%f\t\n",  tripi[32], tripi[33], tripi[34], tripi[35]);
    fprintf(stderr,"           :gca:%f | gcc:%f | gcg:%f | gct:%f\t\n",  tripi[36], tripi[37], tripi[38], tripi[39]);
    fprintf(stderr,"           :gga:%f | ggc:%f | ggg:%f | ggt:%f\t\n",  tripi[40], tripi[41], tripi[42], tripi[43]);
    fprintf(stderr,"           :gta:%f | gtc:%f | gtg:%f | gtt:%f\t\n",  tripi[44], tripi[45], tripi[46], tripi[47]);
    fprintf(stderr,"           :taa:%f | tac:%f | tag:%f | tat:%f\t\n",  tripi[48], tripi[49], tripi[50], tripi[51]);
    fprintf(stderr,"           :tca:%f | tcc:%f | tcg:%f | tct:%f\t\n",  tripi[52], tripi[53], tripi[54], tripi[55]);
    fprintf(stderr,"           :tga:%f | tgc:%f | tgg:%f | tgt:%f\t\n",  tripi[56], tripi[57], tripi[58], tripi[59]);
    fprintf(stderr,"           :tta:%f | ttc:%f | ttg:%f | ttt:%f\t\n",  tripi[60], tripi[61], tripi[62], tripi[63]);
    }else {
    fprintf(stderr,"           :aaa:%f aac:%f aag:%f aau:%f\t\n",   tripi[0],  tripi[1],  tripi[2],  tripi[3]);
    fprintf(stderr,"           :aca:%f acc:%f acg:%f acu:%f\t\n",   tripi[4],  tripi[5],  tripi[6],  tripi[7]);
    fprintf(stderr,"           :aga:%f agc:%f agg:%f agu:%f\t\n",   tripi[8],  tripi[9], tripi[10], tripi[11]);
    fprintf(stderr,"           :aua:%f auc:%f aug:%f auu:%f\t\n",  tripi[12], tripi[13], tripi[14], tripi[15]);
    fprintf(stderr,"           :caa:%f cac:%f cag:%f cau:%f\t\n",  tripi[16], tripi[17], tripi[18], tripi[19]);
    fprintf(stderr,"           :cca:%f ccc:%f ccg:%f ccu:%f\t\n",  tripi[20], tripi[21], tripi[22], tripi[23]);
    fprintf(stderr,"           :cga:%f cgc:%f cgg:%f cgu:%f\t\n",  tripi[24], tripi[25], tripi[26], tripi[27]);
    fprintf(stderr,"           :cua:%f cuc:%f cug:%f cuu:%f\t\n",  tripi[28], tripi[29], tripi[30], tripi[31]);
    fprintf(stderr,"           :gaa:%f gac:%f gag:%f gau:%f\t\n",  tripi[32], tripi[33], tripi[34], tripi[35]);
    fprintf(stderr,"           :gca:%f gcc:%f gcg:%f gcu:%f\t\n",  tripi[36], tripi[37], tripi[38], tripi[39]);
    fprintf(stderr,"           :gga:%f ggc:%f ggg:%f ggu:%f\t\n",  tripi[40], tripi[41], tripi[42], tripi[43]);
    fprintf(stderr,"           :gua:%f guc:%f gug:%f guu:%f\t\n",  tripi[44], tripi[45], tripi[46], tripi[47]);
    fprintf(stderr,"           :uaa:%f uac:%f uag:%f uau:%f\t\n",  tripi[48], tripi[49], tripi[50], tripi[51]);
    fprintf(stderr,"           :uca:%f ucc:%f ucg:%f ucu:%f\t\n",  tripi[52], tripi[53], tripi[54], tripi[55]);
    fprintf(stderr,"           :uga:%f ugc:%f ugg:%f ugu:%f\t\n",  tripi[56], tripi[57], tripi[58], tripi[59]);
    fprintf(stderr,"           :uua:%f uuc:%f uug:%f uuu:%f\t\n",  tripi[60], tripi[61], tripi[62], tripi[63]); 
    }
	if(verboseQMatrix==1 || verboseFrequencies==1) fprintf(stderr, "sum pi(nk=1) %f\n", sumtripi);
	
    }	
	sumtripi=0.00;
	    for(i=0;i<64;i++)
	         sumtripi += tripi[i];
		if(verboseQMatrix==1) {
		   fprintf(stderr, "pi(nk=2) are not 1.00: diff=%e :\n", 1.0 - sumtripi);	 
	     if((sumtripi < 1.00000 - NEARZERO) || (sumtripi > 1.00000 + NEARZERO))
	           fprintf(stderr, "sum ofpi(nk=2) : %f\n", sumtripi); 
		}   
	}	  
	
    for(i=0;i<64;i++)
    {
      ii=(i+4)/4;
      iii=(ii+3)/4;
      for(j=0;j<64;j++)
      {
        jj=(j+4)/4;
        jjj=(jj+3)/4;
		if (mononucneigh2==1) QTij[i][j]=mpi[j/16]*RTij[i][j];
        else QTij[i][j]=tripi[j]*RTij[i][j];
        if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )               /*Hammingdistance==2*/
           QTij[i][j]=0;
        if( (iii != jjj) && ( (ii-4*(iii-1))!=(jj -4*(jjj-1)) ) )     /*Hammingdistance==2*/
           QTij[i][j]=0;
        if(matrixSH==0){if((iii == jjj)) QTij[i][j]=0;}               /*expand model, only substitution on one position*/
        if(i==j) QTij[i][j]=0.00;                                     /*diagonale*/
      }
    }
    
	dig=0.00;
    for(i=0; i<64; i++){
      rowsum[i]=0.00;
      for(j=0; j<64; j++){
         if(i!=j)  rowsum[i]=rowsum[i]+ QTij[i][j];}
	  QTij[i][i]= -1*rowsum[i];                            /*Diagonalentries*/ 
      dig=dig + tripi[i]*rowsum[i];                        /*Diagonalentries multipli. with the rowsum together*/
    }                                                   

    if(verboseQMatrix==1){
    fprintf(stderr,"Triplet:dig=%f\n",dig);
     fprintf(stderr,"\n");
     fprintf(stderr,"     ");
     for(i=0; i<64; i++) fprintf(stderr,"%6s|",triplet[i]);
     fprintf(stderr,"\n");
	 
 
     for(i=0; i<64; i++){
       fprintf(stderr,"%s||",triplet[i]);
       for(j=0; j<64; j++)
       fprintf(stderr,"% .3f|",QTij[i][j]);
       fprintf(stderr,"\n");
     }
	 fprintf(stderr,"Triplet: dig=%f\n",dig);
   }
    
   for(i=0;i<64;i++)
        for(j=0;j<64;j++)
                QTij[i][j] /= dig ;                     /*AB HIER PI*RATE -> NORMIERUNG*/
				
   if(verboseQMatrix==1){
     fprintf(stderr,"hier\n");
     fprintf(stderr,"     ");
     for(i=0; i<64; i++) fprintf(stderr,"%6s|",triplet[i]);
     fprintf(stderr,"\n");

     for(i=0; i<64; i++){
       fprintf(stderr,"%s||",triplet[i]);
       for(j=0; j<64; j++){
	    /* if(QTij[i][j]==0.00) fprintf(stderr,"%  - |",QTij[i][j]);
         else */
		 fprintf(stderr,"% .3f|",QTij[i][j]);
	   }
       fprintf(stderr,"\n");
     }
   }

    dig=0;
    for (i=0; i<64; i++)
       dig -= tripi[i]*QTij[i][i];      /*sum of the diagonal values*/

    if(verboseRate==1 || verboseQMatrix==1)
       fprintf(stderr, "Q1(nk=2): sum(diagonal.values*pi):expected number of substitutions = %f\n", dig);
	  
	return(QTij);
}
   
   
   
  /******************************************************************/
  /*      TRIPLET RATE MATRIX Cat 2 und Overlapping Dinucl.         */
  /******************************************************************/   
   
   
double **rateT2(){
    int i,j, ii,jj ,iii,jjj,k;
    double dig, rowsum[64];
    double **QT2ij,**RT2ij ;
	/*double sumtripi;*/

    QT2ij=(double**)malloc(64*sizeof(double*));
    for(i=0;i<64;i++)
       QT2ij[i]=(double *)malloc(64*sizeof(double));
	   
	RT2ij=(double**)malloc(64*sizeof(double*));
    for(i=0;i<64;i++)
       RT2ij[i]=(double *)malloc(64*sizeof(double));

    for(i=0;i<64;i++)
	   for(j=0;j<64;j++) RT2ij[i][j]=1.00;
	   
	   
     dig=0.00;
	   
	if(prmat==1 && prmat2t==0){			  
	  for(i=0;i<64;i++)
      {
        ii=(i+4)/4;
		iii=(ii+3)/4;
        for(j=0;j<64;j++)
        {
        jj=(j+4)/4;
		jjj=(jj+3)/4;
		 if(((i)%16)/4==0 && ((j)%16)/4==1) RT2ij[i][j]=Rmat[0];
		 if(((i)%16)/4==0 && ((j)%16)/4==2) RT2ij[i][j]=Rmat[1];
	 	 if(((i)%16)/4==0 && ((j)%16)/4==3) RT2ij[i][j]=Rmat[2];
		 if(((i)%16)/4==1 && ((j)%16)/4==2) RT2ij[i][j]=Rmat[3];
		 if(((i)%16)/4==1 && ((j)%16)/4==3) RT2ij[i][j]=Rmat[4];
		 if(((i)%16)/4==2 && ((j)%16)/4==3) RT2ij[i][j]=Rmat[5];
		 if(((i)%16)/4==1 && ((j)%16)/4==0) RT2ij[i][j]=Rmat[0];
		 if(((i)%16)/4==2 && ((j)%16)/4==0) RT2ij[i][j]=Rmat[1];
	 	 if(((i)%16)/4==3 && ((j)%16)/4==0) RT2ij[i][j]=Rmat[2];
		 if(((i)%16)/4==2 && ((j)%16)/4==1) RT2ij[i][j]=Rmat[3];
		 if(((i)%16)/4==3 && ((j)%16)/4==1) RT2ij[i][j]=Rmat[4];
		 if(((i)%16)/4==3 && ((j)%16)/4==2) RT2ij[i][j]=Rmat[5];
		 
		  if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )               /*Hammingdistance==2*/
           RT2ij[i][j]=0;
          if( (iii != jjj) && ( (ii-4*(iii-1))!=(jj -4*(jjj-1)) ) )     /*Hammingdistance==2*/
           RT2ij[i][j]=0;
          if(matrixSH==0){
		     if( ((ii-1)-4*(iii-1) == (jj-1)-4*(jjj-1))  ) RT2ij[i][j]=0;}               /*expand model, only substitution on one position*/
		  if(i==j) RT2ij[i][j]=0.00;                                        /*diagonale*/
		 }
	   }
	   
}   	
	 


if(prmat2t==1){			  
   for(j=0;j<96;j++) fprintf(stderr, " rmat%d Rmat2T %lf\n", j,Rmat2T[j]);	 
	for(k=0;k<16;k++){
	    for(i=0;i<64;i++)
        {
        ii=(i+4)/4;
		iii=(ii+3)/4;
	    for(j=0;j<64;j++)
        {
        jj=(j+4)/4;
		jjj=(jj+3)/4;
		if( (k==(j%4)+(jjj-1)*4) && (k== i%4 + (iii-1)*4)){
		 /* fprintf(stderr, "\n k=%d j=%d %d=%s\n", k,j, (i%4 + (iii-1)*4), triplet[j]);*/
		 if(((i)%16)/4==0 && ((j)%16)/4==1) RT2ij[i][j]=Rmat2T[0+(k*6)];
		 if(((i)%16)/4==0 && ((j)%16)/4==2) RT2ij[i][j]=Rmat2T[1+(k*6)];
	 	 if(((i)%16)/4==0 && ((j)%16)/4==3) RT2ij[i][j]=Rmat2T[2+(k*6)];
		 if(((i)%16)/4==1 && ((j)%16)/4==2) RT2ij[i][j]=Rmat2T[3+(k*6)];
		 if(((i)%16)/4==1 && ((j)%16)/4==3) RT2ij[i][j]=Rmat2T[4+(k*6)];
		 if(((i)%16)/4==2 && ((j)%16)/4==3) RT2ij[i][j]=Rmat2T[5+(k*6)];
		 if(((i)%16)/4==1 && ((j)%16)/4==0) RT2ij[i][j]=Rmat2T[0+(k*6)];
		 if(((i)%16)/4==2 && ((j)%16)/4==0) RT2ij[i][j]=Rmat2T[1+(k*6)];
	 	 if(((i)%16)/4==3 && ((j)%16)/4==0) RT2ij[i][j]=Rmat2T[2+(k*6)];
		 if(((i)%16)/4==2 && ((j)%16)/4==1) RT2ij[i][j]=Rmat2T[3+(k*6)];
		 if(((i)%16)/4==3 && ((j)%16)/4==1) RT2ij[i][j]=Rmat2T[4+(k*6)];
		 if(((i)%16)/4==3 && ((j)%16)/4==2) RT2ij[i][j]=Rmat2T[5+(k*6)];
         }
		  if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )               /*Hammingdistance==2*/
           RT2ij[i][j]=0;
          if( (iii != jjj) && ( (ii-4*(iii-1))!=(jj -4*(jjj-1)) ) )     /*Hammingdistance==2*/
           RT2ij[i][j]=0;
          if(matrixSH==0){
		      if( ((ii-1)-4*(iii-1) == (jj-1)-4*(jjj-1))  ) RT2ij[i][j]=0;}               /*expand model, only substitution on one position*/
			  if(i==j) RT2ij[i][j]=0.00;                                        /*diagonale*/
			}	
		}
	   } 
	 }   	
	 

		if(transitiontransverion==1) {
	     K=2*trtv;
		 fprintf(stderr, "\nhallo  HKY Model or Kimura Model with k=%lf trtv=%lf\n", K, trtv);
		 for(i=0;i<64;i++){ /* wieso vorher 16?? 0312*/
		   ii=(i+4)/4;
		   iii=(ii+3)/4;
		       for(j=0;j<64;j++){
		       jj=(j+4)/4;
		       jjj=(jj+3)/4;
	     if(((i)%16)/4==0 && ((j)%16)/4==2) RT2ij[i][j]=K*RT2ij[i][j];
		 if(((i)%16)/4==1 && ((j)%16)/4==3) RT2ij[i][j]=K*RT2ij[i][j];
		 if(((i)%16)/4==2 && ((j)%16)/4==0) RT2ij[i][j]=K*RT2ij[i][j];
		 if(((i)%16)/4==3 && ((j)%16)/4==1) RT2ij[i][j]=K*RT2ij[i][j];
   
		 if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )               /*Hammingdistance==2*/
           RT2ij[i][j]=0;
		 if( (iii != jjj) && ( (ii-4*(iii-1))!=(jj -4*(jjj-1)) ) )     /*Hammingdistance==2*/
           RT2ij[i][j]=0;
          if(matrixSH==0){
		     if( ((ii-1)-4*(iii-1) == (jj-1)-4*(jjj-1))  ) RT2ij[i][j]=0;}               /*expand model, only substitution on one position*/
		  if(i==j) RT2ij[i][j]=0.00;                 
	}}}
	if(transitiontransverion==1 && prmat==1){
	   fprintf(stderr, "You cannot use TS/TV Parameter and the GTR model!!\n");
	   exit (0);
	}   	 
	 
  if(CpG==1){
		     /*CpG->CpA: C->T; NCG->NTG*/
			 RT2ij[6][14]=CpGCpA;
			 RT2ij[22][30]=CpGCpA;
			 RT2ij[38][46]=CpGCpA;
			 RT2ij[54][62]=CpGCpA;
			 /*CpG->CpA; G->A; CGN->CAN */
			 RT2ij[24][16]=CpGTpG;
			 RT2ij[25][17]=CpGTpG;
			 RT2ij[26][18]=CpGTpG;
			 RT2ij[27][19]=CpGTpG;
			 /********************/
			  fprintf(stderr, "\n%s %s %lf\n",triplet[6],  triplet[14],CpGCpA);
			  fprintf(stderr, "%s %s %lf\n",  triplet[22], triplet[30],CpGCpA);
			  fprintf(stderr, "%s %s %lf\n",  triplet[38], triplet[46],CpGCpA);
			  fprintf(stderr, "%s %s %lf\n",  triplet[54], triplet[62],CpGCpA);
			  fprintf(stderr, "%s %s %lf\n",  triplet[24], triplet[16],CpGTpG);
			  fprintf(stderr, "%s %s %lf\n",  triplet[25], triplet[17],CpGTpG);
			  fprintf(stderr, "%s %s %lf\n",  triplet[26], triplet[18],CpGTpG);
			  fprintf(stderr, "%s %s %lf\n",  triplet[27], triplet[19],CpGTpG);
   }	 
 		   
	   
  if(verboseQMatrix==1){
     fprintf(stderr,"\n RT2 \n");
     fprintf(stderr,"Triplet2:dig=%f\n",dig);
     fprintf(stderr,"\n");
     fprintf(stderr,"        ");
     for(i=0; i<64; i++) fprintf(stderr,"%6s|",triplet[i]);
     fprintf(stderr,"\n");
	  fprintf(stderr,"        ");
	 for(i=0; i<64; i++) fprintf(stderr,"%6c|",nucleotide[((i)%16)/4]);
     fprintf(stderr,"\n");

     for(i=0; i<64; i++){
       fprintf(stderr,"%s||",triplet[i]);
	   fprintf(stderr,"%c||",nucleotide[((i)%16)/4]);
       for(j=0; j<64; j++)
       fprintf(stderr,"% .3f|",RT2ij[i][j]);
       fprintf(stderr,"\n");
     }
	 fprintf(stderr,"Triplet2: dig=%f\n",dig);
  }
  

  
 for(i=0;i<64;i++)
    {
      ii=(i+4)/4;
      iii=(ii+3)/4;
      for(j=0;j<64;j++)
      {
        jj=(j+4)/4;
        jjj=(jj+3)/4;
		if(mononucneigh2==1) QT2ij[i][j]=mpi[((j)%16)/4]*RT2ij[i][j];
        else QT2ij[i][j]=tripi[j]*RT2ij[i][j];
        if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )               /*Hammingdistance==2*/
           QT2ij[i][j]=0;
        if( (iii != jjj) && ( (ii-4*(iii-1))!=(jj -4*(jjj-1)) ) )     /*Hammingdistance==2*/
           QT2ij[i][j]=0;
        if(matrixSH==0){
		     if( ((ii-1)-4*(iii-1) == (jj-1)-4*(jjj-1))  ) QT2ij[i][j]=0;}               /*expand model, only substitution on one position*/
        if(i==j) QT2ij[i][j]=0.00;                                        /*diagonale*/
      }
    }

   dig=0.00;
   for(i=0; i<64; i++){
      rowsum[i]=0.00;
      for(j=0; j<64; j++){
         if(i!=j)  rowsum[i]=rowsum[i]+ QT2ij[i][j];}
	  QT2ij[i][i]= -1*rowsum[i];                           /*Diagonalentries*/ 
      dig=dig + tripi[i]*rowsum[i];                        /*Diagonalentries multipli. with the rowsum together*/
    }                                                   

  if(verboseQMatrix==1){
     fprintf(stderr,"\n QT2 \n");
     fprintf(stderr,"Triplet2:dig=%f\n",dig);
     fprintf(stderr,"\n");
     fprintf(stderr,"     ");
     for(i=0; i<64; i++) fprintf(stderr,"%6s|",triplet[i]);
     fprintf(stderr,"\n");
	 fprintf(stderr,"     ");
	 for(i=0; i<64; i++) fprintf(stderr,"%6s|",triplet[i]);
     fprintf(stderr,"\n");
	 fprintf(stderr,"     ");
	 for(i=0; i<64; i++) fprintf(stderr,"%6c|",nucleotide[((i)%16)/4]);
	 fprintf(stderr,"\n");
		 
     for(i=0; i<64; i++){
       fprintf(stderr,"%s||",triplet[i]);
       for(j=0; j<64; j++)
       fprintf(stderr,"% .3f|",QT2ij[i][j]);
       fprintf(stderr,"\n");
     }
	 fprintf(stderr,"Triplet2: dig=%f\n",dig);
  }
    
  for(i=0;i<64;i++)
        for(j=0;j<64;j++)
                QT2ij[i][j] /= dig ;                     /*AB HIER PI*RATE -> NORMIERUNG*/

  if(verboseQMatrix==1){
     fprintf(stderr,"\n");
	 fprintf(stderr,"\n QT2 \n");
     fprintf(stderr,"     ");
     for(i=0; i<64; i++) fprintf(stderr,"%6s|",triplet[i]);
     fprintf(stderr,"\n");

     for(i=0; i<64; i++){
       fprintf(stderr,"%s||",triplet[i]);
       for(j=0; j<64; j++)
       fprintf(stderr,"% .3f|",QT2ij[i][j]);
       fprintf(stderr,"\n");
     }
   }

    dig=0;
    for (i=0; i<64; i++)
       dig -= tripi[i]*QT2ij[i][i];      /*sum of the diagonal values*/

    if(verboseRate==1 || verboseQMatrix==1)
        fprintf(stderr, "Q2(nk=2): sum(diagonal.values*pi):expected number of substitutions = %f\n", dig);
	
	free(RT2ij);
	return(QT2ij);
}
 

  /******************************************************************/
  /*      TRIPLET RATE MATRIX Cat 2 und Overlapping Dinucl.         */
  /******************************************************************/      
	 
	     
double **rateT3(){
    int i,j, ii,jj ,iii,jjj,k;
    double dig, rowsum[64];
    double **QT3ij,**RT3ij;
	
    QT3ij=(double**)malloc(64*sizeof(double*));
    for(i=0;i<64;i++)
       QT3ij[i]=(double *)malloc(64*sizeof(double));
	 
	   dig=0.00;  
	/****************************************************************/   
	   
	   
	RT3ij=(double**)malloc(64*sizeof(double*));
    for(i=0;i<64;i++)
       RT3ij[i]=(double *)malloc(64*sizeof(double));

    for(i=0;i<64;i++)
	   for(j=0;j<64;j++) RT3ij[i][j]=1.00;
	   
	   
    
	   
	if(prmat==1 && prmat2t==0){			  
	  for(i=0;i<64;i++)
      {
        ii=(i+4)/4;
		iii=(ii+3)/4;
        for(j=0;j<64;j++)
        {
        jj=(j+4)/4;
		jjj=(jj+3)/4;
		 if(((i)%16)/4==0 && ((j)%16)/4==1) RT3ij[i][j]=Rmat[0];
		 if(((i)%16)/4==0 && ((j)%16)/4==2) RT3ij[i][j]=Rmat[1];
	 	 if(((i)%16)/4==0 && ((j)%16)/4==3) RT3ij[i][j]=Rmat[2];
		 if(((i)%16)/4==1 && ((j)%16)/4==2) RT3ij[i][j]=Rmat[3];
		 if(((i)%16)/4==1 && ((j)%16)/4==3) RT3ij[i][j]=Rmat[4];
		 if(((i)%16)/4==2 && ((j)%16)/4==3) RT3ij[i][j]=Rmat[5];
		 if(((i)%16)/4==1 && ((j)%16)/4==0) RT3ij[i][j]=Rmat[0];
		 if(((i)%16)/4==2 && ((j)%16)/4==0) RT3ij[i][j]=Rmat[1];
	 	 if(((i)%16)/4==3 && ((j)%16)/4==0) RT3ij[i][j]=Rmat[2];
		 if(((i)%16)/4==2 && ((j)%16)/4==1) RT3ij[i][j]=Rmat[3];
		 if(((i)%16)/4==3 && ((j)%16)/4==1) RT3ij[i][j]=Rmat[4];
		 if(((i)%16)/4==3 && ((j)%16)/4==2) RT3ij[i][j]=Rmat[5];
		 
		  if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )               /*Hammingdistance==2*/
           RT3ij[i][j]=0;
          if( (iii != jjj) && ( (ii-4*(iii-1))!=(jj -4*(jjj-1)) ) )     /*Hammingdistance==2*/
           RT3ij[i][j]=0;
          if(matrixSH==0){
		     if(  i-4*(ii-1) ==  j-4*(jj-1)   ) RT3ij[i][j]=0;}               /*expand model, only substitution on one position*/
		  if(i==j) RT3ij[i][j]=0.00;                                        /*diagonale*/
		 }
	   }
	   
}   	
	 


if(prmat2t==1){			  
   for(j=0;j<96;j++) fprintf(stderr, " rmat%d Rmat2T %lf\n", j,Rmat2T[j]);	 
	for(k=0;k<16;k++){
	    for(i=0;i<64;i++)
        {
        ii=(i+4)/4;
		iii=(ii+3)/4;
	    for(j=0;j<64;j++)
        {
        jj=(j+4)/4;
		jjj=(jj+3)/4;
		if( (k==(j%4)+(jjj-1)*4) && (k== i%4 + (iii-1)*4)){
		 /* fprintf(stderr, "\n k=%d j=%d %d=%s\n", k,j, (i%4 + (iii-1)*4), triplet[j]);*/
		 fprintf(stderr, "not implemented at the moment, has to debug!\n");
		 exit (0);
		 
		 if(((i)%16)/4==0 && ((j)%16)/4==1) RT3ij[i][j]=Rmat2T[0+(k*6)];
		 if(((i)%16)/4==0 && ((j)%16)/4==2) RT3ij[i][j]=Rmat2T[1+(k*6)];
	 	 if(((i)%16)/4==0 && ((j)%16)/4==3) RT3ij[i][j]=Rmat2T[2+(k*6)];
		 if(((i)%16)/4==1 && ((j)%16)/4==2) RT3ij[i][j]=Rmat2T[3+(k*6)];
		 if(((i)%16)/4==1 && ((j)%16)/4==3) RT3ij[i][j]=Rmat2T[4+(k*6)];
		 if(((i)%16)/4==2 && ((j)%16)/4==3) RT3ij[i][j]=Rmat2T[5+(k*6)];
		 if(((i)%16)/4==1 && ((j)%16)/4==0) RT3ij[i][j]=Rmat2T[0+(k*6)];
		 if(((i)%16)/4==2 && ((j)%16)/4==0) RT3ij[i][j]=Rmat2T[1+(k*6)];
	 	 if(((i)%16)/4==3 && ((j)%16)/4==0) RT3ij[i][j]=Rmat2T[2+(k*6)];
		 if(((i)%16)/4==2 && ((j)%16)/4==1) RT3ij[i][j]=Rmat2T[3+(k*6)];
		 if(((i)%16)/4==3 && ((j)%16)/4==1) RT3ij[i][j]=Rmat2T[4+(k*6)];
		 if(((i)%16)/4==3 && ((j)%16)/4==2) RT3ij[i][j]=Rmat2T[5+(k*6)];
         }
		  if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )               /*Hammingdistance==2*/
           RT3ij[i][j]=0;
          if( (iii != jjj) && ( (ii-4*(iii-1))!=(jj -4*(jjj-1)) ) )     /*Hammingdistance==2*/
           RT3ij[i][j]=0;
          if(matrixSH==0){
		      if(  i-4*(ii-1) ==  j-4*(jj-1)   ) RT3ij[i][j]=0;}               /*expand model, only substitution on one position*/
			  if(i==j) RT3ij[i][j]=0.00;                                        /*diagonale*/
			}	
		}
	   } 
	 }   	
	 

		if(transitiontransverion==1) {
	     K=2*trtv;
		 fprintf(stderr, "\nhallo  HKY Model or Kimura Model with k=%lf trtv=%lf\n", K, trtv);
		 for(i=0;i<64;i++){
		   ii=(i+4)/4;
		   iii=(ii+3)/4;
		       for(j=0;j<64;j++){
		       jj=(j+4)/4;
		       jjj=(jj+3)/4;
	     if(((i)%16)%4==0 && ((j)%16)%4==2) RT3ij[i][j]=K*RT3ij[i][j];
		 if(((i)%16)%4==1 && ((j)%16)%4==3) RT3ij[i][j]=K*RT3ij[i][j];
		 if(((i)%16)%4==2 && ((j)%16)%4==0) RT3ij[i][j]=K*RT3ij[i][j];
		 if(((i)%16)%4==3 && ((j)%16)%4==1) RT3ij[i][j]=K*RT3ij[i][j];
   
		 if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )               /*Hammingdistance==2*/
           RT3ij[i][j]=0;
		 if( (iii != jjj) && ( (ii-4*(iii-1))!=(jj -4*(jjj-1)) ) )     /*Hammingdistance==2*/
           RT3ij[i][j]=0;
          if(matrixSH==0){
		     if(  i-4*(ii-1) ==  j-4*(jj-1)  ) RT3ij[i][j]=0;}               /*expand model, only substitution on one position*/
		  if(i==j) RT3ij[i][j]=0.00;                 
	}}}
	if(transitiontransverion==1 && prmat==1){
	   fprintf(stderr, "You cannot use TS/TV Parameter and the GTR model!!\n");
	   exit (0);
	}   	 
	 
   if(verboseQMatrix==1){
     fprintf(stderr,"\n RT3 \n");
     fprintf(stderr,"Triplet2:dig=%f\n",dig);
     fprintf(stderr,"\n");
     fprintf(stderr,"        ");
     for(i=0; i<64; i++) fprintf(stderr,"%6s|",triplet[i]);
     fprintf(stderr,"\n");
	  fprintf(stderr,"        ");
	 for(i=0; i<64; i++) fprintf(stderr,"%6c|",nucleotide[((i)%16)/4]);
     fprintf(stderr,"\n");

     for(i=0; i<64; i++){
       fprintf(stderr,"%s||",triplet[i]);
	   fprintf(stderr,"%c||",nucleotide[((i)%16)/4]);
       for(j=0; j<64; j++)
       fprintf(stderr,"% .3f|",RT3ij[i][j]);
       fprintf(stderr,"\n");
     }
	 fprintf(stderr,"Triplet2: dig=%f\n",dig);
  }
  


  /***************************************************/	   
	   
      

	for(i=0;i<64;i++)
    {
      ii=(i+4)/4;
      iii=(ii+3)/4;
      for(j=0;j<64;j++)
      {
        jj=(j+4)/4;
        jjj=(jj+3)/4;
		if(mononucneigh2==1) QT3ij[i][j]= mpi[(j%16)%4]*RT3ij[i][j];
        else QT3ij[i][j]=tripi[j]*RT3ij[i][j];
        if( (ii != jj) &&  (i-4*(ii-1) != j-4*(jj-1)) )               /*Hammingdistance==2*/
           QT3ij[i][j]=0;
        if( (iii != jjj) && ( (ii-4*(iii-1))!=(jj -4*(jjj-1)) ) )     /*Hammingdistance==2*/
           QT3ij[i][j]=0;
        if(matrixSH==0){
		     if( ( i-4*(ii-1) ==  j-4*(jj-1)   )  ) QT3ij[i][j]=0;}   /*expand model, only substitution on one position*/
        if(i==j) QT3ij[i][j]=0.00;                                        /*diagonale*/
      }
    }

    dig=0.00;
    for(i=0; i<64; i++){
      rowsum[i]=0.00;
      for(j=0; j<64; j++){
         if(i!=j)  rowsum[i]=rowsum[i]+ QT3ij[i][j];}
	  QT3ij[i][i]= -1*rowsum[i];                         /*Diagonalentries*/ 
      dig=dig + tripi[i]*rowsum[i];                        /*Diagonalentries multipli. with the rowsum together*/
 }                                                   

   
 if(verboseQMatrix==1){
	fprintf(stderr,"\nQT3:\n");
    fprintf(stderr,"Triplet3:dig=%f\n",dig);
     fprintf(stderr,"\n");
     fprintf(stderr,"     ");
     for(i=0; i<64; i++) fprintf(stderr,"%6s|",triplet[i]);
     fprintf(stderr,"\n");
	 
 
     for(i=0; i<64; i++){
       fprintf(stderr,"%2s||",triplet[i]);
       for(j=0; j<64; j++)
       fprintf(stderr,"% .3f|",QT3ij[i][j]);
       fprintf(stderr,"\n");
     }
	 fprintf(stderr,"Triplet3: dig=%f\n",dig);
  }
    
  for(i=0;i<64;i++)
        for(j=0;j<64;j++)
                QT3ij[i][j] /= dig ;                     /*AB HIER PI*RATE -> NORMIERUNG*/

  if(verboseQMatrix==1){
     fprintf(stderr,"\n");
	 fprintf(stderr,"QT3:\n");
     fprintf(stderr,"     ");
     for(i=0; i<64; i++) fprintf(stderr,"%6s|",triplet[i]);
     fprintf(stderr,"\n");

     for(i=0; i<64; i++){
       fprintf(stderr,"%s||",triplet[i]);
       for(j=0; j<64; j++)
       fprintf(stderr,"% .3f|",QT3ij[i][j]);
       fprintf(stderr,"\n");
     }
   }

    dig=0;
    for (i=0; i<64; i++)
       dig -= tripi[i]*QT3ij[i][i];      /*sum of the diagonal values*/

    if(verboseRate==1 || verboseQMatrix==1)
       fprintf(stderr, "Q3(nk=2): sum(diagonal.values*pi):expected number of substitutions = %f\n", dig);
	   
  
	return(QT3ij);

}
   
   
   

   
   
   
