/******************************************************************************
 *   mutate.c                                                                  *
 *                                                                            *
 *   SISSI0.93 (Prerelease Version)                                           *
 *   SImulating Site-Specific Interactions along phylogenetic trees           *
 *                                                                            *
 *   Copyright (C) 2004-2005 by Tanja Gesell, Arndt von Haeseler              *
 *   sissi@cs.uni-duesseldorf.de                                              *
 *                                                                            *
 *   All parts of the source code except where indicated are distributed      *
 *	 under the GNU public licence.                                            *
 *	 See http://www.opensource.org for details.                               *
 *                                                                            *
 *   This program is distributed in the hope that it will be useful,          *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 *   GNU General Public License for more details.                             *
 *                                                                            *
 *   Any feedback is very welcome!              sissi@cs.uni-duesseldorf.de   *
 *****************************************************************************/

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include <sys/stat.h>
#include "dkiss.h"
#include "global.h"
#include "treefile.h"
#include "models.h"
#include "neighfile.h"
#include "evolve.h"
#include "mutate.h"


FILE  *dinucright_fv;
FILE  *dinucleft_fv;
/* FILE  *scalingf_fv;*/
FILE  *dh_fv;
FILE  *qx_fv;
char  filename[256];
int printfile;
int catnei;
int numseq;                 /* number of recent sequence, resp.number of taxons */




double *scalingfactor;
int    sitefactor;
int    sitefactornorm;
int    sitefactor2;
int    sitefactornorm2;
int    verboseScalingFactor;
int    verboseunnormScalingFactor;


 /* Compute rates */
 double *rate(char *seq, double *Qij, double **QSHij, double **QTij, double **QT2ij, double **QT3ij);
 double  rate_sum(double rat[]);
 double *rate_independent(char *seq, double *Qij);
 int fastrates;
 /* PROBABILITY TO SELECT THE POSITION  */
 double *selsite(double rat[], double rsum);            
 int    rand_choose(double proselsite[]);                 /* RANDOM CHOOSE: Select one positon for the next mutation  */
 /* calculation of substitutions:  */
 double *probmut( int secpos, double *Qij, char *seq);    /* INDEPENDENT case*/
 int     choosenuc(double *probmutij);
 double *probmutSH(int secpos, double **D, char *seq);    /* nk=1 */
 int doubletnucleotidmutation(int doubletmutation, int secpos, char *seq);
 int     choosedoublet(double *probmutij_doub);
 double *probmutTriplet(int secpos, double **D, char *seq); /* nk=2 */
 int choosetriplet(double *probmutij_trip);
 int tripletnucleotidmutation(int tripletmutation, int secpos, char *seq);
 /*     MIX       */
 double *probmutSHmix(int secpos, double **D, char *seq);
 int     choosenucmix(double *probmutij, double *probmutij_dmix, int secpos, char *seq );
 int     tripletnucleotidmutation(int tripletmutation, int secpos, char *seq);
 /* Triplett  with categorie  */
 double *cprobmutTriplet(int pos, double **QTij,double **QT2ij,double **QT3ij, char *seq);    
 int cchoosetriplet(double *probmutij_trip);
 int ctripletnucleotidmutation(int tripletmutation, int secpos, char *seq); 
 /*************************************************/
 double timeoverall(double rsum) ;   /* TIME FOR THE NEXT MUATATION                        */
 char *copyseq(char *rseq);
 char *changeseq(char *seqi, int secpos, int mutation);
 void AllocateMemory();
 void RandomSequence_independent(char *seq);
 void RandomSequence(char *seq);
 void CreateSequences(TTree *tree);
 void SetSequence(char *seq, char *source);
 double getrandomdouble();
  /*************************************************/
 double *newcurrentrate(int changepos, double *currentrat, char *seq, double *Qij, double **QSHij, double **QTij, double **QT2ij, double **QT3ij); 
 double *changerate(int currentpos, double *currentrat, char *seq, double *Qij, double **QSHij, double **QTij, double **QT2ij, double **QT3ij);






 double getrandomdouble()
 {
#  if SPRNG
     if(randomgeneratordkiss == 1)
#  endif    
	   return(dkiss());               /*random with dkiss()*/
#  if SPRNG	   
     else	
       return(sprng(randstream));  
#  endif     
 /*#       else
	   return(sprng(randstream));*/
 /*#       endif*/
 }

 /****************************************************************************************************************/
 /*                                 TIME                                                                         */
 /****************************************************************************************************************/
 /******************************************************/
 /* TIME FOR THE NEXT MUATATION                        */
 /* IS EXPONENTIAL DISTRIBUTE OVER THE SUM OF THE RATES*/
 /******************************************************/
 double timeoverall(double rsum)
 {
         double t_expsum;
         double a;
		 
           a=1-getrandomdouble();
           t_expsum=log(a)/rsum;
		  /* if(verbosewaiting==1) fprintf(stderr,"t_expsum: %f\n", t_expsum);*/
		return(t_expsum);
  }
 /****************************/
 /* COPY SEQI TO SEQ J       */
 /****************************/
 char *copyseq(char *rseq){
            int pos;
            char *seqi;

            seqi=(char*)malloc(seqlen*sizeof(char));
            for(pos=0;pos<seqlen;pos++)
              seqi[pos]=rseq[pos];
            
			return(seqi);
  }
 /**************************************************************/
 /*CHANGE NUCLEOTID POSITION I TO J  INDEPENDENT=DEPENDENT CASE*/
 /**************************************************************/
 char *changeseq(char *seqi, int secpos, int mutation)
 {
   seqi[secpos]=nucleotide[mutation];
   return(seqi);
 }
 /**************************************************************/
 /*                    SEQUENCE                                */
 /**************************************************************/		
 /*
  * Random Sequence indpendent
  */		
  void RandomSequence_independent(char *seq)
  {
	char *P, *rseq;
	double p[4], x;
    int i, nr, ink, pos;

	rseq=(char*)calloc(seqlen,sizeof(char));
	P=seq;
	
	   p[0]=0;
       p[1]=mpi[0];
       p[2]=mpi[0]+mpi[1];
       p[3]=mpi[0]+mpi[1]+mpi[2];

	for(pos=0; pos<seqlen; pos++)
	{
			do{
			   nr=0;
			   x=getrandomdouble();
			   for(ink=1; ink<4;ink++)
				  if(x>p[ink]) nr=ink;
			}while(nr<0 || nr>3);

			rseq[pos]=nucleotide[nr];
	}
           	
	for (i=0; i<seqlen; i++) {
		*P=rseq[i];
		P++;
	}
	if(verboserandom==1) for(pos=0; pos<seqlen; pos++) fprintf(stderr, "%c", rseq[pos]);
 }

 /*
  * Random Sequnece
  */
  void RandomSequence(char *seq)
  {
	char *P, *rseq;
	double p[4],d[16],t[64], x;
    int i, nr, ink, pos, secneighpos1, secneighpos2;
    int *fill;

    rseq=(char*)calloc(seqlen,sizeof(char));
	fill=(int *)calloc(seqlen,sizeof(int));   
	
    P=seq;
	
	
	   p[0]=0;
       p[1]=mpi[0];
       p[2]=mpi[0]+mpi[1];
       p[3]=mpi[0]+mpi[1]+mpi[2];


       d[0] = 0;
       d[1] = dipi[0];
       d[2] = dipi[0]+dipi[1];
       d[3] = dipi[0]+dipi[1]+dipi[2];
       d[4] = dipi[0]+dipi[1]+dipi[2]+dipi[3];
       d[5] = dipi[0]+dipi[1]+dipi[2]+dipi[3]+dipi[4];
       d[6] = dipi[0]+dipi[1]+dipi[2]+dipi[3]+dipi[4]+dipi[5];
       d[7] = dipi[0]+dipi[1]+dipi[2]+dipi[3]+dipi[4]+dipi[5]+dipi[6];
       d[8] = dipi[0]+dipi[1]+dipi[2]+dipi[3]+dipi[4]+dipi[5]+dipi[6]+dipi[7];
       d[9] = dipi[0]+dipi[1]+dipi[2]+dipi[3]+dipi[4]+dipi[5]+dipi[6]+dipi[7]+dipi[8];
       d[10]= dipi[0]+dipi[1]+dipi[2]+dipi[3]+dipi[4]+dipi[5]+dipi[6]+dipi[7]+dipi[8]+dipi[9];
       d[11]= dipi[0]+dipi[1]+dipi[2]+dipi[3]+dipi[4]+dipi[5]+dipi[6]+dipi[7]+dipi[8]+dipi[9]+dipi[10];
       d[12]= dipi[0]+dipi[1]+dipi[2]+dipi[3]+dipi[4]+dipi[5]+dipi[6]+dipi[7]+dipi[8]+dipi[9]+dipi[10]+dipi[11];
       d[13]= dipi[0]+dipi[1]+dipi[2]+dipi[3]+dipi[4]+dipi[5]+dipi[6]+dipi[7]+dipi[8]+dipi[9]+dipi[10]+dipi[11]+dipi[12];
       d[14]= dipi[0]+dipi[1]+dipi[2]+dipi[3]+dipi[4]+dipi[5]+dipi[6]+dipi[7]+dipi[8]+dipi[9]+dipi[10]+dipi[11]+dipi[12]+dipi[13];
       d[15]= dipi[0]+dipi[1]+dipi[2]+dipi[3]+dipi[4]+dipi[5]+dipi[6]+dipi[7]+dipi[8]+dipi[9]+dipi[10]+dipi[11]+dipi[12]+dipi[13]+dipi[14];

       t[0] = 0;
       t[1] = tripi[0];
       for(i=2;i<64;i++)
		  t[i] = t[i-1]+tripi[i-1];
       if(verboserandom==1) fprintf(stderr, "Random Sequence: sum=%f\n",t[63]);
	   
	   if(seqlen>MAX_POS){ 
	         fprintf(stderr, "Max positions of the neighbourhood sytem %d.\n", MAX_POS);
			 fprintf(stderr, "change #define MaX_POS in neigh.h or write an email to\n");
			 fprintf(stderr, "sissi@cs.uni-duesseldorf.de\n");
			 exit(0) ;
	    }		 
	   
	   for(pos=0; pos<seqlen; pos++)
       {
          if(fill[pos]==0)
          {
             if(Nseq.item[pos].nNei_==0)
             {
                  do{
                      nr=0;
                      x=getrandomdouble();
                      for(ink=1; ink<4;ink++)
                        if(x>p[ink]) nr=ink;
                  }while(nr<0 || nr>3);

                 rseq[pos]=nucleotide[nr];
                 fill[pos]=1;
		    }
            else
            { 
		     if(Nseq.item[pos].nNei_==1)
             {
				secneighpos1=Nseq.item[pos].neiArr_[0];
				   do
				   {
                         nr = 0;
                         x  = getrandomdouble();
                         for(ink=1; ink <16; ink++)
                               if(x>d[ink]) nr=ink;
				   }while(nr<0 || nr>15);

				   rseq[pos]=doublet[nr][0];
				   rseq[secneighpos1]=doublet[nr][1];
				   fill[pos]=fill[secneighpos1]=1;
             }
			
            if( Nseq.item[pos].nNei_==2)
            {
				 secneighpos1=Nseq.item[pos].neiArr_[0];
				 secneighpos2=Nseq.item[pos].neiArr_[1];

                      do
                      {
                         nr = 0;
                         x  = getrandomdouble();
                         for(ink=1; ink <64; ink++)
                               if(x>t[ink]) nr=ink;
                       }while(nr<0 || nr>63);


                if(verboserandom==1) fprintf(stderr, "tripletnr: %d=%s ", nr, triplet[nr]);
				rseq[pos]=triplet[nr][0];
				rseq[secneighpos1]=triplet[nr][1];
				rseq[secneighpos2]=triplet[nr][2];
				/*fprintf(stderr, "tripletnr: 1:%c, 2%c, 3%c", triplet[nr][0], triplet[nr][1], triplet[nr][2]);*/
				fill[pos]=fill[secneighpos1]=fill[secneighpos2]=1;
				
           }
           else
           {   if(Nseq.item[pos].nNei_ > 2)
              fprintf(stderr, "to much neighbors at the moment!");
           }
	    }
	  }
	}
  
	
   for (i=0; i<seqlen; i++) {
		*P=rseq[i];
		P++;
	}
   if(verboserandom==1) for(pos=0; pos<seqlen; pos++) fprintf(stderr, "%c", rseq[pos]);
}
/******************************************************************************/
void CreateSequences(TTree *tree)
{
	TNode *P;
	
	P=tree->nodeList;
	while (P!=NULL) {
	    if(P->sequence != NULL) free(P->sequence);
		P->sequence=CAllocMem(numBases+1, "sequences", "CreateSequences", 0);
		P=P->next;
	}
}
/******************************************************************************/
void SetSequence(char *seq, char *source)
{
	int i;
	char *P, *Q;
	
	P=seq;
	Q=source;
	
	for(i=0;i<seqlen;i++){
	*P=source[i];
	P++;
	}
}
/**************************************************************/
/*             MUTATE  SEQUENCE                               */
/**************************************************************/	
void MutateSequence(char *seq, double len, double *Qij, double **QSHij, double **QTij,double **QT2ij,double **QT3ij ){
  char    *P;
  char    *ancestorseq;
  double  *rat;                                  
  double  rsum;
  double  t=0;
  int secpos,i;
  /*int *secneighpositions;    */    
  int     substitutions, osubstitutions;
  double  *pro_selsite;
  double  *probmutij; 
  double *probmutij_mix;   
  double  *probmutikjl;
  double  *probmuttriple;
  int     mutation;
  int     doubletmutation; 
  int     tripletmutation; 
  char dhFilename[256];

		
  ancestorseq =(char*)malloc(numBases*sizeof(char));
  for(i=0; i<numBases; i++)
		ancestorseq[i]=seq[i];

	P=seq;
	if(neighborFile == 0)
		   rat = rate_independent(seq, Qij);
	else   
		   rat  = rate(seq,Qij,QSHij,QTij,QT2ij,QT3ij); 
	rsum = rate_sum(rat);
   /* if(verbosewaiting==1) fprintf(stderr,"\nhier rsum= %f\n",rsum);*/
	/*************Time*****************************************/
	t    = timeoverall(rsum);
	substitutions=0;
	/**********************************              
	        EVOLUTION PROCESS                        
	 ********************************* */
	 if(t>len && verboseWarning==1){
		fprintf(stderr, "No substitution! (waiting time:%f > branch length:%f)\n", t, len);
	 }
	
	 while(t<len)
	 {
        substitutions++; 
	    pro_selsite=NULL;  
		pro_selsite=selsite(rat,rsum);        /*probability for position selection*/
		secpos=-1;
		secpos  = rand_choose(pro_selsite);   /*actuell position*/
		/*printf("%d\n",secpos);*/
		/*********************************/
	    /*which subsitution*/
		mutation=-1;
		
       if(neighborFile==0){
            probmutij=probmut(secpos,Qij,seq);
			mutation=choosenuc(probmutij);
       }else{	   		
		  if(mix==1){
               probmutij=probmut(secpos,Qij,seq);
               probmutij_mix  = probmutSHmix(secpos, QSHij,seq);
			   mutation = choosenucmix(probmutij,probmutij_mix,secpos,seq);
	      } 
	      else{	 	   
	 	      if(Nseq.item[secpos].nNei_ == 0){          /*independent*/
			     probmutij=probmut(secpos,Qij,seq);
			     mutation=choosenuc(probmutij);
		      }
		      if(Nseq.item[secpos].nNei_ == 1){
		    	probmutikjl       = probmutSH(secpos, QSHij,seq);
			    doubletmutation   = choosedoublet(probmutikjl);
			    mutation          = doubletnucleotidmutation(doubletmutation,secpos,seq);				   
		      }
		     if(Nseq.item[secpos].nNei_ == 2)
		     {
			 if(catnei==1){
				probmuttriple        = cprobmutTriplet(secpos, QTij,QT2ij, QT3ij, seq);
				tripletmutation      = cchoosetriplet(probmuttriple);
				mutation             = ctripletnucleotidmutation(tripletmutation, secpos, seq);
			 }else{
				probmuttriple        = probmutTriplet(secpos, QTij, seq);
				tripletmutation      = choosetriplet(probmuttriple);
				mutation             = tripletnucleotidmutation(tripletmutation, secpos, seq);
			 }	
		 }
       }	
     }	 
	/*********************************/
	seq=changeseq(seq,secpos,mutation);
	P=seq;
	/*******************************************************/
	 /* free(secneighpositions);*/
	if(fastrates==0){
	   free(rat);
	   if(neighborFile == 0)
		  rat  = rate_independent(seq, Qij);
	   else   
		  rat  = rate(seq,Qij,QSHij,QTij,QT2ij,QT3ij);
	}else	  
	   rat = changerate(secpos,rat,seq,Qij,QSHij,QTij,QT2ij,QT3ij);
    rsum= -1;
	rsum = rate_sum(rat);
	/* if(verbosewaiting==1) fprintf(stderr, "\n rsum= %f\n",rsum);*/
	t += timeoverall(rsum);
	/********************************************************/
	}	
	free(rat);
	osubstitutions=0;
	for(i=0;i<numBases;i++){
		if(ancestorseq[i]!=seq[i]) osubstitutions=osubstitutions+1;
	}
	if(verboseEvolutiontime==1){
	  if(len==0.00) fprintf(stderr, "branch length: %f - no substituion\n", len);
	  else{
	      if(printfile==1) {	
		          sprintf(dhFilename, "%s_dh.sissi", filename);
	              if((dh_fv=fopen(dhFilename,"a"))==NULL)
	                        fprintf(dh_fv, "dinuc.txt could not open\n");	
	              fprintf(dh_fv ,  "branch length: %f \t substitutions: %6d %10f \t Hammingdistance: %6d %10f \n", len,substitutions,(double)substitutions/seqlen,osubstitutions,(double)osubstitutions/seqlen);
	              }else fprintf(stderr , "branch length: %f \t substitutions: %6d %10f \t Hammingdistance: %6d %10f \n", len,substitutions,(double)substitutions/seqlen,osubstitutions,(double)osubstitutions/seqlen);
		}
	}	
	free(ancestorseq);			
 }
 /****************************************************************************************************************/
 /*                              Choose a site k                                                                 */
 /****************************************************************************************************************/
 /**********************************/
 /*  DEPENDENT = INDEPENDENT CASE  */
 /**********************************/
  double *selsite(double rat[], double rsum){
       int pos;
       double *proselsite;
     
       proselsite=(double *)malloc(seqlen*sizeof(double));
       for(pos=0;pos<seqlen;pos++){
                proselsite[pos]=rat[pos]/rsum ;
		}
        return(proselsite);
  }
  /************************************************************/
  /* RANDOM CHOOSE: Select one positon for the next mutation  */
  /************************************************************/
   int rand_choose(double proselsite[]){
         double x;
         double dummy=0;
         int m;
         int secpos;

         x=getrandomdouble();
         m=0;
		 secpos=-1;

         do
         {
             secpos=m;
             dummy += proselsite[m];
             m++;
         }
         while(x>dummy && m<seqlen);
         free(proselsite);
         return(secpos);
    }
   /****************************************************************************************************************/
   /*                                Replace xk with yo                                                            */
   /****************************************************************************************************************/
    /***************************************/
	/*    INDEPENDENT CASE                 */
	/***************************************/       
    double *probmut( int secpos, double *Qij, char *seq){
         int nucsecpos,j ;
         double *probmutij;
		 
         nucsecpos=-1;
         probmutij =(double *)malloc(4*sizeof(double));
         nucsecpos= nucleoTO123(seq[secpos]);

         for(j=0;j<4;j++) probmutij[j]=0.00;
         for(j=0;j<4;j++){
            if(nucsecpos != j){
               probmutij[j]= -Qij[nucsecpos*4+j]/Qij[nucsecpos*4+nucsecpos];
            }
         }
         return(probmutij);
     }
     /*CHOOSE PER RANDOM NUCLEOTID J */
	 int choosenuc(double *probmutij){
           int j, mutation;
           double dummy, x;

           x=getrandomdouble();
           mutation=0;
           j=0;
           dummy=0.00;
           do{
              dummy += probmutij[j];
              mutation=j;
              ++j;
            }while(x>=dummy);

            return(mutation);
	 }
/***************************************/
/* nk=1                                */
/***************************************/
double *probmutSH(int secpos, double **D, char *seq){
           int doubletsecpos,j ;
           double *probmutij_doub;
           double probsum=0.00;
		   int secposneigh=-1;
		   
           
		   secposneigh=Nseq.item[secpos].neiArr_[0];
           probmutij_doub=(double *)malloc(16*sizeof(double));
           doubletsecpos= doubletsTO15(seq[secpos], seq[secposneigh]);

           for(j=0 ; j<16 ; j++) probmutij_doub[j]=0.00;
           for(j=0 ; j<16; j++) {
             if(doubletsecpos != j) {
               probmutij_doub[j] = - D[doubletsecpos][j]/D[doubletsecpos][doubletsecpos];
               probsum += probmutij_doub[j] ;
             }
           }
		return(probmutij_doub);
}
/*CHOOSE PER RANDOM THE MUTATION PROCESS TO DOUBLET L  */
int choosedoublet(double *probmutij_doub){
           int j ;
           double dummy , x;
           int doubletmutation;


           x=getrandomdouble();
           doubletmutation=0;
           j=0;
           dummy=0.00;

           do{
             dummy =  dummy + probmutij_doub[j];
             if(probmutij_doub[j] != 0) doubletmutation=j;
             ++j;
           }while(x>=dummy ) ;
           /*printf("doubletmutation: %d  ", doubletmutation);*/
		   free(probmutij_doub);
           return (doubletmutation);
}
/*AND GIVE THE NUCLEOTIDCHANGEMUTATION J */
int doubletnucleotidmutation(int doubletmutation, int secpos, char *seq)
{
	        int doubletorg, ii, jj;
            int mutation=-1;
			int secposneigh=-1;
			
            secposneigh=Nseq.item[secpos].neiArr_[0];
            doubletorg= doubletsTO15(seq[secpos], seq[secposneigh]);
            ii=(doubletorg+4)/4;
            jj=(doubletmutation+4)/4;

            if((ii==jj) && (doubletorg - 4*(ii-1)) == (doubletorg - 4*(jj-1))){ fprintf(stderr, "doppelmutation!"); exit(1);} /* Fehler???? doubletorg->doubletmutation */
            if( (doubletorg- 4*(ii-1)) == (doubletmutation- 4*(jj-1)) ) mutation=doubletmutation/4 ; /* if(second position is the same)*/
            /*printf("doubletorg %d-> doubletmutation%d:  mutation %d an pos %d", doubletorg, doubletmutation,mutation, secpos);*/
            return(mutation);
}


/*******************************************************/
/*                  MIX                                */
/*******************************************************/		  
double *probmutSHmix(int secpos, double **D, char *seq){
           int j, state, secposneighsate ;
           double *probmutij_dmix;
           double probsum=0.00;
		   int secposneigh=-1;
		    
		   secposneigh=Nseq.item[secpos].neiArr_[0]; /*neighbor position*/
		   secposneighsate= nucleoTO123(seq[secposneigh]); /* current state ofneighbor position*/
		   state=nucleoTO123(seq[secpos]);   /* current state */
		
		   probmutij_dmix=(double *)malloc(4*sizeof(double));
                 
          /* printf("\nstate:%c\n",nucleotide[state]);*/
           for(j=0 ; j<4 ; j++) probmutij_dmix[j]=0.00;
           for(j=0 ; j<4; j++) {
             if(state != j) {
               probmutij_dmix[j] = - D[(state*4)+secposneighsate][(j*4)+secposneighsate]/D[(state*4)+secposneighsate][(state*4)+secposneighsate];
			/*printf("%d,%d,%d\n"   ,(state*4)+secposneighsate, (j*4)+secposneighsate,(state*4)+secposneighsate);*/
			/*printf("%s,%s,%s\n"   ,doublet[(state*4)+secposneighsate], doublet[(j*4)+secposneighsate],doublet[(state*4)+secposneighsate]);   */
				 probsum += probmutij_dmix[j] ;
             }
		  /* printf("j:%d, probsum=%f \n",j,probsum) ;*/
           }
          /* printf("sum %f\n",probsum);*/
			return(probmutij_dmix);
}
		
int choosenucmix(double *probmutij, double *probmutij_dmix, int secpos, char *seq ){
           int j, mutation, state;
           double dummy, x;
		   double *probmutij_dsum;
		   double probsum=0;
		   
		   state=nucleoTO123(seq[secpos]);
		   
		   probmutij_dsum=(double *)malloc(4*sizeof(double));
		   for(j=0 ; j<4 ; j++) probmutij_dsum[j]=0.00;
		   for(j=0 ; j<4; j++) {
             if(state != j) 
			   probmutij_dsum[j] =Nseq.item[secpos].neiVal_[0] * probmutij_dmix[j] + (1-Nseq.item[secpos].neiVal_[0]) * probmutij[j];
			   /*  printf("correlationvalue %f\n",1- Nseq.item[secpos].neiVal_[0]);*/
			   probsum += probmutij_dsum[j] ;
			}   
		   /* printf("sum %f\n",probsum);*/
		   
		   x=getrandomdouble();
           mutation=0;
           j=0;
           dummy=0.00;
           do{
              dummy += probmutij_dsum[j];
			  mutation=j;
              ++j;
            }while(x>=dummy);
			
			free(probmutij_dsum);
			free(probmutij);
			free(probmutij_dmix);

            return(mutation);
 }
 


/*******************************************************/
/*              nk=2                                   */
/*******************************************************/
double *probmutTriplet(int secpos, double **D, char *seq){
           int    tripletsecpos,j ;
           double *probmutij_trip;
           double probsum=0.00;
		   int    secposneigh1=-1;
		   int    secposneigh2=-1;

		   secposneigh1=Nseq.item[secpos].neiArr_[0];
		   secposneigh2=Nseq.item[secpos].neiArr_[1];
           probmutij_trip=(double *)malloc(64*sizeof(double));
		   tripletsecpos= tripletTO64(seq[secpos],seq[secposneigh1],seq[secposneigh2]);
         /*  fprintf(stderr, "\ntriplet %c%c%c:%d:%s on position %d\n",seq[secpos],seq[secposneigh1],seq[secposneigh2],tripletsecpos,triplet[tripletsecpos], secpos);  */
           for(j=0 ; j<64 ; j++) probmutij_trip[j]=0.00;
           for(j=0 ; j<64; j++) {
             if(tripletsecpos != j) {
               probmutij_trip[j] = - D[tripletsecpos][j]/D[tripletsecpos][tripletsecpos];
               probsum += probmutij_trip[j] ;
             }
          /* printf("j:%d, doubletsec%d probmutij_doub=%f \n",j,doubletsecpos,probmutij_doub[j]) ;*/
           }
          /* printf("sum %f\n",probsum);*/
		  /* printf("stop\n"); */

           return(probmutij_trip);
}
/*******************************************************/
int choosetriplet(double *probmutij_trip){
           int j ;
           double dummy , x;
           int tripletmutation;


           x=getrandomdouble();
           tripletmutation=0;
           j=0;
           dummy=0.00;

           do{
             dummy =  dummy + probmutij_trip[j];
             if(probmutij_trip[j] != 0) tripletmutation=j;
             ++j;
           }while(x>=dummy ) ;
           /*fprintf(stderr, "tripletmutation: %d  \n", tripletmutation);*/
		   free(probmutij_trip);
           return (tripletmutation);
}
/*AND GIVE THE NUCLEOTIDCHANGEMUTATION J */
int tripletnucleotidmutation(int tripletmutation, int secpos, char *seq){
            int tripletorg, ii, jj, iii, jjj;
            int mutation;
			int secposneigh1=-1;
			int secposneigh2=-1;

			
            secposneigh1=Nseq.item[secpos].neiArr_[0];
			secposneigh2=Nseq.item[secpos].neiArr_[1];
            tripletorg= tripletTO64(seq[secpos], seq[secposneigh1], seq[secposneigh2]);
			/*fprintf(stderr, " %d -> %d \n", tripletorg, tripletmutation);*/
            ii =(tripletorg+4)/4;
			iii=(tripletorg+16)/16;
            jj=(tripletmutation+4)/4;
			jjj=(tripletmutation+16)/16;

            
            if( ((tripletorg-4*(ii-1)) == (tripletmutation-4*(jj-1))) && ( ((ii-1)-4*(iii-1))==((jj-1)-4*(jjj-1)) ) ) {
			      mutation=tripletmutation/16 ; 
			   /*  fprintf(stderr, "triplet: %d:%s -> %d:%s on position %d :%d=%c\n", tripletorg,triplet[tripletorg], tripletmutation,triplet[tripletmutation], secpos, mutation, nucleotide[mutation]); */
			}
			else {
				  fprintf(stderr, "triplet: %d:%s -> %d:%s on position %d \n", tripletorg,triplet[tripletorg], tripletmutation,triplet[tripletmutation], secpos);
			      fprintf(stderr, "dritte:%d==%d, zweite:%d==%d\n",(tripletorg-4*(ii-1)),(tripletmutation-4*(jj-1)),(ii-1)-4*(iii-1),(jj-1)-4*(jjj-1)); 
                  fprintf(stderr, "WRONG MUTATION IN TRIPLET!\n"); exit(1);
		    }	  
            /*printf("doubletorg %d-> doubletmutation%d:  mutation %d an pos %d", doubletorg, doubletmutation,mutation, secpos);*/
            return(mutation);
}
/*******************************************************/
/* Triplet  with categorie                            */
/*******************************************************/
double *cprobmutTriplet(int pos, double **QTij,double **QT2ij,double **QT3ij, char *seq){
           int       j,t=-1 ;
           double    *probmutij_trip;
           double    probsum=0.00;
		  /* int       secposneigh1=-1;
		   int       secposneigh2=-1; */
		   
		    probmutij_trip=(double *)malloc(64*sizeof(double));
			for(j=0 ; j<64 ; j++) probmutij_trip[j]=0.00;
			
			/******************************************************/
			/*      extra option für overlappings mit catpos      */
			/******************************************************/
			/* if(ovelappings==1){
			
			
			
			
			*/
			   
			
			if(Nseq.item[pos].cat_pos==1 && Nseq.item[ Nseq.item[pos].neiArr_[0] ].cat_pos==2 &&Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==3){ 
			        t=tripletTO64(seq[pos],seq[Nseq.item[pos].neiArr_[0]],seq[Nseq.item[pos].neiArr_[1]]);
					for(j=0 ; j<64; j++) {
                         if(t != j) {
                               probmutij_trip[j] = - QTij[t][j]/QTij[t][t];
                               probsum += probmutij_trip[j] ;
					      }
                    }
				 } 
				 
				 if(Nseq.item[pos].cat_pos==1 && Nseq.item[ Nseq.item[pos].neiArr_[0] ].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==2){ 
			       t=tripletTO64(seq[pos],seq[Nseq.item[pos].neiArr_[1]],seq[Nseq.item[pos].neiArr_[0]]);
                   for(j=0 ; j<64; j++) {
                         if(t != j) {
                               probmutij_trip[j] = - QTij[t][j]/QTij[t][t];
                               probsum += probmutij_trip[j] ;
					      }
                    }

				 }
				 
				 if(Nseq.item[pos].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==1 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==3){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[0]],seq[pos],seq[Nseq.item[pos].neiArr_[1]]);
				   for(j=0 ; j<64; j++) {
                         if(t != j) {
                               probmutij_trip[j] = - QT2ij[t][j]/QT2ij[t][t];
                               probsum += probmutij_trip[j] ;
					      }
                    }

                 
				 }
				 
				 if(Nseq.item[pos].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==1){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[1]],seq[pos],seq[Nseq.item[pos].neiArr_[0]]);
				   for(j=0 ; j<64; j++) {
                         if(t != j) {
                               probmutij_trip[j] = - QT2ij[t][j]/QT2ij[t][t];
                               probsum += probmutij_trip[j] ;
					      }
                    }

				 }
				 
				 if(Nseq.item[pos].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==1 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==2){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[0]],seq[Nseq.item[pos].neiArr_[1]],seq[pos]);
				   for(j=0 ; j<64; j++) {
                         if(t != j) {
                               probmutij_trip[j] = - QT3ij[t][j]/QT3ij[t][t];
                               probsum += probmutij_trip[j] ;
					      }
                    }

				}

                 if(Nseq.item[pos].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==1){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[1]],seq[Nseq.item[pos].neiArr_[0]],seq[pos]);
				   for(j=0 ; j<64; j++) {
                         if(t != j) {
                               probmutij_trip[j] = - QT3ij[t][j]/QT3ij[t][t];
                               probsum += probmutij_trip[j] ;
					      }
                    }

                   
				 }
				 
				 
				 
				 if(Nseq.item[pos].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[0]].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==2){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[0]],seq[pos],seq[Nseq.item[pos].neiArr_[1]]);
				   for(j=0 ; j<64; j++) {
                         if(t != j) {
                               probmutij_trip[j] = - QT2ij[t][j]/QT2ij[t][t];
                               probsum += probmutij_trip[j] ;
					      }
                    }

                 
				 }


                  if(Nseq.item[pos].cat_pos==0) fprintf(stderr, "\nPos %d Cateforie 0\n", pos); 

				 if(verbosewaiting==1) fprintf(stderr, "triple %d:%s with", t, triplet[t]);
                 if(verbosewaiting==1) fprintf(stderr, "sum 1.00=%f->", probsum);
				 

           return(probmutij_trip);
}
/***************************************/
int cchoosetriplet(double *probmutij_trip){
           int j ;
           double dummy , x;
           int tripletmutation;


           x=getrandomdouble();
           tripletmutation=0;
           j=0;
           dummy=0.00;

           do{
             dummy =  dummy + probmutij_trip[j];
             if(probmutij_trip[j] != 0) tripletmutation=j;
             ++j;
           }while(x>=dummy ) ;
           return (tripletmutation);
         }


          /*AND GIVE THE NUCLEOTIDCHANGEMUTATION J */
          int ctripletnucleotidmutation(int tripletmutation, int secpos, char *seq)
          {
            int codonnr, ii, iii;
            int mutation=-1;
			
			
			ii =(tripletmutation+4)/4;
			iii=(tripletmutation+16)/16;
			
			codonnr=Nseq.item[secpos].cat_pos;
			if(verbosewaiting==1) fprintf(stderr,"->%s", triplet[tripletmutation]);
			if(codonnr==1) mutation=tripletmutation/16 ;  
			if(codonnr==2) mutation=(ii-1)%4; 
			if(codonnr==3) mutation=tripletmutation%4 ; 
			if(verbosewaiting==1) fprintf(stderr, " %d : %c\n", codonnr, nucleotide[mutation]);
			return(mutation);
}
/****************************************************************************************************************/
/* CALCULATE THE SPONTANEOUS SUBSTITUTIONSRATE FOR THE POSITIONS                                                */
/****************************************************************************************************************/
double *rate(char *seq, double *Qij, double **QSHij, double **QTij, double **QT2ij, double **QT3ij)
{
           double *rat ;
           double addrat=0;
           int pos,d,w,t;

           rat=(double *)calloc(seqlen,sizeof(double));
           /*bzero(rat, seqlen*sizeof(double));*/
           w=0;
		   
		   
		   
	for(pos=0; pos<seqlen;pos++){
	  rat[pos]=10;

      if(mix==1){
	  d=doubletsTO15(seq[pos],seq[Nseq.item[pos].neiArr_[0]]);
	  rat[pos]=Nseq.item[pos].neiVal_[0]*(QSHij[d][d]) + (1-Nseq.item[pos].neiVal_[0]) *(Qij[nucleoTO123(seq[pos])*4+nucleoTO123(seq[pos])]);
	  if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
	  addrat=addrat + rat[pos];
      }
     else{
	       if(Nseq.item[pos].nNei_ == 0){
               rat[pos]=Qij[nucleoTO123(seq[pos])*4+nucleoTO123(seq[pos])];
			   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
		   }
		   else if(Nseq.item[pos].nNei_ == 1){
               d=doubletsTO15(seq[pos],seq[Nseq.item[pos].neiArr_[0]]);
			   rat[pos]=QSHij[d][d];
			   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
		   }
		   else if(Nseq.item[pos].nNei_ == 2){
			    if(catnei==1){
				
				 if(Nseq.item[pos].cat_pos==1 && Nseq.item[ Nseq.item[pos].neiArr_[0] ].cat_pos==2 &&Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==3){ 
			       t=tripletTO64(seq[pos],seq[Nseq.item[pos].neiArr_[0]],seq[Nseq.item[pos].neiArr_[1]]);
                   rat[pos]= QTij[t][t];
				   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
				 } 
				 
				 if(Nseq.item[pos].cat_pos==1 && Nseq.item[ Nseq.item[pos].neiArr_[0] ].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==2){ 
			       t=tripletTO64(seq[pos],seq[Nseq.item[pos].neiArr_[1]],seq[Nseq.item[pos].neiArr_[0]]);
                   rat[pos]= QTij[t][t];
				   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
				 }
				 
				 if(Nseq.item[pos].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==1 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==3){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[0]],seq[pos],seq[Nseq.item[pos].neiArr_[1]]);
                   rat[pos]= QT2ij[t][t];
				   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
				 }
				 
				 if(Nseq.item[pos].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==1){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[1]],seq[pos],seq[Nseq.item[pos].neiArr_[0]]);
                   rat[pos]= QT2ij[t][t];
				   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
				 }
				 
				 if(Nseq.item[pos].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==1 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==2){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[0]],seq[Nseq.item[pos].neiArr_[1]],seq[pos]);
                   rat[pos]= QT3ij[t][t];
				   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
				 }

                 if(Nseq.item[pos].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==1){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[1]],seq[Nseq.item[pos].neiArr_[0]],seq[pos]);
                   rat[pos]= QT3ij[t][t];
				   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
				 }
				 
				 /*099*/
				  if(Nseq.item[pos].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==2){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[1]],seq[pos],seq[Nseq.item[pos].neiArr_[0]]);
                   rat[pos]= QT2ij[t][t];
				   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
				  }
				  
				    if(Nseq.item[pos].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==3){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[1]],seq[pos],seq[Nseq.item[pos].neiArr_[0]]);
                   rat[pos]= QT3ij[t][t];
				   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
				  }
				  
				      if(Nseq.item[pos].cat_pos==1 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==1 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==1){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[1]],seq[pos],seq[Nseq.item[pos].neiArr_[0]]);
                   rat[pos]= QTij[t][t];
				   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
				  }

                  if(Nseq.item[pos].cat_pos==0) fprintf(stderr,"\nulllala\n"); 

				 
			}	                     
			else{
			   t=tripletTO64(seq[pos],seq[Nseq.item[pos].neiArr_[0]],seq[Nseq.item[pos].neiArr_[1]]);
               rat[pos]= QTij[t][t];
			     if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
			}
		}
    }		 
			 if(rat[pos]==10) {printf("1WRONG RATE1\n") ; exit (1);}
          /*  if(verbosewaiting==1) addrat += rat[pos];*/
			

	}  
		 /* if(verbosewaiting==1) fprintf(stderr,"summierte Raten: %f\n",addrat);*/
		 /* fprintf(stderr"\n addrat=%f\n", addrat);*/
          return(rat);
      
}




/************************************************************************/
/********************** sissiDI01 ************************/
/************************************************************************/
double *rateDI(char *seq, double *Qij, double **QSHij, double **QTij, double **QT2ij, double **QT3ij)
{
           double *rat ;
           double addrat=0;
           int pos,d,w,t;

           rat=(double *)calloc(seqlen,sizeof(double));
           /*bzero(rat, seqlen*sizeof(double));*/
           w=0;
		   
		   
		   
	for(pos=0; pos<seqlen;pos++){
	  rat[pos]=10;

      if(mix==1){
	  d=doubletsTO15(seq[pos],seq[Nseq.item[pos].neiArr_[0]]);
	  rat[pos]=Nseq.item[pos].neiVal_[0]*(QSHij[d][d]) + (1-Nseq.item[pos].neiVal_[0]) *(Qij[nucleoTO123(seq[pos])*4+nucleoTO123(seq[pos])]);
	  addrat=addrat + rat[pos];
      }
     else{
	       if(Nseq.item[pos].nNei_ == 0){
               rat[pos]=Qij[nucleoTO123(seq[pos])*4+nucleoTO123(seq[pos])];
		   }
		   else if(Nseq.item[pos].nNei_ == 1){
               d=doubletsTO15(seq[pos],seq[Nseq.item[pos].neiArr_[0]]);
             			   rat[pos]=QSHij[d][d];
		   }
		   else if(Nseq.item[pos].nNei_ == 2){
			    if(catnei==1){
				
				 if(Nseq.item[pos].cat_pos==1 && Nseq.item[ Nseq.item[pos].neiArr_[0] ].cat_pos==2 &&Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==3){ 
			       t=tripletTO64(seq[pos],seq[Nseq.item[pos].neiArr_[0]],seq[Nseq.item[pos].neiArr_[1]]);
                   rat[pos]= QTij[t][t];
				 } 
				 
				 if(Nseq.item[pos].cat_pos==1 && Nseq.item[ Nseq.item[pos].neiArr_[0] ].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==2){ 
			       t=tripletTO64(seq[pos],seq[Nseq.item[pos].neiArr_[1]],seq[Nseq.item[pos].neiArr_[0]]);
                   rat[pos]= QTij[t][t];
				 }
				 
				 if(Nseq.item[pos].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==1 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==3){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[0]],seq[pos],seq[Nseq.item[pos].neiArr_[1]]);
                   rat[pos]= QT2ij[t][t];
				 }
				 
				 if(Nseq.item[pos].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==1){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[1]],seq[pos],seq[Nseq.item[pos].neiArr_[0]]);
                   rat[pos]= QT2ij[t][t];
				 }
				 
				 if(Nseq.item[pos].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==1 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==2){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[0]],seq[Nseq.item[pos].neiArr_[1]],seq[pos]);
                   rat[pos]= QT3ij[t][t];
				 }

                 if(Nseq.item[pos].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==1){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[1]],seq[Nseq.item[pos].neiArr_[0]],seq[pos]);
                   rat[pos]= QT3ij[t][t];
				 }
/*check tripletTO64: order of the positions!!)*/
				 
				 if(Nseq.item[pos].cat_pos==1 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==1 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==1){ 
			       t=tripletTO64(seq[pos],seq[Nseq.item[pos].neiArr_[0]],seq[Nseq.item[pos].neiArr_[1]]);
                   rat[pos]= QTij[t][t];
				  /* fprintf(stderr, "overlap with the same categorie: %d",Nseq.item[pos].cat_pos );*/
                 }
				 
				  if(Nseq.item[pos].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==2 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==2){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[0]],seq[pos], seq[Nseq.item[pos].neiArr_[1]]);
				 /*  fprintf(stderr, "tri: %d, %c %c %c : (k-1)pos %d, pos: %d , k+1, pos %d \n", t, seq[Nseq.item[pos].neiArr_[0]],seq[pos], seq[Nseq.item[pos].neiArr_[1]],Nseq.item[pos].neiArr_[0], pos, Nseq.item[pos].neiArr_[1]);*/
                   rat[pos]= QT2ij[t][t];
				   /*fprintf(stderr, "overlap with the same categorie: %d",Nseq.item[pos].cat_pos );*/
				  /* fprintf(stderr, "%d\n",pos);*/
                 }
				 
				  if(Nseq.item[pos].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[0] ].cat_pos==3 && Nseq.item[Nseq.item[pos].neiArr_[1]].cat_pos==3){ 
			       t=tripletTO64(seq[Nseq.item[pos].neiArr_[0]],seq[Nseq.item[pos].neiArr_[1]],seq[pos]);
                   rat[pos]= QT3ij[t][t];
				  /* fprintf(stderr, "overlap with the same categorie: %d",Nseq.item[pos].cat_pos );*/
                 }


                  if(Nseq.item[pos].cat_pos==0) fprintf(stderr,"\nulllala\n"); 

				 
			}	                     
			else{
			   t=tripletTO64(seq[pos],seq[Nseq.item[pos].neiArr_[0]],seq[Nseq.item[pos].neiArr_[1]]);
               rat[pos]= QTij[t][t];
			}
		}
    }		 
			 if(rat[pos]==10) {printf("2WRONG RATE1\n") ; 
			  fprintf(stderr, "on position %d", pos);
			  exit (1);
			 }
          /*  if(verbosewaiting==1) addrat += rat[pos];*/
			

	}  
		 /* if(verbosewaiting==1) fprintf(stderr,"summierte Raten: %f\n",addrat);*/
		 /* fprintf(stderr"\n addrat=%f\n", addrat);*/
          return(rat);
      
}



double *rateDIz(char *seq, double **QT2ij)
{
	double *rat ;
	int pos,t;

	rat=(double *)calloc(seqlen,sizeof(double));
	for(pos=0; pos<seqlen;pos++){
	  rat[pos]=10;
	 /* fprintf(stderr, "%d von %d\n",pos, seqlen);*/
	  if(pos==0){
         t=tripletTO64(seq[seqlen-1],seq[0],seq[1]);
	  }else if(pos==seqlen-1){	 
		 t=tripletTO64(seq[seqlen-2],seq[seqlen-1],seq[0]);
	  }
	  else {
	     t=tripletTO64(seq[pos-1],seq[pos],seq[pos+1]);
	  }
		
	  rat[pos]= QT2ij[t][t];
	/*  fprintf(stderr,"%d, %lf\n", pos, rat[pos]);*/
	}		 
	if(rat[pos]==10) {printf("3WRONG RATE1\n") ; 
			  fprintf(stderr, "on position %d", pos);
			  exit (1);
	}
    return(rat);
      
}
/************************************************************************/
/************************************************************************/
/************************************************************************/

double *rate_independent(char *seq, double *Qij){
           double *rat ;
          /* double addrat=0;*/
           int pos,w;

           rat=(double *)calloc(seqlen,sizeof(double));
		   w=0;
		   
		   for(pos=0; pos<seqlen;pos++) rat[pos]=-10;
		   for(pos=0; pos<seqlen;pos++){
               rat[pos]=Qij[nucleoTO123(seq[pos])*4+nucleoTO123(seq[pos])];
			   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  rat[pos]= rat[pos]*scalingfactor[pos];
		   }		 
		   if(rat[pos]==-10) {printf("4WRONG RATE2\n") ; exit (1);}
          /*  if(verbosewaiting==1) addrat += rat[pos];*/
		  /* if(verbosewaiting==1) fprintf(stderr,"summierte Raten: %f\n",addrat);*/
		  /* fprintf(stderr"\n addrat=%f\n", addrat);*/
          return(rat);
      
 }
 /****************************************************************************************************************/
 /*    CALCULATE THE SPONTANEOUS SUBSTITUTIONSRATE OVER ALL POSITIONS  q(x)                                       */
 /****************************************************************************************************************/
  /**********************************/
  /*  DEPENDENT = INDEPENDENT CASE  */
  /**********************************/
    double rate_sum(double rat[]){
         int lpos;
         double rsum;

         rsum=0.0;

         for(lpos=0; lpos<seqlen; lpos++)
            rsum += rat[lpos];
		/* rsum = rsum/seqlen;	*/
        if(verbosewaiting==1) fprintf(stderr, "q(x) = %f\n", rsum);
         return(rsum);

   }
   
   
   
   double *changerate(int currentpos, double *currentrat, char *seq, double *Qij, double **QSHij, double **QTij, double **QT2ij, double **QT3ij)
   {
          if(Nseq.item[currentpos].nNei_ == 0){
			   currentrat[currentpos]=10;
			   newcurrentrate(currentpos, currentrat, seq, Qij, QSHij, QTij, QT2ij, QT3ij);
			   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[currentpos]= currentrat[currentpos]*scalingfactor[currentpos];

		   }
		   else if(Nseq.item[currentpos].nNei_ == 1){
		       currentrat[currentpos]=10;
			   currentrat[Nseq.item[currentpos].neiArr_[0]]=10;
			   newcurrentrate(currentpos, currentrat, seq, Qij, QSHij, QTij, QT2ij, QT3ij);
			   newcurrentrate(Nseq.item[currentpos].neiArr_[0], currentrat, seq, Qij, QSHij, QTij, QT2ij, QT3ij);
			   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[currentpos]= currentrat[currentpos]*scalingfactor[currentpos];
               if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[Nseq.item[currentpos].neiArr_[0]]= currentrat[Nseq.item[currentpos].neiArr_[0]]*scalingfactor[Nseq.item[currentpos].neiArr_[0]];

		  
		   
			 }
		   else if(Nseq.item[currentpos].nNei_ == 2){
			        currentrat[currentpos]=10;
					currentrat[Nseq.item[currentpos].neiArr_[0]]=10;
					currentrat[Nseq.item[currentpos].neiArr_[1]]=10;
				    newcurrentrate(currentpos, currentrat, seq, Qij, QSHij, QTij, QT2ij, QT3ij);
			        newcurrentrate(Nseq.item[currentpos].neiArr_[0], currentrat, seq, Qij, QSHij, QTij, QT2ij, QT3ij);
				    newcurrentrate(Nseq.item[currentpos].neiArr_[1], currentrat, seq, Qij, QSHij, QTij, QT2ij, QT3ij);
					if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[currentpos]= currentrat[currentpos]*scalingfactor[currentpos];
                    if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[Nseq.item[currentpos].neiArr_[0]]= currentrat[Nseq.item[currentpos].neiArr_[0]]*scalingfactor[Nseq.item[currentpos].neiArr_[0]];
					if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[Nseq.item[currentpos].neiArr_[1]]= currentrat[Nseq.item[currentpos].neiArr_[1]]*scalingfactor[Nseq.item[currentpos].neiArr_[1]];
		   }
		   if(currentrat[currentpos]==10) {printf("WRONG 5RATE3\n") ; exit (1);}
			
           return(currentrat);
	}
	
	
	double *newcurrentrate(int changepos, double *currentrat, char *seq, double *Qij, double **QSHij, double **QTij, double **QT2ij, double **QT3ij)
    {
	       int d, t;
	
	
		   if(Nseq.item[changepos].nNei_ == 0){
               currentrat[changepos]=Qij[nucleoTO123(seq[changepos])*4+nucleoTO123(seq[changepos])];
			   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[changepos]= currentrat[changepos]*scalingfactor[changepos];
		   }
		   else if(Nseq.item[changepos].nNei_ == 1){
               d=doubletsTO15(seq[changepos],seq[Nseq.item[changepos].neiArr_[0]]);
			   currentrat[changepos]=QSHij[d][d];
			   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[changepos]= currentrat[changepos]*scalingfactor[changepos];
		   }
		   else if(Nseq.item[changepos].nNei_ == 2){
		     if(catnei==1){
				   if(Nseq.item[changepos].cat_pos==1 && Nseq.item[Nseq.item[changepos].neiArr_[0] ].cat_pos==2 &&Nseq.item[Nseq.item[changepos].neiArr_[1]].cat_pos==3){ 
			         t=tripletTO64(seq[changepos],seq[Nseq.item[changepos].neiArr_[0]],seq[Nseq.item[changepos].neiArr_[1]]);
                     currentrat[changepos]= QTij[t][t];
					 if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[changepos]= currentrat[changepos]*scalingfactor[changepos];
				    } 
				 
				   if(Nseq.item[changepos].cat_pos==1 && Nseq.item[ Nseq.item[changepos].neiArr_[0] ].cat_pos==3 && Nseq.item[Nseq.item[changepos].neiArr_[1]].cat_pos==2){ 
			         t=tripletTO64(seq[changepos],seq[Nseq.item[changepos].neiArr_[1]],seq[Nseq.item[changepos].neiArr_[0]]);
                     currentrat[changepos]= QTij[t][t];
					 if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[changepos]= currentrat[changepos]*scalingfactor[changepos];
				   }
				 
				   if(Nseq.item[changepos].cat_pos==2 && Nseq.item[Nseq.item[changepos].neiArr_[0] ].cat_pos==1 && Nseq.item[Nseq.item[changepos].neiArr_[1]].cat_pos==3){ 
			         t=tripletTO64(seq[Nseq.item[changepos].neiArr_[0]],seq[changepos],seq[Nseq.item[changepos].neiArr_[1]]);
                     currentrat[changepos]= QT2ij[t][t];
					 if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[changepos]= currentrat[changepos]*scalingfactor[changepos];
				   }
				 
				   if(Nseq.item[changepos].cat_pos==2 && Nseq.item[Nseq.item[changepos].neiArr_[0] ].cat_pos==3 && Nseq.item[Nseq.item[changepos].neiArr_[1]].cat_pos==1){ 
			         t=tripletTO64(seq[Nseq.item[changepos].neiArr_[1]],seq[changepos],seq[Nseq.item[changepos].neiArr_[0]]);
                     currentrat[changepos]= QT2ij[t][t];
					 if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[changepos]= currentrat[changepos]*scalingfactor[changepos];
				   }
				 
				   if(Nseq.item[changepos].cat_pos==3 && Nseq.item[Nseq.item[changepos].neiArr_[0] ].cat_pos==1 && Nseq.item[Nseq.item[changepos].neiArr_[1]].cat_pos==2){ 
			         t=tripletTO64(seq[Nseq.item[changepos].neiArr_[0]],seq[Nseq.item[changepos].neiArr_[1]],seq[changepos]);
                     currentrat[changepos]= QT3ij[t][t];
					 if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[changepos]= currentrat[changepos]*scalingfactor[changepos];
				   }

                   if(Nseq.item[changepos].cat_pos==3 && Nseq.item[Nseq.item[changepos].neiArr_[0] ].cat_pos==2 && Nseq.item[Nseq.item[changepos].neiArr_[1]].cat_pos==1){ 
			         t=tripletTO64(seq[Nseq.item[changepos].neiArr_[1]],seq[Nseq.item[changepos].neiArr_[0]],seq[changepos]);
                     currentrat[changepos]= QT3ij[t][t];
					 if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[changepos]= currentrat[changepos]*scalingfactor[changepos];
				   }

                   if(Nseq.item[changepos].cat_pos==0) fprintf(stderr,"\n?\n"); 

			}	                     
			else{
			   t=tripletTO64(seq[changepos],seq[Nseq.item[changepos].neiArr_[0]],seq[Nseq.item[changepos].neiArr_[1]]);
               currentrat[changepos]= QTij[t][t];
			   if(sitefactor==1 || sitefactornorm ==1 || sitefactor2==1 || sitefactornorm2 ==1)  currentrat[changepos]= currentrat[changepos]*scalingfactor[changepos];
			}
		
    		 
			/* if(currentrat[changepos]==10) {printf("WRONG RATE\n") ; exit(0);}*/
          /*  if(verbosewaiting==1) addcurrentrat += currentrat[pos];*/
			

	}  
		 /* if(verbosewaiting==1) fprintf(stderr,"summierte Raten: %f\n",addcurrentrat);*/
		 /* fprintf(stderr"\n addcurrentrat=%f\n", addcurrentrat);*/
          return(currentrat);
      
}





