/******************************************************************************
 *   neighfile.c                                                                 *
 *                                                                            *
 *   SISSI0.93 (Prerelease Version)                                           *
 *   SImulating Site-Specific Interactions along phylogenetic trees           *
 *                                                                            *
 *   Copyright (C) 2004-2005 by Tanja Gesell, Arndt von Haeseler              *
 *   sissi@cs.uni-duesseldorf.de                                              *
 *                                                                            *
 *   All parts of the source code except where indicated are distributed      *
 *	 under the GNU public licence.                                            *
 *	 See http://www.opensource.org for details.                               *
 *                                                                            *
 *   This program is distributed in the hope that it will be useful,          *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of           *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            *
 *   GNU General Public License for more details.                             *
 *                                                                            *
 *   Any feedback is very welcome!              sissi@cs.uni-duesseldorf.de   *
 *****************************************************************************/



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <ctype.h>
#include "global.h"
#include "neighfile.h" 
#include "mutate.h"
#if 0
 #include <sys/stat.h>
#endif

int seqlen;
int verboseNeighborNei; /*neighfile.c*/
Nseq_t Nseq;
double minimumvalue;
int ctcountfreq;



/*prototypes */
 void   readneighnorm(FILE *fv, char *neighborFileName);
 void   readneighcat(FILE *fv, char *neighborFileName) ;
 int    readctfile(FILE *ct_fv, char *ctFileName);
 double **readneighPvalues(FILE *neighbor_fv, char *neighborFileName);
 void   read_npos(double **p_value_table);

/***********************************************************************************/
/*      READ THE NEIGHBORHOODFILE   - ncOption - ct File (e.g. RNase P database)   */
/***********************************************************************************/
 int  readctfile(FILE *ct_fv, char *ctFileName) 
 {
   struct ctHead
   {
         int num_bases;
         int has_dG;
         int has_initial;
		 float dG_val;
         float initial_val;
		 char name[250];
		 int is_rna, is_dna;
		 int num_strands;
	};
	 
	
    int i,j;
	char   current_line[300];
	struct ctHead head;
	/* struct ctLine lines;*/
	int    dummynr, paired, dummy;
	char  *seq_ct;
	/*int countfreq=1;*/
	int numInd=0, numDoub=0;
	int d;
	double sumdipi=0.0;
	double sumpi=0.0;
	
    /* printf("file name%s", ctFileName);
	 if(ctFile){
         if ((ct_fv=fopen(ctFileName, "r"))==NULL){
           fprintf(stderr, "Error opening ctFile file: '%s'\n", ctFileName);
           exit(0); 
		  }
		  
	 }	*/
	 
	

	if(!fgets(current_line, 300, ct_fv)) return 0;
	 fprintf(stderr, "\n%s", current_line);
	
	/***********/
  
  if (sscanf(current_line, "%d dG = %G [initially %g] %[^\n]",
	     &head.num_bases, &head.dG_val, &head.initial_val, head.name) == 4)
    {
      head.has_dG = 1;
      head.has_initial = 1;
	  if(verboseNeighborCt==1) fprintf(stderr, "case1");
    }
  else if (sscanf(current_line, "%d dG = %g %[^\n]", &head.num_bases, &head.dG_val, head.name) == 3)
    {head.has_dG = 1;  if(verboseNeighborCt==1)fprintf(stderr, "case2");}
  else if (sscanf(current_line, "%d ENERGY = %g [initially %g] %[^\n]",
		  &head.num_bases, &head.dG_val, &head.initial_val, head.name) == 4)
    {
      head.has_dG = 2;
      head.has_initial = 1;
	  if(verboseNeighborCt==1) fprintf(stderr, "case3");
    }
  else if (sscanf(current_line, "%d ENERGY = %g %[^\n]", &head.num_bases, &head.dG_val, head.name) == 3)
    {head.has_dG = 2; if(verboseNeighborCt==1) fprintf(stderr, "case4");}
  else if (sscanf(current_line, "%d %[^\n]", &head.num_bases, head.name) == 2);
  else  { fprintf(stderr, "nothing\n"); return 0; }

  if (head.num_bases <= 0) return 0;


	  seqlen=head.num_bases;
	  
	  seq_ct=(char*)malloc(seqlen*sizeof(char));
	  
	  while (head.name[strlen(head.name) - 1] == ' ')
         head.name[strlen(head.name) - 1] = '\0';
		 
	 /****************************/	 
    for(i=0;i<seqlen;i++) 
	{
	  Nseq.item[i].id_pos=i;
	  Nseq.item[i].cat_pos=0;
	  Nseq.item[i].nNei_=0;
	  for(j=0;j<MAX_NEI;j++){
		     Nseq.item[i].neiArr_[j]=0;
			 Nseq.item[i].neiVal_[j]=0.00;
	  }
	 }	   
	 
	/*fprintf(stderr, "\nctFile\n");	*/ 
	for (i = 0; i < head.num_bases; i++)
     {
		if (!fgets(current_line, 100, ct_fv)) return 0;
		if(verboseNeighborNei==1) fprintf(stderr, "%s",current_line);
        if (sscanf(current_line, "%d %c %d %d %d %d", &dummynr , &seq_ct[i], &dummy, &dummy , &paired, &dummy) != 6) {
		     fprintf(stderr, "scanf misake"); return 0;}
		if(paired !=0) {
		    Nseq.item[i].nNei_=1;
		    Nseq.item[i].neiArr_[0]=paired-1;
		    Nseq.item[i].neiVal_[0]=1.00;
			numDoub++;
		}else numInd++;	

		if ( dummynr != i + 1)
	    {
	      fprintf(stderr, "ctFile: The sequence does not go up in sequentially increasing numeric order\n");
	      return 0;
	    }
    }
	
	
		 
	if(verboseNeighborNei==1){
	     fprintf(stderr, "nei.file");
         for(i=0;i<seqlen;i++){
			fprintf(stderr,"\nPos%5.d-%d| ",i, Nseq.item[i].cat_pos);
			for(j=0;j<MAX_NEI;j++){
			  if(Nseq.item[i].neiVal_[j] != 0.0)  fprintf(stderr, "%d:(%f)", Nseq.item[i].neiArr_[j],Nseq.item[i].neiVal_[j] );
		    }
		 }
		 fprintf(stderr, "\n");
	}	

	if(verboseNeighborCt==1) fprintf(stderr, "\nRootseq.ct:\n"); 
	if(verboseNeighborCt==1){
	for(i=0; i< seqlen; i++) fprintf(stderr, "%c", seq_ct[i]);}
	fprintf(stderr, "%d independent sites and %d doublet sites = %d sites\n", numInd, numDoub, numInd+numDoub); 
	
	
/* Count the frequencies */	

if(ctcountfreq==1){	
	for(i=0;i<16;i++) dipi[i]=0.00;
	for(i=0;i<4;i++)   mpi[i]=-1;
	for(i=0;i<seqlen;i++){
	   if(Nseq.item[i].nNei_==0) {
	      d=nucleoTO123(seq_ct[i]);
		  mpi[d]++;
		}
		d=0;  
	   if(Nseq.item[i].nNei_==1){
	      d= doubletsTO15(seq_ct[i] ,seq_ct[Nseq.item[i].neiArr_[0]]);
	      dipi[d]++;
	   }	  
	}
	
	  fprintf(stderr, "\n");
	  sumpi=0;
	  for(i=0;i<4;i++) sumpi += mpi[i];
	  fprintf(stderr, "ct: sum of singlefreq. %f\n", sumpi);
	  fprintf(stderr, "ct: frequencies:a:%lf\t c:%lf\t g:%lf\t u:%lf\t\n",  mpi[0], mpi[1], mpi[2], mpi[3]);
	  for(i=0;i<4;i++) mpi[i] = mpi[i]/sumpi;
	  fprintf(stderr, "ct: frequencies:a:%lf\t c:%lf\t g:%lf\t u:%lf\t\n",  mpi[0], mpi[1], mpi[2], mpi[3]);
	  sumpi=0;
	  for(i=0;i<4;i++) sumpi += mpi[i];
	  fprintf(stderr, "ct: sum of singlefreq. %f\n", sumpi);

		for(i=0;i<16;i++) {
		  if( dipi[i]==0.00) {
		      dipi[i]=minimumvalue ;
			  fprintf(stderr, "%s is not occur in the root sequence, e.g. is count with the minimum value %f\n", doublet[i], minimumvalue);	
		  }
		}
		
		
        fprintf(stderr, "\n");
		fprintf(stderr, "frequencies:aa:%lf\t ac:%lf\t ag:%lf\t au:%lf\t\n",  dipi[0], dipi[1], dipi[2], dipi[3]);
        fprintf(stderr, "           :ca:%lf\t cc:%lf\t cg:%lf\t cu:%lf\t\n",  dipi[4], dipi[5], dipi[6], dipi[7]);
        fprintf(stderr, "           :ga:%lf\t gc:%lf\t gg:%lf\t gu:%lf\t\n",  dipi[8], dipi[9],dipi[10],dipi[11]);
        fprintf(stderr, "           :ua:%lf\t uc:%lf\t ug:%lf\t uu:%lf\t\n",  dipi[12],dipi[13],dipi[14],dipi[15]);
    
		 sumdipi=0.00;
	 	 for(i=0;i<16;i++) sumdipi+= dipi[i];	
		 fprintf(stderr, "ct: sum of doubletfreq. %f\n", sumdipi);
	 
	 if(sumdipi!=1){
	    for(i=0;i<16;i++) dipi[i] = dipi[i]/sumdipi;
	 
	 
	  
			fprintf(stderr, "frequencies:aa:%lf\t ac:%lf\t ag:%lf\t au:%lf\t\n",  dipi[0], dipi[1], dipi[2], dipi[3]);
			fprintf(stderr, "           :ca:%lf\t cc:%lf\t cg:%lf\t cu:%lf\t\n",  dipi[4], dipi[5], dipi[6], dipi[7]);
            fprintf(stderr, "           :ga:%lf\t gc:%lf\t gg:%lf\t gu:%lf\t\n",  dipi[8], dipi[9],dipi[10],dipi[11]);
            fprintf(stderr, "           :ua:%lf\t uc:%lf\t ug:%lf\t uu:%lf\t\n", dipi[12],dipi[13],dipi[14],dipi[15]);
	         sumdipi=0.00;
	 	     for(i=0;i<16;i++) sumdipi+= dipi[i];	
	         fprintf(stderr, "ct: sum of doubletfreq. %f\n", sumdipi);
	}	 
}		   

free(seq_ct);
fclose(ct_fv);
return 1;

}



  /***********************************************************************************/
  /*      READ THE NEIGHBORHOODFILE   - ncOption - with different categories         */
  /***********************************************************************************/
   void   readneighcat(FILE *neighbor_fv, char *neighborFileName){
	int i,j,lpos,len,m,cm;
	char *datei,  *cc;
	char *tokendatei;
	char trennzeichen[]=")";
	double count;
	int c=0;
	char *s=NULL, *p_str=NULL ;
    FILE *tmp;
		
	for(i=0;i<seqlen;i++) 
	{
	       Nseq.item[i].id_pos=i;
		   Nseq.item[i].cat_pos=0;
           Nseq.item[i].nNei_=0;
		   for(j=0;j<MAX_NEI;j++){
		     Nseq.item[i].neiArr_[j]=0;
			 Nseq.item[i].neiVal_[j]=0.00;
		   }
	}	   	 

	
		tmp = fopen(neighborFileName, "r");
		for (len=0; EOF != fgetc(tmp); len++) /* empty loop - just counting :-) */;	
		fclose(tmp);

		datei=(char*)malloc(len*sizeof(char));         /*speicher die groesse des files freimachen (aus dem stat struct)*/
    	fread(datei,sizeof(char),len,neighbor_fv);       /*ganzer block eingelesen in den string datei*/
		
		
	    tokendatei=strtok(datei,trennzeichen);                     /*strtok: split file in token*/
        lpos=0;
	    for(i=0; i<len ; i++){
         if(datei[i]=='\n') lpos++; 
		 if(datei[i]=='-'){
            c=i+1;
		 }	
         if(datei[i]==':'){
           j=i-1;
           count=1;
            while(datei[j] != ' '){
               count++;
               j--;
            }
            p_str=(char*)calloc((strlen(&tokendatei[i+2])+1),sizeof(char));
            strcpy(p_str, &tokendatei[i+2]);
            s=(char*)calloc((count+1),sizeof(char));
			cc=(char*)calloc((1),sizeof(char));
           /* bzero(s, (count+1)*sizeof(char));
			bzero(cc, (1)*sizeof(char));*/
           /*strncpy(s, &datei[j], sizeof(count+1)); */ /*IMPROVEMENTS*/
		   
			strncpy(s, &datei[j], (count+1)*sizeof(char));
			
			/*strncpy(cc, &datei[c], sizeof(1));*/
			*cc=datei[c];
		    cm=getintNumber(cc);
            m=getintNumber(s);
			if(Nseq.item[lpos].id_pos != lpos) {
			    printf("Check: Sequencelength (-l) and the neighbourhood system must have the same length!"); 
			    exit (0); 
			}
			Nseq.item[lpos].id_pos=lpos;
			Nseq.item[lpos].cat_pos=cm;
		    Nseq.item[lpos].neiArr_[Nseq.item[lpos].nNei_]=m;
		    Nseq.item[lpos].neiVal_[Nseq.item[lpos].nNei_]=getdoubleNumber(p_str);
		    ++Nseq.item[lpos].nNei_;
         }
       }

	   if(lpos != seqlen) fprintf(stderr, "\n\n  WARNING: sequencelength and the positions of the neighborhoodsystem are not the same!!!\n");
       free(p_str);
	   free(s);
	   free(datei);
       fclose(neighbor_fv);
	
	   
	if(verboseNeighborNei==1){
	     fprintf(stderr, "\t\t\t***neighborhood system***\n");
         for(i=0;i<seqlen;i++){
			fprintf(stderr,"\nPos%5.d-%d| ",i, Nseq.item[i].cat_pos);
			for(j=0;j<MAX_NEI;j++){
			  if(Nseq.item[i].neiVal_[j] != 0.0)  fprintf(stderr, "%d:(%f)", Nseq.item[i].neiArr_[j],Nseq.item[i].neiVal_[j] );
		    }
		 }
		 fprintf(stderr, "\n");
	 }	
 }
  /*******************************************************/
  /*      READ THE NEIGHBORHOODFILE   - nnOption         */
  /*******************************************************/
   void   readneighnorm(FILE *neighbor_fv , char *neighborFileName){
	  int i,j,lpos,len,m;
	  char *datei, *s=NULL, *p_str=NULL;
	  char *tokendatei;
	  char trennzeichen[]=")";
	  double count;
#if 0
      struct stat stbuf;
	 /* double **p_value_table;*/
#endif
	FILE *tmp;
 
	  
	  for(i=0;i<seqlen;i++){
		Nseq.item[i].id_pos=i;
		Nseq.item[i].nNei_=0;
		for(j=0;j<MAX_NEI;j++){
		     Nseq.item[i].neiArr_[j]=0;
			 Nseq.item[i].neiVal_[j]=0.00;
		}
	  }	   	 
	  
	  
		tmp = fopen(neighborFileName, "r");
		for (len=0; EOF != fgetc(tmp); len++) /* empty loop - just counting :-) */;	
		fclose(tmp);

		datei=(char*)malloc(len*sizeof(char));         /*speicher die groesse des files freimachen (aus dem stat struct)*/
    	fread(datei,sizeof(char),len,neighbor_fv);       /*ganzer block eingelesen in den string datei*/

	  
	  
	  
#if 0
	stat (neighborFileName,&stbuf);                             /*stat: count the attributes*/ 
	datei=(char*)malloc((stbuf.st_size)*sizeof(char));         /*speicher die groesse des files freimachen (aus dem stat struct)*/
    	fread(datei,sizeof(char),stbuf.st_size, neighbor_fv);       /*ganzer block eingelesen in den string datei*/
	    len=stbuf.st_size;                                         /*laenge des eingelesenen blocks*/
#endif		
		
	    tokendatei=strtok(datei,trennzeichen);                     /*strtok: splitten die datei in token, ueberall wo )*/
        lpos=0;
	    for(i=0; i<len ; i++){
         if(datei[i]=='\n') lpos++;
         if(datei[i]==':'){
           j=i-1;
           count=1;
            while(datei[j] != ' '){
               count++;
               j--;
            }
            p_str=(char*)malloc((strlen(&tokendatei[i+2])+1)*sizeof(char));
            strcpy(p_str, &tokendatei[i+2]);
            s=(char*)calloc((count+1),sizeof(char));
            /*bzero(s, (count+1)*sizeof(char));*/
           /* strncpy(s, &datei[j], sizeof(count+1));  */   /*IMPROVEMENTS*/
		   strncpy(s, &datei[j], (count+1)*sizeof(char));
            m=getintNumber(s);
			if(Nseq.item[lpos].id_pos != lpos) {
			   printf("Check: Sequencelength (-l) and the neighbourhood system must have the same length!");
			   exit (0); 
			}
			Nseq.item[lpos].id_pos=lpos;
		    Nseq.item[lpos].neiArr_[Nseq.item[lpos].nNei_]=m;
		    Nseq.item[lpos].neiVal_[Nseq.item[lpos].nNei_]=getdoubleNumber(p_str);
		    ++Nseq.item[lpos].nNei_;
         }
       }

	   if(lpos != seqlen) fprintf(stderr, "\n\n  WARNING: sequencelength and the positions of the neighborhoodsystem are not the same!!!\n");

	   free(p_str);
	   free(s);
	   free(datei);
       fclose(neighbor_fv);
	
	   
	if(verboseNeighborNei==1){
	     fprintf(stderr, "\t\t\t***neighborhood system***\n");
         for(i=0;i<seqlen;i++){
			fprintf(stderr,"\nPos%5.d| ",i);
			for(j=0;j<MAX_NEI;j++){
			  if(Nseq.item[i].neiVal_[j] != 0.0)  fprintf(stderr, "%d:(%f)", Nseq.item[i].neiArr_[j],Nseq.item[i].neiVal_[j] );
		    }
		 }
		 fprintf(stderr, "\n");
	 }	
	 
	/*return 0;*/
	}
	
	
 /**************************************************************/
  /*      READ THE NEIGHBORHOODFILE                            */
  /*************************************************************/
  double   **readneighPvalues(FILE *neighbor_fv, char *neighborFileName){
	int i,j,lpos,len,m;
	char *datei, *s=NULL, *p_str=NULL;
	char *tokendatei;
	char trennzeichen[]=")";
	double count;
	double **p_value_table;
#if 0
    struct stat stbuf;
#endif	
    FILE *tmp;
    
	p_value_table=(double**)malloc(seqlen*sizeof(double*));          /*Speicher fuer die Tabelle alloziieren*/
	for(i=0;i<seqlen;i++){
          p_value_table[i]=(double*)malloc(seqlen*sizeof(double));
    }
                                                                   
	for(i=0; i< seqlen; i++){
		for(j=0; j< seqlen ; j++){
			p_value_table[i][j]=0.00;                           /*corelation-values*/
		}
	}

	/*if(neighborFile){
         if ((neighbor_fv=fopen(neighborFileName, "r"))==NULL){
           fprintf(stderr, "Error opening neighborhood file: '%s'\n", neighborFileName);
           exit(0); 
	}*/
        
		
		tmp = fopen(neighborFileName, "r");
		for (len=0; EOF != fgetc(tmp); len++) /* empty loop - just counting :-) */;	
		fclose(tmp);

		datei=(char*)malloc(len*sizeof(char));         /*speicher die groesse des files freimachen (aus dem stat struct)*/
    	fread(datei,sizeof(char),len,neighbor_fv);       /*ganzer block eingelesen in den string datei*/

		
#if 0		
		stat (neighborFileName,&stbuf);     /*stat: count the attributes*/
		datei=(char*)malloc((stbuf.st_size)*sizeof(char));         /*speicher die groesse des files freimachen (aus dem stat struct)*/
    	fread(datei,sizeof(char),stbuf.st_size,neighbor_fv);       /*ganzer block eingelesen in den string datei*/
	    len=stbuf.st_size;                                         /*laenge des eingelesenen blocks*/
#endif		
		
		
	    tokendatei=strtok(datei,trennzeichen);                     /*strtok: splitten die datei in token, ueberall wo )*/
        lpos=0;
	    for(i=0; i<len ; i++){
         if(datei[i]=='\n') lpos++;
         if(datei[i]==':'){
           j=i-1;
           count=1;
            while(datei[j] != ' '){
               count++;
               j--;
            }
            p_str=(char*)malloc((strlen(&tokendatei[i+2])+1)*sizeof(char));
            strcpy(p_str, &tokendatei[i+2]);
            s=(char*)calloc((count+1),sizeof(char));
           /* bzero(s, (count+1)*sizeof(char));*/
          /*  strncpy(s, &datei[j], sizeof(count+1)); */  /*IMPROVEMENTS*/
			strncpy(s, &datei[j], (count+1)*sizeof(char));
            m=getintNumber(s);
            p_value_table[lpos][m]=getdoubleNumber(p_str);
         }
       }

	   if(lpos != seqlen) fprintf(stderr, "\n\n  WARNING: sequencelength and the positions of the neighborhoodsystem are not the same!!!\n");

	   free(p_str);
	   free(s);
       fclose(neighbor_fv);
	   free(datei);
	
	   
	if(verboseNeighborNei==1){
	     fprintf(stderr, "\t\t\t***neighborhood system***\n");
         for(i=0;i<seqlen;i++){
			fprintf(stderr,"\nPos%5.d| ",i);
			for(j=0;j<seqlen;j++) {
				if(p_value_table[i][j] != 0.00) fprintf(stderr, "%d:(%f)",j,p_value_table[i][j]);
		    }
		 }
		 fprintf(stderr, "\n");
	}	   
	return(p_value_table);
	}
  /*******************************************************/

    void read_npos(double **p_value_table){
        int i,j, dummy,m;
        double c;

        for(i=0;i<seqlen;i++) {
            dummy=0;
            m=0;
            c=0.00;
            Nseq.item[i].id_pos = i ;
            for(j=0;j<seqlen;j++){
                 if(p_value_table[i][j] != 0.00){
                        dummy++;
                        c += p_value_table[i][j];
                        Nseq.item[i].neiArr_[m]=j;                     
                        m++ ;
                 }
            }
            Nseq.item[i].nNei_ =dummy;    
			
		}
		
		free(p_value_table);

     }
   


